@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -192,6 +192,8 @@ extern "C" {
 #define BUF_VIDEO_VMNC		0x02660000
 #define BUF_VIDEO_SNOW		0x02670000
 #define BUF_VIDEO_VP8		0x02680000
+#define BUF_VIDEO_VP9		0x02690000
+#define BUF_VIDEO_HEVC		0x026A0000
 /*@}*/
 
 /**
@@ -269,6 +271,9 @@ extern "C" {
 #define BUF_AUDIO_AMR_WB	0x03400000
 #define BUF_AUDIO_EAC3		0x03410000
 #define BUF_AUDIO_AAC_LATM	0x03420000
+#define BUF_AUDIO_ADPCM_G726	0x03430000
+#define BUF_AUDIO_OPUS		0x03440000
+#define BUF_AUDIO_TRUEHD        0x03450000
 /*@}*/
 
 /**
@@ -412,6 +417,11 @@ struct buf_element_s {
   * Valid until revoked or next stream.                               */
 #define BUF_FLAG_COLOR_MATRIX 0x4000
 
+/** Optimization: try to merge this buf with previous one still in fifo.
+  * This is for demuxers that dont know a frame's size before sending it
+  * (mpeg-ts). Decoders will never see this. */
+#define BUF_FLAG_MERGE 0x8000
+
 /**
  * \defgroup buffer_special Special buffer types:
  * Sometimes there is a need to relay special information from a demuxer
@@ -625,7 +635,7 @@ struct fifo_buffer_s
   /*
    * private variables for buffer pool management
    */
-  buf_element_t   *buffer_pool_top;    /* a stack actually */
+  buf_element_t   *buffer_pool_top;    /* a heap actually */
   pthread_mutex_t  buffer_pool_mutex;
   pthread_cond_t   buffer_pool_cond_not_empty;
   int              buffer_pool_num_free;
@@ -638,6 +648,16 @@ struct fifo_buffer_s
   void            *alloc_cb_data[BUF_MAX_CALLBACKS];
   void            *put_cb_data[BUF_MAX_CALLBACKS];
   void            *get_cb_data[BUF_MAX_CALLBACKS];
+
+  /* get a buffer large enough for given byte size.
+   * result may still be smaller, do check buf->max_size.
+   */
+  buf_element_t *(*buffer_pool_size_alloc) (fifo_buffer_t *self, size_t size);
+
+  /* private */
+  int              buffer_pool_num_waiters;
+  int              buffer_pool_large_wait;
+  int              fifo_num_waiters;
 } ;
 
 /**
@@ -646,7 +666,7 @@ struct fifo_buffer_s
  * @param buf_size Size of each buffer.
  * @internal Only used by video and audio decoder loops.
  */
-fifo_buffer_t *_x_fifo_buffer_new (int num_buffers, uint32_t buf_size) XINE_MALLOC;
+fifo_buffer_t *_x_fifo_buffer_new (int num_buffers, uint32_t buf_size) XINE_PROTECTED;
 
 /**
  * @brief Allocate and initialise new dummy FIFO buffers.
@@ -654,7 +674,7 @@ fifo_buffer_t *_x_fifo_buffer_new (int num_buffers, uint32_t buf_size) XINE_MALL
  * @param buf_size Size of each buffer.
  * @internal Only used by video and audio decoder loops.
  */
-fifo_buffer_t *_x_dummy_fifo_buffer_new (int num_buffers, uint32_t buf_size) XINE_MALLOC;
+fifo_buffer_t *_x_dummy_fifo_buffer_new (int num_buffers, uint32_t buf_size);
 
 
 /**
