@@ -22,6 +22,8 @@
 
 #include <pthread.h>
 
+#include <xine/attributes.h>
+
 typedef struct {
   pthread_mutex_t   lock;
   void*             object;               /* referenced object */
@@ -31,7 +33,7 @@ typedef struct {
 
 typedef void (*refcounter_destructor)(void*);
 
-refcounter_t* _x_new_refcounter(void *object, refcounter_destructor destructor) XINE_MALLOC XINE_PROTECTED;
+refcounter_t* _x_new_refcounter(void *object, refcounter_destructor destructor) XINE_PROTECTED;
 
 int _x_refcounter_inc(refcounter_t *refcounter) XINE_PROTECTED;
 
