@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -106,15 +106,18 @@ struct xine_s {
 
   xine_log_cb_t              log_cb;
   void                      *log_cb_user_data;
+
+  int                        flags;
+
+  /* set when pauseing with port ticket granted, for XINE_PARAM_VO_SINGLE_STEP. */
+  int                        live_pause;
+  pthread_mutex_t            pause_mutex;
+  /* special values for set_speed_internal () */
+# define XINE_LIVE_PAUSE_ON 0x7ffffffd
+# define XINE_LIVE_PAUSE_OFF 0x7ffffffc
 #endif
 };
 
-/* FIXME-ABI Some global flag bits */
-/* See xine_set_flags() */
-#ifdef XINE_ENGINE_INTERNAL
-extern int _x_flags XINE_PROTECTED;
-#endif
-
 /*
  * xine thread tickets
  */
@@ -261,6 +264,11 @@ struct xine_stream_s {
   uint32_t                   ignore_speed_change:1;  /*< speed changes during stop can be disastrous */
   uint32_t                   video_thread_created:1;
   uint32_t                   audio_thread_created:1;
+  /* 3: wait for first frame to decode (stream start).
+   * 2: wait for first frame to display (stream seek).
+   * 1: after 2, first frame is decoded but not yet displayed.
+   * 0: waiting done.
+   */
   uint32_t                   first_frame_flag:2;
   uint32_t                   demux_action_pending:1;
   uint32_t                   demux_thread_created:1;
@@ -272,6 +280,7 @@ struct xine_stream_s {
   uint32_t                   early_finish_event:1;   /*< do not wait fifos get empty before sending event */
   uint32_t                   gapless_switch:1;       /*< next stream switch will be gapless */
   uint32_t                   keep_ao_driver_open:1;
+  uint32_t                   finished_naturally:1;
 
   input_class_t             *eject_class;
   demux_plugin_t            *demux_plugin;
@@ -360,6 +369,10 @@ struct xine_stream_s {
   broadcaster_t             *broadcaster;
 
   refcounter_t              *refcounter;
+
+  xine_keyframes_entry_t        *index_array;
+  int                        index_size, index_used, index_lastadd;
+  pthread_mutex_t            index_mutex;
 #endif
 };
 
@@ -418,6 +431,23 @@ void _x_extra_info_merge( extra_info_t *dst, extra_info_t *src ) XINE_PROTECTED;
 void _x_get_current_info (xine_stream_t *stream, extra_info_t *extra_info, int size) XINE_PROTECTED;
 
 
+/** @brief Register a list of stream keyframes.
+    @param stream The stream that index is for.
+    @param list   The array of entries to add.
+    @param size   The count of entries.
+    @return 0 (OK), 1 (Fail).
+*/
+int _x_keyframes_set (xine_stream_t *stream, xine_keyframes_entry_t *list, int size) XINE_PROTECTED;
+
+/** @brief Register a stream keyframe to seek index.
+    @note  This will try not to duplicate already registered frames.
+    @param stream The stream that index is for.
+    @param pos    The frame time AND normpos.
+    @return  The index *g* into the index where that frame has been added, or -1.
+*/
+int _x_keyframes_add (xine_stream_t *stream, xine_keyframes_entry_t *pos) XINE_PROTECTED;
+
+
 /* demuxer helper functions from demux.c */
 
 /*
@@ -468,6 +498,11 @@ void _x_demux_send_mrl_reference (xine_stream_t *stream, int alternative,
  */
 void _x_mrl_unescape(char *mrl) XINE_PROTECTED;
 
+/*
+ * Return a copy of mrl without authentication credentials
+ */
+char *_x_mrl_remove_auth(const char *mrl) XINE_PROTECTED;
+
 /*
  * plugin_loader functions
  *
@@ -490,8 +525,8 @@ int              _x_decoder_available  (xine_t *xine, uint32_t buftype) XINE_PRO
  * load a specific video output plugin
  */
 
-vo_driver_t *_x_load_video_output_plugin(xine_t *this,
-					 char *id, int visual_type, void *visual) XINE_PROTECTED;
+vo_driver_t *_x_load_video_output_plugin(xine_t *this_gen,
+                                         const char *id, int visual_type, void *visual) XINE_PROTECTED;
 
 /*
  * audio output plugin dynamic loading stuff
