@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2016 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -30,7 +30,6 @@ extern "C" {
 #include <pthread.h>
 
 #ifdef WIN32
-#  include <winsock.h>
 #else
 #  include <sys/time.h>
 #endif
@@ -56,6 +55,55 @@ extern "C" {
 #  define extern DL_IMPORT extern
 #endif
 
+
+/* Amiga style doubly linked lists, taken from TJtools. */
+/* Why does this work with aliasing? */
+
+#define DLIST_IS_EMPTY(l) ((void *)((l)->head) == (void *)(&(l)->null))
+
+#define DLIST_REMOVE(n) { \
+  dnode_t *dl_rm_this = (dnode_t *)(n); \
+  dnode_t *dl_rm_prev = dl_rm_this->prev; \
+  dnode_t *dl_rm_next = dl_rm_this->next; \
+  dl_rm_next->prev = dl_rm_prev; \
+  dl_rm_prev->next = dl_rm_next; \
+}
+
+#define DLIST_ADD_HEAD(n,l) { \
+  dlist_t *dl_ah_list = (dlist_t *)(l); \
+  dnode_t *dl_ah_node = (dnode_t *)(n); \
+  dnode_t *dl_ah_head = dl_ah_list->head; \
+  dl_ah_node->next = dl_ah_head; \
+  dl_ah_node->prev = (void *)dl_ah_list; \
+  dl_ah_list->head = dl_ah_node; \
+  dl_ah_head->prev = dl_ah_node; \
+}
+
+#define DLIST_ADD_TAIL(n,l) { \
+  dlist_t *dl_at_list = (dlist_t *)(l); \
+  dnode_t *dl_at_node = (dnode_t *)(n); \
+  dnode_t *dl_at_tail = dl_at_list->tail; \
+  dl_at_node->next = (void *)(&dl_at_list->null); \
+  dl_at_node->prev = dl_at_tail; \
+  dl_at_tail->next = dl_at_node; \
+  dl_at_list->tail = dl_at_node; \
+}
+
+#define DLIST_INIT(l) { \
+  dlist_t *dl_in_list = (dlist_t *)(l); \
+  dl_in_list->head = (void *)(&dl_in_list->null); \
+  dl_in_list->null = NULL; \
+  dl_in_list->tail = (void *)(&dl_in_list->head); }
+
+typedef struct dnode_st {
+  struct dnode_st *next, *prev;
+} dnode_t;
+
+typedef struct {
+  dnode_t *head, *null, *tail;
+} dlist_t;
+
+
   /*
    * debugable mutexes
    */
@@ -155,8 +203,8 @@ static inline void _x_freep(void *ptr) {
 /*
  * Copy blocks of memory.
  */
-void *xine_memdup (const void *src, size_t length) XINE_MALLOC XINE_PROTECTED;
-void *xine_memdup0 (const void *src, size_t length) XINE_MALLOC XINE_PROTECTED;
+void *xine_memdup (const void *src, size_t length) XINE_PROTECTED;
+void *xine_memdup0 (const void *src, size_t length) XINE_PROTECTED;
 
 /**
  * Get/resize/free aligned memory.
@@ -306,12 +354,24 @@ extern void (*yuy2_to_yv12)
    unsigned char *v_dst, int v_dst_pitch,
    int width, int height) XINE_PROTECTED;
 
-#define SCALEFACTOR 65536
+/* convert full range rgb to mpeg range yuv */
+#define SCALESHIFT 16
+#define SCALEFACTOR (1<<SCALESHIFT)
 #define CENTERSAMPLE 128
 
-/* These conversions are normalised for the MPEG Y'CbCr colourspace.
- * (Yes, we know that we call it YUV elsewhere.)
- */
+/* new fast and more accurate macros. Simply recompile to use them */
+#define COMPUTE_Y(r, g, b) \
+  (unsigned char) \
+  ((y_r_table[r] + y_g_table[g] + y_b_table[b]) >> SCALESHIFT)
+#define COMPUTE_U(r, g, b) \
+  (unsigned char) \
+  ((u_r_table[r] + u_g_table[g] + uv_br_table[b]) >> SCALESHIFT)
+#define COMPUTE_V(r, g, b) \
+  (unsigned char) \
+  ((uv_br_table[r] + v_g_table[g] + v_b_table[b]) >> SCALESHIFT)
+
+/* Binaries using these old ones keep working,
+   and get the full vs mpeg range bug fixed transparently as well.
 #define COMPUTE_Y(r, g, b) \
   (unsigned char) \
   ((y_r_table[r] + y_g_table[g] + y_b_table[b]) / SCALEFACTOR)
@@ -321,6 +381,7 @@ extern void (*yuy2_to_yv12)
 #define COMPUTE_V(r, g, b) \
   (unsigned char) \
   ((v_r_table[r] + v_g_table[g] + v_b_table[b]) / SCALEFACTOR + CENTERSAMPLE)
+*/
 
 #define UNPACK_BGR15(packed_pixel, r, g, b) \
   b = (packed_pixel & 0x7C00) >> 7; \
@@ -346,6 +407,8 @@ extern int y_r_table[256] XINE_PROTECTED;
 extern int y_g_table[256] XINE_PROTECTED;
 extern int y_b_table[256] XINE_PROTECTED;
 
+extern int uv_br_table[256] XINE_PROTECTED;
+
 extern int u_r_table[256] XINE_PROTECTED;
 extern int u_g_table[256] XINE_PROTECTED;
 extern int u_b_table[256] XINE_PROTECTED;
@@ -457,7 +520,12 @@ void xine_hexdump (const void *buf, int length) XINE_PROTECTED;
   #endif /* __GNUC__ */
 #else /* LOG */
   #ifdef __GNUC__
-    #define lprintf(fmt, args...)     do {} while(0)
+    #ifdef DEBUG
+      #define lprintf(fmt, args...)                                           \
+        do { if (0) { printf(fmt, ##args); } } while(0)  /* silence unused warnings, check format, ... */
+    #else
+      #define lprintf(fmt, args...)     do {} while(0)
+    #endif
   #else
   #ifdef _MSC_VER
 void __inline lprintf(const char * fmt, ...) {}
