@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2012 the xine-project
+ * Copyright (C) 2008-2017 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -115,17 +115,18 @@ to the extent permitted by law.\n",
       (major == 1 && minor == 2 && sub > 0) ||
       (major == 1 && minor > 2) ||
       (major > 1))
-    xine_set_flags (xine, XINE_FLAG_NO_WRITE_CACHE);
+    if (xine_set_flags)
+      xine_set_flags (xine, XINE_FLAG_NO_WRITE_CACHE);
 
   xine_init (xine);
 
-  char *text = NULL;
+  char *text = NULL, *freeme = NULL;
   char *sep, *sep2;
   switch (which)
   {
   case 'a':
   case 'm':
-    text = xine_get_mime_types (xine);
+    freeme = text = xine_get_mime_types (xine);
     if (!text || !*text)
       goto read_fail;
     sep = sep2 = text - 1;
@@ -142,7 +143,7 @@ to the extent permitted by law.\n",
     break;
 
   case 'e':
-    text = xine_get_file_extensions (xine);
+    freeme = text = xine_get_file_extensions (xine);
     if (!text || !*text)
       goto read_fail;
     sep = text - 1;
@@ -157,6 +158,8 @@ to the extent permitted by law.\n",
     break;
   }
 
+  xine_exit(xine);
+  free(freeme);
   return 0;
 
  read_fail:
