@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -302,7 +302,7 @@ static void *init_plugin (xine_t *xine, void *data) {
 
   fooaudio_class_t *this ;
 
-  this = (fooaudio_class_t *) xine_malloc (sizeof (fooaudio_class_t));
+  this = (fooaudio_class_t *) calloc (1, sizeof (fooaudio_class_t));
 
   this->decoder_class.open_plugin     = open_plugin;
   this->decoder_class.identifier      = "fooaudio";
