@@ -77,7 +77,7 @@ typedef struct {
  * @param nbits new position in bits from the beginning.
  */
 static void bebf_UNUSED bebf_seek (bebf_t *bebf, uint32_t nbits) {
-  const uint32_t *p = (const uint32_t *)(((unsigned long int)bebf->readstart + (nbits >> 3)) & ~(unsigned long int)3);
+  const uint32_t *p = (const uint32_t *)(((uintptr_t)bebf->readstart + (nbits >> 3)) & ~(uintptr_t)3);
   uint32_t n = (int32_t)nbits - (((const uint8_t *)p - bebf->readstart) << 3);
   bebf->readbits = n;
   if (p < bebf->readstop) {
@@ -112,7 +112,7 @@ static void bebf_UNUSED bebf_align (bebf_t *bebf) {
  */
 static void bebf_UNUSED bebf_set_read (bebf_t *bebf, const uint8_t *rp, uint32_t nbytes) {
   bebf->readstart = rp;
-  bebf->readstop = (const uint32_t *)(((unsigned long int)rp + nbytes + 3) & ~(unsigned long int)3);
+  bebf->readstop = (const uint32_t *)(((uintptr_t)rp + nbytes + 3) & ~(uintptr_t)3);
   bebf_seek (bebf, 0);
 }
 
@@ -192,7 +192,7 @@ static void bebf_UNUSED bebf_skip (bebf_t *bebf, uint32_t nbits) {
  * @return     stream position in bits.
  */
 static uint32_t bebf_UNUSED bebf_tell (bebf_t *bebf) {
-  return (((uint8_t *)bebf->readptr - bebf->readstart) << 3) + bebf->readbits - 64;
+  return (((const uint8_t *)bebf->readptr - bebf->readstart) << 3) + bebf->readbits - 64;
 }
 
 /**
@@ -806,7 +806,8 @@ typedef enum {
   BEBF_LATM_NEED_MORE_DATA,
   BEBF_LATM_IS_RAW,
   BEBF_LATM_IS_ADTS,
-  BEBF_LATM_IS_LATM
+  BEBF_LATM_IS_LATM,
+  BEBF_LATM_IS_UNKNOWN
 } bebf_latm_parser_status_t;
 
 /**
@@ -846,7 +847,7 @@ static bebf_latm_parser_status_t bebf_UNUSED bebf_latm_test (const uint8_t *in,
       return BEBF_LATM_IS_LATM;
     } while (0);
   }
-  return nbytes < BEBF_TEST_MAX ? BEBF_LATM_NEED_MORE_DATA : BEBF_LATM_IS_RAW;
+  return nbytes < BEBF_TEST_MAX ? BEBF_LATM_NEED_MORE_DATA : BEBF_LATM_IS_UNKNOWN;
 }
 
 /**
