@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2016 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -33,9 +33,6 @@
 #include <unistd.h>
 #include <string.h>
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <assert.h>
 
 #define LOG_MODULE "a52_decoder"
 #define LOG_VERBOSE
@@ -681,15 +678,13 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
   lprintf ("open_plugin called\n");
 
   this = calloc(1, sizeof (a52dec_decoder_t));
+  if (!this)
+    return NULL;
 
-  this->audio_decoder.decode_data         = a52dec_decode_data;
-  this->audio_decoder.reset               = a52dec_reset;
-  this->audio_decoder.discontinuity       = a52dec_discontinuity;
-  this->audio_decoder.dispose             = a52dec_dispose;
-  this->stream                            = stream;
-  this->class                             = (a52dec_class_t *) class_gen;
-
-  this->audio_caps        = stream->audio_out->get_capabilities(stream->audio_out);
+  /* Do these first, when compiler still knows "this" is all zeroed.
+   * Let it optimize away this on most systems where clear mem
+   * interpretes as 0, 0f or NULL safely.
+   */
   this->syncword          = 0;
   this->sync_state        = 0;
   this->output_open       = 0;
@@ -697,6 +692,16 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
   this->pts_list[0]       = 0;
   this->pts_list_position = 0;
 
+  this->audio_decoder.decode_data   = a52dec_decode_data;
+  this->audio_decoder.reset         = a52dec_reset;
+  this->audio_decoder.discontinuity = a52dec_discontinuity;
+  this->audio_decoder.dispose       = a52dec_dispose;
+
+  this->stream = stream;
+  this->class  = (a52dec_class_t *)class_gen;
+
+  this->audio_caps = stream->audio_out->get_capabilities (stream->audio_out);
+
   if( !this->a52_state ) {
     this->a52_state =
 #ifdef HAVE_A52DEC_A52_H /* External liba52 */
@@ -808,6 +813,19 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
   return &this->audio_decoder;
 }
 
+static void dispose_class (audio_decoder_class_t *this_gen) {
+
+  a52dec_class_t  *this = (a52dec_class_t *)this_gen;
+  config_values_t *config = this->config;
+
+  config->unregister_callback (config, "audio.a52.level");
+  config->unregister_callback (config, "audio.a52.dynamic_range");
+  config->unregister_callback (config, "audio.a52.surround_downmix");
+  config->unregister_callback (config, "audio.a52.lfe_level");
+
+  free (this);
+}
+
 static void *init_plugin (xine_t *xine, void *data) {
 
   a52dec_class_t *this;
@@ -818,7 +836,7 @@ static void *init_plugin (xine_t *xine, void *data) {
   this->decoder_class.open_plugin     = open_plugin;
   this->decoder_class.identifier      = "a/52dec";
   this->decoder_class.description     = N_("liba52 based a52 audio decoder plugin");
-  this->decoder_class.dispose         = default_audio_decoder_class_dispose;
+  this->decoder_class.dispose         = dispose_class;
 
   cfg = this->config = xine->config;
 
