@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -41,9 +41,6 @@
 #include <unistd.h>
 #include <string.h>
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <assert.h>
 
 #define LOG_MODULE "libdts"
 #define LOG_VERBOSE
@@ -160,9 +157,11 @@ static void dts_decode_frame (dts_decoder_t *this, const int64_t pts) {
         return;
 
       data_out=(uint8_t *) audio_buffer->mem;
-      if (this->ac5_length > 8191) {
+      if (this->ac5_length > MAX_AC5_FRAME) {
+        /* XXX is this even possible ? ac5_length is checked in dts_decode_data() */
         xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "libdts: ac5_length too long\n");
         this->ac5_pcm_length = 0;
+        return;
       }
 
       switch (this->ac5_pcm_length) {
@@ -435,10 +434,11 @@ static void dts_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
     case 2:  /* Filling frame_buffer with sync_info bytes */
 	  *this->frame_ptr++ = *current++;
 	  this->frame_todo--;
-	  if (this->frame_todo < 1) {
-	    this->sync_state = 3;
-          } else break;
-
+          if (this->frame_todo > 0) {
+            break;
+          }
+          this->sync_state = 3;
+          /* fall through */
     case 3:  /* Ready for decode */
 #if 0
           dtsdec_decode_frame (this, this->pts_list[0]);
