@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -105,6 +105,7 @@ typedef struct alsa_driver_s {
 
   struct {
     pthread_t          thread;
+    int                thread_created;
     pthread_mutex_t    mutex;
     char              *name;
     snd_mixer_t       *handle;
@@ -133,6 +134,7 @@ static int my_snd_mixer_wait(snd_mixer_t *mixer, int timeout) {
   struct pollfd  spfds[16];
   struct pollfd *pfds = spfds;
   int            err, count;
+  void          *freeme = NULL;
 
   count = snd_mixer_poll_descriptors(mixer, pfds, sizeof(spfds) / sizeof(spfds[0]));
 
@@ -140,17 +142,24 @@ static int my_snd_mixer_wait(snd_mixer_t *mixer, int timeout) {
     return count;
 
   if ((unsigned int) count > sizeof(spfds) / sizeof(spfds[0])) {
-    pfds = calloc(count, sizeof(*pfds));
+    freeme = pfds = calloc(count, sizeof(*pfds));
 
     if (!pfds)
       return -ENOMEM;
 
     err = snd_mixer_poll_descriptors(mixer, pfds, (unsigned int) count);
+    // XXX is this assert correct ? if triggered, it would be alsa bug, not xine bug ?
     assert(err == count);
+    if (err < 0) {
+      free(freeme);
+      return err;
+    }
   }
 
   err = poll(pfds, (unsigned int) count, timeout);
 
+  free(freeme);
+
   if (err < 0)
     return -errno;
 
@@ -273,12 +282,16 @@ static void error_callback(const char *file, int line,
 #ifdef DEBUG
   va_list   args;
   char     *buf = NULL;
+  int       result;
 
   va_start(args, fmt);
-  vasprintf(&buf, fmt, args);
+  result = vasprintf(&buf, fmt, args);
   va_end(args);
-  printf("%s: %s() %s.\n", file, function, buf);
-  free(buf);
+
+  if (result >= 0) {
+    printf("%s: %s() %s.\n", file, function, buf);
+    free(buf);
+  }
 #endif
 }
 
@@ -925,7 +938,7 @@ static void ao_alsa_exit(ao_driver_t *this_gen) {
    * (i.e. xscreensaver).
    */
 
-  if(this->mixer.handle && this->mixer.thread != 0) {
+  if(this->mixer.handle && this->mixer.thread_created) {
     this->mixer.running = 0;
     pthread_join(this->mixer.thread, NULL);
     snd_mixer_close(this->mixer.handle);
@@ -1346,10 +1359,12 @@ static void ao_alsa_mixer_init(ao_driver_t *this_gen) {
     pthread_attr_getschedparam(&pth_attrs, &pth_params);
     pth_params.sched_priority = sched_get_priority_min(SCHED_OTHER);
     pthread_attr_setschedparam(&pth_attrs, &pth_params);
-    pthread_create(&this->mixer.thread, &pth_attrs, ao_alsa_handle_event_thread, (void *) this);
+    if (pthread_create(&this->mixer.thread, &pth_attrs, ao_alsa_handle_event_thread, (void *) this)) {
+      xprintf (this->class->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": pthread_create() failed\n");
+    } else {
+      this->mixer.thread_created = 1;
+    }
     pthread_attr_destroy(&pth_attrs);
-  } else {
-    this->mixer.thread = 0;
   }
 }
 
