@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2011 the xine project
+ * Copyright (C) 2004-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -224,7 +224,7 @@ static void dsound_destroy(LPDIRECTSOUND ds) {
 
 /* fill out wave format header */
 static void dsound_fill_wfx(WAVEFORMATEX *wfx, uint32_t bits, uint32_t rate, int channels, size_t frame_size) {
-  memset(wfx, 0, sizeof(wfx));
+  memset(wfx, 0, sizeof(*wfx));
   wfx->wFormatTag = WAVE_FORMAT_PCM;
   wfx->nChannels = channels;
   wfx->nSamplesPerSec = rate;
@@ -380,8 +380,8 @@ static int audio_set_volume(dx2_driver_t *this, int volume) {
 
 /* add given data into the ring buffer */
 static int audio_fill(dx2_driver_t *this, char *data, size_t size) {
-  DWORD size1, size2;
-  void *ptr1, *ptr2;
+  DWORD size1 = 0, size2 = 0;
+  void *ptr1 = NULL, *ptr2 = NULL;
   HRESULT err;
 
   /* lock a part of the buffer, begin position on free space */
@@ -592,7 +592,7 @@ static void *buffer_service(void *data) {
         // check for buffer underrun
         data_in_buffer =  buffer_occupied_size(this);
         if ( data_in_buffer < buffer_min){
-          xine_log(this->class->xine, XINE_LOG_MSG, _(LOG_MODULE ": play cursor overran (data %u, min %u), flushing buffers\n"),
+          xine_log(this->class->xine, XINE_LOG_MSG, _(LOG_MODULE ": play cursor overran (data %zu, min %zu), flushing buffers\n"),
                    data_in_buffer, buffer_min);
           if (!audio_stop(this)) goto fail;
           if (!audio_flush(this)) goto fail;
@@ -602,6 +602,10 @@ static void *buffer_service(void *data) {
         // just wait BUFFER_MIN_MS before next check
         xine_usec_sleep(BUFFER_MIN_MS * 1000);
 				break;
+
+      default:
+        pthread_mutex_unlock(&this->data_mutex);
+        break;
     }
   }
   return NULL;
@@ -694,15 +698,6 @@ static int ao_dx2_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
   if (!audio_create_buffers(this)) return 0;
   if (!audio_set_volume(this, this->volume)) goto fail_buffers;
 
-  if (pthread_cond_init(&this->data_cond, NULL) != 0) {
-    xine_log(this->class->xine, XINE_LOG_MSG, _(LOG_MODULE ": can't create pthread condition: %s\n"), strerror(errno));
-    goto fail_buffers;
-  }
-  if (pthread_mutex_init(&this->data_mutex, NULL) != 0) {
-    xine_log(this->class->xine, XINE_LOG_MSG, _(LOG_MODULE ": can't create pthread mutex: %s\n"), strerror(errno));
-    goto fail_cond;
-  }
-
   /* creating the service thread and waiting for its signal */
   pthread_mutex_lock(&this->data_mutex);
   if (pthread_create(&this->buffer_service, NULL, buffer_service, this) != 0) {
@@ -716,9 +711,6 @@ static int ao_dx2_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
 
 fail_mutex:
   pthread_mutex_unlock(&this->data_mutex);
-  pthread_mutex_destroy(&this->data_mutex);
-fail_cond:
-  pthread_cond_destroy(&this->data_cond);
 fail_buffers:
   audio_destroy_buffers(this);
   return 0;
@@ -825,14 +817,7 @@ static void ao_dx2_close(ao_driver_t *this_gen) {
     return;
   }
   lprintf("pthread joined\n");
-  pthread_mutex_unlock(&this->data_mutex);
 
-  if (pthread_cond_destroy(&this->data_cond) != 0) {
-    xine_log(this->class->xine, XINE_LOG_MSG, _(LOG_MODULE ": can't destroy pthread condition: %s\n"), strerror(errno));
-  }
-  if (pthread_mutex_destroy(&this->data_mutex) != 0) {
-    xine_log(this->class->xine, XINE_LOG_MSG, _(LOG_MODULE ": can't destroy pthread mutex: %s\n"), strerror(errno));
-  }
   audio_destroy_buffers(this);
 }
 
@@ -842,7 +827,17 @@ static void ao_dx2_exit(ao_driver_t *this_gen) {
 
   lprintf("exit instance\n");
 
-  dsound_destroy(this->ds);
+  if (this->ds) {
+    dsound_destroy(this->ds);
+  }
+
+  if (pthread_cond_destroy(&this->data_cond) != 0) {
+    xine_log(this->class->xine, XINE_LOG_MSG, _(LOG_MODULE ": can't destroy pthread condition: %s\n"), strerror(errno));
+  }
+  if (pthread_mutex_destroy(&this->data_mutex) != 0) {
+    xine_log(this->class->xine, XINE_LOG_MSG, _(LOG_MODULE ": can't destroy pthread mutex: %s\n"), strerror(errno));
+  }
+
   free(this);
 }
 
@@ -926,10 +921,22 @@ static ao_driver_t *open_plugin(audio_driver_class_t *class_gen, const void *dat
 
   this->failed = 0;
 
-  if ((this->ds = dsound_create()) == NULL) {
+  if (pthread_cond_init(&this->data_cond, NULL) != 0) {
+    xine_log(this->class->xine, XINE_LOG_MSG, _(LOG_MODULE ": can't create pthread condition: %s\n"), strerror(errno));
     free(this);
     return NULL;
   }
+  if (pthread_mutex_init(&this->data_mutex, NULL) != 0) {
+    xine_log(this->class->xine, XINE_LOG_MSG, _(LOG_MODULE ": can't create pthread mutex: %s\n"), strerror(errno));
+    pthread_cond_destroy(&this->data_cond);
+    free(this);
+    return NULL;
+  }
+
+  if ((this->ds = dsound_create()) == NULL) {
+    ao_dx2_exit(&this->ao_driver);
+    return NULL;
+  }
   test_capabilities(this);
 
   return (ao_driver_t *)this;
