@@ -1,7 +1,7 @@
 /* -*- Mode: C; c-basic-offset: 2; indent-tabs-mode: nil -*- */
 
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -307,7 +307,10 @@ static int connect_context(pulse_driver_t *this) {
       p = "Xine";
 
     this->context = pa_context_new(pa_threaded_mainloop_get_api(this->mainloop), p);
-    _x_assert(this->context);
+    if (!this->context) {
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG, "audio_pulse_out: pa_context_new() failed\n");
+      return -1;
+    }
 
     pa_context_set_state_callback(this->context, __xine_pa_context_state_callback, this);
 
@@ -349,6 +352,7 @@ static int connect_context(pulse_driver_t *this) {
      return -1;
   }
 
+  pa_operation_unref(operation);
   return 0;
 }
 
@@ -409,7 +413,8 @@ static int ao_pulse_open(ao_driver_t *this_gen,
       ss.format = PA_SAMPLE_FLOAT32NE;
       break;
     default:
-      _x_assert(!"Should not be reached");
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG, "audio_pulse_out: unsupported bit depth %u\n", bits);
+      goto fail;
   }
 
 #if PA_CHECK_VERSION(1,0,0)
@@ -462,7 +467,8 @@ static int ao_pulse_open(ao_driver_t *this_gen,
       cm.channels = 6;
       break;
     default:
-      _x_assert(!"Should not be reached");
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG, "audio_pulse_out: unsupported mode %08x\n", mode);
+      goto fail;
   }
 
   if (!pa_channel_map_valid(&cm)) {
@@ -502,7 +508,6 @@ static int ao_pulse_open(ao_driver_t *this_gen,
 
   _x_assert(!this->stream);
   this->stream = pa_stream_new_extended(this->context, "Audio Stream", formatv, formatc, proplist);
-  _x_assert(this->stream);
 
   if (proplist != NULL)
     pa_proplist_free(proplist);
@@ -513,9 +518,13 @@ static int ao_pulse_open(ao_driver_t *this_gen,
 #else
   _x_assert(!this->stream);
   this->stream = pa_stream_new(this->context, "Audio Stream", &ss, &cm);
-  _x_assert(this->stream);
 #endif
 
+  if (!this->stream) {
+    xprintf (this->xine, XINE_VERBOSITY_DEBUG, "audio_pulse_out: pa_Stream_new() failed\n");
+    goto fail;
+  }
+
   pa_stream_set_state_callback(this->stream, __xine_pa_stream_state_callback, this);
   pa_stream_set_write_callback(this->stream, __xine_pa_stream_request_callback, this);
   pa_stream_set_latency_update_callback(this->stream, __xine_pa_stream_notify_callback, this);
@@ -544,17 +553,16 @@ static int ao_pulse_open(ao_driver_t *this_gen,
   if (encoding != PA_ENCODING_INVALID) {
     const pa_format_info *info = pa_stream_get_format_info(this->stream);
 
-    _x_assert(info);
-    if (pa_format_info_is_pcm (info)) {
-      xprintf (this->xine, XINE_VERBOSITY_DEBUG, "digital pass-through not available\n");
-    } else {
-      xprintf (this->xine, XINE_VERBOSITY_DEBUG, "digital pass-through enabled\n");
+    if (info) {
+      if (pa_format_info_is_pcm (info)) {
+        xprintf (this->xine, XINE_VERBOSITY_DEBUG, "digital pass-through not available\n");
+      } else {
+        xprintf (this->xine, XINE_VERBOSITY_DEBUG, "digital pass-through enabled\n");
+      }
     }
   }
 #endif
 
-  pa_threaded_mainloop_unlock(this->mainloop);
-
   /* Now we must handle a problem: at init time, xine might have tried to set the default volume value
    * This won't work with pulseaudio, because, at that time, pulseaudio doesn't have a stream.
    * As a workaround, we re-do the volume thingie here */
@@ -593,6 +601,8 @@ static int ao_pulse_open(ao_driver_t *this_gen,
 
   pa_operation_unref(operation);
 
+  pa_threaded_mainloop_unlock(this->mainloop);
+
   return this->sample_rate;
 
  fail:
@@ -987,8 +997,13 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
     }
   }
 
+  _x_assert(!this->mainloop);
   this->mainloop = pa_threaded_mainloop_new();
-  _x_assert(this->mainloop);
+  if (!this->mainloop) {
+    xprintf (this->xine, XINE_VERBOSITY_DEBUG, "audio_pulse_out: pa_threaded_mainloop_new() failed\n");
+    ao_pulse_exit((ao_driver_t *) this);
+    return NULL;
+  }
   pa_threaded_mainloop_start(this->mainloop);
 
   /*
