@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2016 the xine project
+ * Copyright (C) 2001-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -51,6 +51,7 @@
 typedef struct {
   audio_decoder_class_t   decoder_class;
 
+  xine_t                 *xine;
   float                   gain;
 } ff_audio_class_t;
 
@@ -120,7 +121,7 @@ static void ff_aac_mode_set (ff_audio_decoder_t *this, int reset) {
         "ffmpeg_audio_dec: looking for possible AAC ADTS syncwords...\n");
     }
     if ((this->aac_mode < 0) || (this->aac_mode == AAC_MODE_ADTS)) {
-      if (this->context && this->context->extradata_size) {
+      if (this->context->extradata_size) {
         xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
           "ffmpeg_audio_dec: AAC raw mode with global header\n");
         this->aac_mode = AAC_MODE_RAW;
@@ -166,7 +167,7 @@ static int ff_aac_mode_parse (ff_audio_decoder_t *this, uint8_t *buf, int size,
         if (this->buftype == BUF_AUDIO_AAC_LATM) {
           xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
             "ffmpeg_audio_dec: stream says LATM but is ADTS -> switching decoders\n");
-          if (this->context && this->decoder_ok) {
+          if (this->decoder_ok) {
             pthread_mutex_lock (&ffmpeg_lock);
             avcodec_close (this->context);
             pthread_mutex_unlock (&ffmpeg_lock);
@@ -221,7 +222,7 @@ static void ff_audio_ensure_buffer_size(ff_audio_decoder_t *this, int size) {
     xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
             _("ffmpeg_audio_dec: increasing buffer to %d to avoid overflow.\n"),
             this->bufsize);
-    this->buf = xine_realloc_aligned (this->buf, this->bufsize + FF_INPUT_BUFFER_PADDING_SIZE);
+    this->buf = xine_realloc_aligned (this->buf, this->bufsize + AV_INPUT_BUFFER_PADDING_SIZE);
   }
 }
 
@@ -232,9 +233,9 @@ static void ff_audio_handle_special_buffer(ff_audio_decoder_t *this, buf_element
 
     free (this->context->extradata);
     this->context->extradata_size = buf->decoder_info[2];
-    this->context->extradata = malloc (buf->decoder_info[2] + FF_INPUT_BUFFER_PADDING_SIZE);
+    this->context->extradata = malloc (buf->decoder_info[2] + AV_INPUT_BUFFER_PADDING_SIZE);
     memcpy (this->context->extradata, buf->decoder_info_ptr[2], buf->decoder_info[2]);
-    memset (this->context->extradata + buf->decoder_info[2], 0, FF_INPUT_BUFFER_PADDING_SIZE);
+    memset (this->context->extradata + buf->decoder_info[2], 0, AV_INPUT_BUFFER_PADDING_SIZE);
 
     ff_aac_mode_set (this, 0);
   }
@@ -451,10 +452,10 @@ static void ff_handle_header_buffer(ff_audio_decoder_t *this, buf_element_t *buf
             this->ff_channels, this->ff_bits, this->ff_sample_rate,
             this->context->block_align);
           if (!data_len) break;
-          e = malloc (data_len + FF_INPUT_BUFFER_PADDING_SIZE);
+          e = malloc (data_len + AV_INPUT_BUFFER_PADDING_SIZE);
           if (!e) break;
           xine_fast_memcpy (e, p, data_len);
-          memset (e + data_len, 0, FF_INPUT_BUFFER_PADDING_SIZE);
+          memset (e + data_len, 0, AV_INPUT_BUFFER_PADDING_SIZE);
           this->context->extradata = e;
           this->context->extradata_size = data_len;
           break;
@@ -509,7 +510,7 @@ static void ff_map_channels (ff_audio_decoder_t *this) {
   ff_map = this->context->channel_layout;
   if (!ff_map) /* wma2 bug */
 #endif
-    ff_map = (1 << this->context->channels) - 1;
+    ff_map = ((uint64_t)1 << this->context->channels) - 1;
 
   if ((caps != this->ao_caps) || (ff_map != this->ff_map)) {
     int i, j;
@@ -1008,7 +1009,7 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
       offset = 0;
 
       /* pad input data */
-      memset(&this->buf[this->size], 0, FF_INPUT_BUFFER_PADDING_SIZE);
+      memset(this->buf + this->size, 0, AV_INPUT_BUFFER_PADDING_SIZE);
 
       while (this->size>=0) {
 
@@ -1228,7 +1229,7 @@ static void ff_audio_dispose (audio_decoder_t *this_gen) {
     pthread_mutex_unlock (&ffmpeg_lock);
   }
 
-  if( this->context && this->decoder_ok ) {
+  if( this->decoder_ok ) {
 #if XFF_AUDIO > 3
     XFF_FREE_FRAME (this->av_frame);
 #endif
@@ -1242,11 +1243,9 @@ static void ff_audio_dispose (audio_decoder_t *this_gen) {
   xine_free_aligned (this->buf);
   xine_free_aligned (this->decode_buffer);
 
-  if (this->context) {
-    _x_freep (&this->context->extradata);
-    this->context->extradata_size = 0;
-    XFF_FREE_CONTEXT (this->context);
-  }
+  _x_freep (&this->context->extradata);
+  this->context->extradata_size = 0;
+  XFF_FREE_CONTEXT (this->context);
 
   free (this_gen);
 }
@@ -1256,26 +1255,43 @@ static audio_decoder_t *ff_audio_open_plugin (audio_decoder_class_t *class_gen,
   ff_audio_decoder_t *this ;
 
   this = calloc(1, sizeof (ff_audio_decoder_t));
+  if (!this)
+    return NULL;
 
-  this->class = (ff_audio_class_t *)class_gen;
-
-  this->audio_decoder.decode_data         = ff_audio_decode_data;
-  this->audio_decoder.reset               = ff_audio_reset;
-  this->audio_decoder.discontinuity       = ff_audio_discontinuity;
-  this->audio_decoder.dispose             = ff_audio_dispose;
-
+  /* Do these first, when compiler still knows stream is all zeroed.
+   * Let it optimize away this on most systems where clear mem
+   * interpretes as 0, 0f or NULL safely.
+   */
   this->output_open = 0;
   this->ff_channels = 0;
+  this->size        = 0;
+  this->decoder_ok  = 0;
+
+  this->class  = (ff_audio_class_t *)class_gen;
   this->stream = stream;
-  this->buf = NULL;
-  this->size = 0;
-  this->bufsize = 0;
-  this->decoder_ok = 0;
 
-  ff_audio_ensure_buffer_size(this, AUDIOBUFSIZE);
+  this->audio_decoder.decode_data   = ff_audio_decode_data;
+  this->audio_decoder.reset         = ff_audio_reset;
+  this->audio_decoder.discontinuity = ff_audio_discontinuity;
+  this->audio_decoder.dispose       = ff_audio_dispose;
+
+  this->bufsize = AUDIOBUFSIZE;
+  do {
+    this->buf = xine_malloc_aligned (AUDIOBUFSIZE + AV_INPUT_BUFFER_PADDING_SIZE);
+    if (this->buf) {
+      this->context = XFF_ALLOC_CONTEXT ();
+      if (this->context) {
+        this->decode_buffer = xine_malloc_aligned (AVCODEC_MAX_AUDIO_FRAME_SIZE);
+        if (this->decode_buffer)
+          break;
+        XFF_FREE_CONTEXT (this->context);
+      }
+      xine_free_aligned (this->buf);
+    }
+    free (this);
+    return NULL;
+  } while (0);
 
-  this->context = XFF_ALLOC_CONTEXT();
-  this->decode_buffer = xine_malloc_aligned (AVCODEC_MAX_AUDIO_FRAME_SIZE);
 #if XFF_AUDIO > 3
   this->av_frame = NULL;
 #endif
@@ -1288,16 +1304,31 @@ static void ff_gain_cb (void *user_data, xine_cfg_entry_t *entry) {
   class->gain = (float)0x7fff * powf ((float)10, (float)entry->num_value / (float)20);
 }
 
+static void dispose_audio_class (audio_decoder_class_t *this_gen) {
+
+  ff_audio_class_t *this = (ff_audio_class_t *)this_gen;
+  config_values_t *config = this->xine->config;
+
+  config->unregister_callback (config, "audio.processing.ffmpeg_gain_dB");
+
+  free (this);
+}
+
 void *init_audio_plugin (xine_t *xine, void *data) {
 
   ff_audio_class_t *this ;
 
   this = calloc(1, sizeof (ff_audio_class_t));
+  if (!this) {
+    return NULL;
+  }
 
   this->decoder_class.open_plugin     = ff_audio_open_plugin;
   this->decoder_class.identifier      = "ffmpeg audio";
   this->decoder_class.description     = N_("ffmpeg based audio decoder plugin");
-  this->decoder_class.dispose         = default_audio_decoder_class_dispose;
+  this->decoder_class.dispose         = dispose_audio_class;
+
+  this->xine = xine;
 
   pthread_once( &once_control, init_once_routine );
 
