@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2012 the xine project
+ * Copyright (C) 2001-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -20,12 +20,22 @@
  * Simple MPEG-ES parser/framer by Thibaut Mattern (tmattern@noos.fr)
  *   based on libmpeg2 decoder.
  */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include "ff_mpeg_parser.h"
+
+#include <stdint.h>
+#include <stdlib.h>
+
 #define LOG_MODULE "mpeg_parser"
 #define LOG_VERBOSE
 /*
 #define LOG
 */
-#include "ff_mpeg_parser.h"
+#include <xine/xine_internal.h>
 
 /* mpeg frame rate table from lavc */
 static const int frame_rate_tab[][2] = {
@@ -48,9 +58,9 @@ static const int frame_rate_tab[][2] = {
     {    0,    0},
 };
 
-void mpeg_parser_init (mpeg_parser_t *parser)
+void mpeg_parser_init (mpeg_parser_t *parser, size_t padding_size)
 {
-  parser->chunk_buffer = malloc(BUFFER_SIZE + FF_INPUT_BUFFER_PADDING_SIZE);
+  parser->chunk_buffer = malloc(BUFFER_SIZE + padding_size);
   mpeg_parser_reset(parser);
 }
 
@@ -216,14 +226,17 @@ static int parse_chunk (mpeg_parser_t *parser, int code, uint8_t *buffer, int le
     switch (buffer[0] & 0xf0) {
     case 0x10:     /* sequence extension */
       parser->is_mpeg1 = 0;
+      break;
     }
+    break;
 
   default:
     if (code >= 0xb9)
       lprintf ("stream not demultiplexed ?\n");
 
-    if (code >= 0xb0)
-      break;
+    //if (code >= 0xb0)
+    //  break;
+    break;
   }
   return is_frame_done;
 }
