@@ -234,4 +234,15 @@
 #define XFF_PACKET_UNREF av_packet_unref
 #endif
 
+#ifndef AV_INPUT_BUFFER_PADDING_SIZE
+#  define AV_INPUT_BUFFER_PADDING_SIZE FF_INPUT_BUFFER_PADDING_SIZE
+#endif
+#ifndef AV_CODEC_CAP_DR1
+#  define AV_CODEC_CAP_DR1 CODEC_CAP_DR1
+#endif
+#ifndef AV_CODEC_FLAG2_FAST
+#  define AV_CODEC_FLAG2_FAST CODEC_FLAG2_FAST
+#endif
+
+
 #endif /* XINE_AVCODEC_COMPAT_H */
