@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2013 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -595,6 +595,8 @@ static void read_chapter_comment (demux_ogg_t *this, ogg_packet *op) {
         chapter_time = chapter_name = 0;
       }
     }
+    free(chapter_name);
+    free(chapter_time);
   }
   vorbis_comment_clear(&vc);
   vorbis_info_clear(&vi);
@@ -897,7 +899,7 @@ static void decode_vorbis_header (demux_ogg_t *this, const int stream_num, ogg_p
 static void decode_speex_header (demux_ogg_t *this, const int stream_num, ogg_packet *op) {
 #ifdef HAVE_SPEEX
   void *st;
-  SpeexMode *mode;
+  const SpeexMode *mode;
   SpeexHeader *header;
 
   this->si[stream_num]->buf_types = BUF_AUDIO_SPEEX
@@ -909,7 +911,7 @@ static void decode_speex_header (demux_ogg_t *this, const int stream_num, ogg_pa
 
   if (header) {
     int bitrate;
-    mode = (SpeexMode *) speex_mode_list[header->mode];
+    mode = speex_mode_list[header->mode];
 
     st = speex_decoder_init (mode);
 
@@ -982,6 +984,7 @@ static void decode_video_header (demux_ogg_t *this, const int stream_num, ogg_pa
   this->si[stream_num]->buf_types |= channel;
   this->si[stream_num]->headers = 0; /* header is sent below */
 
+#ifdef LOG
   lprintf ("subtype          %.4s\n", (char*)&locsubtype);
   lprintf ("time_unit        %" PRId64 "\n", loctime_unit);
   lprintf ("samples_per_unit %" PRId64 "\n", locsamples_per_unit);
@@ -991,6 +994,7 @@ static void decode_video_header (demux_ogg_t *this, const int stream_num, ogg_pa
   lprintf ("width            %d\n", locwidth);
   lprintf ("height           %d\n", locheight);
   lprintf ("buf_type         %08x\n",this->si[stream_num]->buf_types);
+#endif
 
   bih.biSize=sizeof(xine_bmiheader);
   bih.biWidth = locwidth;
@@ -1075,6 +1079,7 @@ static void decode_audio_header (demux_ogg_t *this, const int stream_num, ogg_pa
       /*break;*/
     }
 
+#ifdef LOG
     lprintf ("subtype          0x%x\n", codec);
     lprintf ("time_unit        %" PRId64 "\n", loctime_unit);
     lprintf ("samples_per_unit %" PRId64 "\n", locsamples_per_unit);
@@ -1085,6 +1090,7 @@ static void decode_audio_header (demux_ogg_t *this, const int stream_num, ogg_pa
     lprintf ("blockalign       %d\n", locblockalign);
     lprintf ("avgbytespersec   %d\n", locavgbytespersec);
     lprintf ("buf_type         %08x\n",this->si[stream_num]->buf_types);
+#endif
 
     buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
     buf->type = this->si[stream_num]->buf_types;
@@ -1302,6 +1308,8 @@ static void decode_flac_header (demux_ogg_t *this, const int stream_num, ogg_pac
   buf_element_t *buf;
   xine_waveformatex wave;
 
+  memset(&wave, 0, sizeof(wave));
+
   /* Packet type */
   _x_assert(op->packet[0] == 0x7F);
 
@@ -2238,9 +2246,9 @@ static const demuxer_info_t demux_info_ogg = {
   10                       /* priority */
 };
 
-extern const demuxer_info_t dec_info_vorbis;
-extern const demuxer_info_t dec_info_speex;
-extern const demuxer_info_t dec_info_theora;
+extern const decoder_info_t dec_info_vorbis;
+extern const decoder_info_t dec_info_speex;
+extern const decoder_info_t dec_info_theora;
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
