@@ -1,3 +1,32 @@
+/*
+ * Copyright (C) 2000-2017 the xine project
+ *
+ * This file is part of xine, a free video player.
+ *
+ * xine is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * xine is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
+ *
+ * demultiplexer for asf streams
+ *
+ * based on ffmpeg's
+ * ASF compatible encoder and decoder.
+ * Copyright (c) 2000, 2001 Gerard Lantau.
+ *
+ * GUID list from avifile
+ * some other ideas from MPlayer
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -282,6 +311,16 @@ static int asf_header_parse_file_properties(asf_header_t *header, uint8_t *buffe
   return 1;
 }
 
+static void asf_header_delete_stream_properties(asf_stream_t *asf_stream) {
+  if (asf_stream) {
+    if (asf_stream->private_data)
+      free(asf_stream->private_data);
+    if (asf_stream->error_correction_data)
+      free(asf_stream->error_correction_data);
+    free(asf_stream);
+  }
+}
+
 static int asf_header_parse_stream_properties(asf_header_t *header, uint8_t *buffer, int buffer_len) {
   asf_reader_t reader;
   uint16_t flags = 0;
@@ -337,20 +376,28 @@ static int asf_header_parse_stream_properties(asf_header_t *header, uint8_t *buf
   if (stream_id >= 0) {
     header->streams[stream_id] = asf_stream;
     header->stream_count++;
+  } else {
+    asf_header_delete_stream_properties(asf_stream);
   }
   return 1;
 
 exit_error:
-  if (asf_stream) {
-    if (asf_stream->private_data)
-      free(asf_stream->private_data);
-    if (asf_stream->error_correction_data)
-      free(asf_stream->error_correction_data);
-    free(asf_stream);
-  }
+  asf_header_delete_stream_properties(asf_stream);
   return 0;
 }
 
+static void asf_header_delete_stream_extended_properties(asf_stream_extension_t *asf_stream_extension) {
+  int i;
+
+  if (asf_stream_extension->stream_name_count > 0) {
+    for (i = 0; i < asf_stream_extension->stream_name_count; i++) {
+      free(asf_stream_extension->stream_names[i]);
+    }
+    free(asf_stream_extension->stream_names);
+  }
+  free(asf_stream_extension);
+}
+
 static int asf_header_parse_stream_extended_properties(asf_header_t *header, uint8_t *buffer, int buffer_len) {
   asf_reader_t reader;
   uint32_t flags = 0;
@@ -466,6 +513,10 @@ static int asf_header_parse_stream_extended_properties(asf_header_t *header, uin
   lprintf("  no_cleanpoints_flag:               %d\n", asf_stream_extension->no_cleanpoints_flag);
   lprintf("  resend_live_cleanpoints_flag:      %d\n", asf_stream_extension->resend_live_cleanpoints_flag);
 
+  if (stream_id < 0) {
+    asf_header_delete_stream_extended_properties(asf_stream_extension);
+  }
+
   return 1;
 }
 
@@ -641,8 +692,10 @@ static int asf_header_parse_content_description(asf_header_t *header_pub, uint8_
   if (!content)
     return 0;
 
-  if ( (iconv_cd = iconv_open("UTF-8", "UCS-2LE")) == (iconv_t)-1 )
+  if ( (iconv_cd = iconv_open("UTF-8", "UCS-2LE")) == (iconv_t)-1 ) {
+    free(content);
     return 0;
+  }
 
   asf_reader_init(&reader, buffer, buffer_len);
   asf_reader_get_16(&reader, &title_length);
@@ -677,10 +730,6 @@ asf_header_t *asf_header_new (uint8_t *buffer, int buffer_len) {
   uint32_t object_count;
   uint16_t junk;
 
-  asf_header = calloc(1, sizeof(asf_header_internal_t));
-  if (!asf_header)
-    return NULL;
-
   lprintf("parsing_asf_header\n");
   if (buffer_len < 6) {
     printf("invalid buffer size\n");
@@ -788,26 +837,6 @@ static void asf_header_delete_content(asf_content_t *asf_content) {
   free(asf_content);
 }
 
-static void asf_header_delete_stream_properties(asf_stream_t *asf_stream) {
-  if (asf_stream->private_data)
-    free(asf_stream->private_data);
-  if (asf_stream->error_correction_data)
-    free(asf_stream->error_correction_data);
-  free(asf_stream);
-}
-
-static void asf_header_delete_stream_extended_properties(asf_stream_extension_t *asf_stream_extension) {
-  int i;
-
-  if (asf_stream_extension->stream_name_count > 0) {
-    for (i = 0; i < asf_stream_extension->stream_name_count; i++) {
-      free(asf_stream_extension->stream_names[i]);
-    }
-    free(asf_stream_extension->stream_names);
-  }
-  free(asf_stream_extension);
-}
-
 void asf_header_delete (asf_header_t *header_pub) {
   asf_header_internal_t *header = (asf_header_internal_t *)header_pub;
   int i;
