@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2013 the xine project
+ * Copyright (C) 2001-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -160,7 +160,7 @@ static int open_ac3_file(demux_ac3_t *this) {
       return 0;
   }
 
-  lprintf("peak size: %d\n", peak_size);
+  lprintf("peak size: %zu\n", peak_size);
 
   /* Check for wav header, as we'll handle AC3 with a wav header shoved
   * on the front for CD burning */
