@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2016 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -692,15 +692,16 @@ static audio_index_entry_t *audio_cur_index_entry(demux_avi_t *this,
 static void AVI_close(avi_t *AVI){
   int i;
 
-  if(AVI->idx) free(AVI->idx);
-  if(AVI->video_idx.vindex) free(AVI->video_idx.vindex);
-  if(AVI->bih) free(AVI->bih);
+  _x_freep(&AVI->idx);
+  _x_freep(&AVI->video_idx.vindex);
+  _x_freep(&AVI->bih);
 
   for(i=0; i<AVI->n_audio; i++) {
-    if(AVI->audio[i]->audio_idx.aindex) free(AVI->audio[i]->audio_idx.aindex);
-    if(AVI->audio[i]->wavex) free(AVI->audio[i]->wavex);
-    free(AVI->audio[i]);
+    _x_freep(&AVI->audio[i]->audio_idx.aindex);
+    _x_freep(&AVI->audio[i]->wavex);
+    _x_freep(&AVI->audio[i]);
   }
+
   free(AVI);
 }
 
@@ -884,7 +885,8 @@ static avi_t *XINE_MALLOC AVI_init(demux_avi_t *this) {
           avi_audio_t *a = (avi_audio_t *) calloc(1, sizeof(avi_audio_t));
           if(a==NULL) {
             this->AVI_errno = AVI_ERR_NO_MEM;
-            return 0;
+            AVI_close(AVI);
+            return NULL;
           }
           AVI->audio[AVI->n_audio] = a;
 
@@ -927,7 +929,8 @@ static avi_t *XINE_MALLOC AVI_init(demux_avi_t *this) {
           malloc((n < sizeof(xine_bmiheader)) ? sizeof(xine_bmiheader) : n);
         if(AVI->bih == NULL) {
           this->AVI_errno = AVI_ERR_NO_MEM;
-          return 0;
+          AVI_close(AVI);
+          return NULL;
         }
 
         memcpy (AVI->bih, hdrl_data+i, n);
@@ -977,7 +980,8 @@ static avi_t *XINE_MALLOC AVI_init(demux_avi_t *this) {
         wavex = (xine_waveformatex *)malloc(n);
         if (!wavex) {
           this->AVI_errno = AVI_ERR_NO_MEM;
-          return 0;
+          AVI_close(AVI);
+          return NULL;
         }
         memcpy((void *)wavex, hdrl_data+i, n);
         _x_waveformatex_le2me( wavex );
@@ -1033,6 +1037,7 @@ static avi_t *XINE_MALLOC AVI_init(demux_avi_t *this) {
       {
          lprintf("broken index !, dwSize=%d, entries=%d\n", n, superindex->nEntriesInUse);
          i += 8 + n;
+         free(superindex);
          continue;
       }
 
@@ -1060,6 +1065,8 @@ static avi_t *XINE_MALLOC AVI_init(demux_avi_t *this) {
          AVI->audio[AVI->n_audio-1]->audio_superindex = superindex;
          AVI->is_opendml = 1;
       } else {
+        free(superindex->aIndex);
+        free(superindex);
          xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
 	               "demux_avi: there should not be an index there, lasttag = %d\n", lasttag);
       }
