@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -269,7 +269,7 @@ static int process_ipmovie_chunk(demux_ipmovie_t *this) {
 
       case OPCODE_INIT_VIDEO_BUFFERS:
         lprintf("initialize video buffers\n");
-        if (this->decode_map) free(this->decode_map);
+        _x_freep(&this->decode_map);
         if ((opcode_version > 2) || (opcode_size > 8)) {
           lprintf("bad init_video_buffers opcode\n");
           chunk_type = CHUNK_BAD;
@@ -284,7 +284,7 @@ static int process_ipmovie_chunk(demux_ipmovie_t *this) {
         this->bih.biHeight = _X_LE_16(&scratch[2]) * 8;
         /* set up staging area for decode map */
         this->decode_map_size = (this->bih.biWidth / 8) * (this->bih.biHeight / 8) / 2;
-        this->decode_map = xine_xmalloc(this->decode_map_size);
+        this->decode_map = calloc(1, this->decode_map_size);
         if (!this->decode_map)
           this->status = DEMUX_FINISHED;
         lprintf("video resolution: %d x %d\n",
@@ -644,7 +644,7 @@ static int demux_ipmovie_seek (demux_plugin_t *this_gen,
 static void demux_ipmovie_dispose (demux_plugin_t *this_gen) {
   demux_ipmovie_t *this = (demux_ipmovie_t *) this_gen;
 
-  free(this->decode_map);
+  _x_freep(&this->decode_map);
   free(this);
 }
 
@@ -673,6 +673,9 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   demux_ipmovie_t    *this;
 
   this         = calloc(1, sizeof(demux_ipmovie_t));
+  if (!this) {
+    return NULL;
+  }
   this->stream = stream;
   this->input  = input;
 
@@ -714,7 +717,9 @@ void *demux_ipmovie_init_plugin (xine_t *xine, void *data) {
   demux_ipmovie_class_t     *this;
 
   this = calloc(1, sizeof(demux_ipmovie_class_t));
-
+  if (!this) {
+    return NULL;
+  }
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.description     = N_("Interplay MVE Movie demux plugin");
   this->demux_class.identifier      = "Interplay MVE";
