@@ -177,10 +177,11 @@ static int parse_info(demux_matroska_t *this) {
 
 static int parse_video_track (demux_matroska_t *this, matroska_video_track_t *vt) {
   ebml_parser_t *ebml = this->ebml;
-  int next_level = 4;
+  int this_level = ebml->level;
+  int next_level = this_level;
   uint64_t val;
 
-  while (next_level == 4) {
+  while (next_level == this_level) {
     ebml_elem_t elem;
 
     if (!ebml_read_elem_head(ebml, &elem))
@@ -230,9 +231,10 @@ static int parse_video_track (demux_matroska_t *this, matroska_video_track_t *vt
 
 static int parse_audio_track (demux_matroska_t *this, matroska_audio_track_t *at) {
   ebml_parser_t *ebml = this->ebml;
-  int next_level = 4;
+  int this_level = ebml->level;
+  int next_level = this_level;
 
-  while (next_level == 4) {
+  while (next_level == this_level) {
     ebml_elem_t elem;
     uint64_t    val;
     double      fval;
@@ -278,9 +280,10 @@ static int parse_audio_track (demux_matroska_t *this, matroska_audio_track_t *at
 
 static int parse_content_compression (demux_matroska_t *this, matroska_track_t *track) {
   ebml_parser_t *ebml = this->ebml;
-  int next_level = 6;
+  int this_level = ebml->level;
+  int next_level = this_level;
 
-  while (next_level == 6) {
+  while (next_level == this_level) {
     ebml_elem_t elem;
     uint64_t    val;
 
@@ -327,9 +330,10 @@ static int parse_content_compression (demux_matroska_t *this, matroska_track_t *
 
 static int parse_content_encoding (demux_matroska_t *this, matroska_track_t *track) {
   ebml_parser_t *ebml = this->ebml;
-  int next_level = 5;
+  int this_level = ebml->level;
+  int next_level = this_level;
 
-  while (next_level == 5) {
+  while (next_level == this_level) {
     ebml_elem_t elem;
     uint64_t    val;
 
@@ -388,9 +392,10 @@ static int parse_content_encoding (demux_matroska_t *this, matroska_track_t *tra
 
 static int parse_content_encodings (demux_matroska_t *this, matroska_track_t *track) {
   ebml_parser_t *ebml = this->ebml;
-  int next_level = 4;
+  int this_level = ebml->level;
+  int next_level = this_level;
 
-  while (next_level == 4) {
+  while (next_level == this_level) {
     ebml_elem_t elem;
 
     if (!ebml_read_elem_head(ebml, &elem))
@@ -882,6 +887,8 @@ static void handle_sub_ssa (demux_plugin_t *this_gen, matroska_track_t *track,
                             uint8_t *data, size_t data_len,
                             int64_t data_pts, int data_duration,
                             int input_normpos, int input_time) {
+  demux_matroska_t *this = (demux_matroska_t *)this_gen;
+
   buf_element_t *buf;
   uint32_t *val;
   int commas = 0;
@@ -902,8 +909,8 @@ static void handle_sub_ssa (demux_plugin_t *this_gen, matroska_track_t *track,
   buf->type = track->buf_type;
   buf->decoder_flags = decoder_flags | BUF_FLAG_SPECIAL;
   buf->decoder_info[1] = BUF_SPECIAL_CHARSET_ENCODING;
-  buf->decoder_info_ptr[2] = "utf-8";
-  buf->decoder_info[2] = strlen(buf->decoder_info_ptr[2]);
+  buf->decoder_info_ptr[2] = this->class->literal_utf_8;
+  buf->decoder_info[2] = LITERAL_UTF_8_LEN;
 
   val = (uint32_t *)buf->content;
   *val++ = data_pts / 90;                    /* start time */
@@ -971,8 +978,8 @@ static void handle_sub_utf8 (demux_plugin_t *this_gen, matroska_track_t *track,
     buf->type = track->buf_type;
     buf->decoder_flags = decoder_flags | BUF_FLAG_SPECIAL;
     buf->decoder_info[1] = BUF_SPECIAL_CHARSET_ENCODING;
-    buf->decoder_info_ptr[2] = "utf-8";
-    buf->decoder_info[2] = strlen(buf->decoder_info_ptr[2]);
+    buf->decoder_info_ptr[2] = this->class->literal_utf_8;
+    buf->decoder_info[2] = LITERAL_UTF_8_LEN;
 
     val = (uint32_t *)buf->content;
     *val++ = data_pts / 90;                    /* start time */
@@ -992,6 +999,111 @@ static void handle_sub_utf8 (demux_plugin_t *this_gen, matroska_track_t *track,
   }
 }
 
+/* 0x02 (not a keyframe) | 0x01 (is visible) */
+static int vp9_frametype (const uint8_t *h) {
+  if ((h[0] & 0xc0) != 0x80) /* frame marker missing */
+    return 1;
+  if ((h[0] & 0x30) == 0x30) { /* profile 3+ */
+    if (h[0] & 0x08) /* profile 4+ not supported */
+      return 0;
+    if (h[0] & 0x04) /* 3 bits ref */
+      return 1;
+    return h[0] & 3;
+  }
+  if (h[0] & 0x08) /* 3 bits ref */
+    return 1;
+  return (h[0] >> 1) & 3;
+}
+
+static void handle_vp9 (demux_plugin_t *this_gen, matroska_track_t *track, int decoder_flags,
+  uint8_t *data, size_t data_len, int64_t data_pts, int data_duration, int input_normpos, int input_time) {
+  /* !@$%*#~ undocumented multiframe feature */
+  do {
+    uint8_t mfhead;
+    uint8_t *l;
+    int mfframes, i;
+    size_t mfbytes, mfsize;
+
+    if (!data_len)
+      return;
+    mfhead = data[data_len - 1];
+    if ((mfhead & 0xe0) != 0xc0)
+      break;
+    mfframes =  (mfhead & 7) + 1;
+    mfbytes  = ((mfhead >> 3) & 3) + 1;
+    mfsize   =   mfframes * mfbytes + 2;
+    if (mfsize > data_len)
+      break;
+    l = data + data_len - mfsize;
+    if (*l++ != mfhead)
+      break;
+    data_len -= mfsize;
+    data_duration /= mfframes;
+    for (i = 0; i < mfframes; i++) {
+      size_t flen = *l++;
+      if (mfbytes > 1)
+        flen += (size_t)(*l++) << 8;
+      if (mfbytes > 2)
+        flen += (size_t)(*l++) << 16;
+      if (mfbytes > 3)
+        flen += (size_t)(*l++) << 24;
+      if (flen > data_len)
+        flen = data_len;
+      if (flen) {
+        /* Nasty kludge: take invisible frames out of pts sequence.
+         * A conventional b-frame sequence looks like this:
+         * I1 P1 B1 B2 B3 P2 B4 ...
+         * after reordering:
+         * I1 B1 B2 B3 P1 B4 ... P2
+         * VP9 now avoids reordering and storing both dts and pts
+         * by adding "P" as "invisible" to a (multiframe group):
+         * I1 (P1 B1) B2 B3 (P2 B4) ...
+         * "B3" usually is very small, and yields a quite verbatim copy of "P1",
+         * which does not stay completely hidden this way.
+         */
+        int64_t pts = data_pts;
+        int type = vp9_frametype (data);
+        if (type & 2)
+          decoder_flags &= ~BUF_FLAG_KEYFRAME;
+        else
+          decoder_flags |= BUF_FLAG_KEYFRAME;
+        if (type & 1) { /* visible */
+          if (!pts)
+            pts = track->delayed_pts;
+          track->delayed_pts = data_pts = 0;
+        } else { /* invisible */
+          pts = 0;
+          track->delayed_pts = data_pts;
+        }
+        _x_demux_send_data (track->fifo, data, flen, pts, track->buf_type,
+          decoder_flags, input_normpos, input_time, data_duration, 0);
+        data += flen;
+        data_len -= flen;
+      }
+    }
+    return;
+  } while (0);
+
+  {
+    int64_t pts = data_pts;
+    int type = vp9_frametype (data);
+    if (type & 2)
+      decoder_flags &= ~BUF_FLAG_KEYFRAME;
+    else
+      decoder_flags |= BUF_FLAG_KEYFRAME;
+    if (type & 1) { /* visible */
+      if (!pts)
+        pts = track->delayed_pts;
+      track->delayed_pts = data_pts = 0;
+    } else { /* invisible */
+      pts = 0;
+      track->delayed_pts = data_pts;
+    }
+    _x_demux_send_data (track->fifo, data, data_len, pts, track->buf_type,
+      decoder_flags, input_normpos, input_time, data_duration, 0);
+  }
+}
+
 
 static int uncompress_zlib(demux_matroska_t *this,
                            const uint8_t *data, size_t data_len, uint8_t **out_data, size_t *out_data_len)
@@ -1078,7 +1190,7 @@ static void handle_vobsub (demux_plugin_t *this_gen, matroska_track_t *track,
   }
   else
   {
-    lprintf("VobSub: track %d isn't compressed (%d bytes)\n",
+    lprintf("VobSub: track %d isn't compressed (%zu bytes)\n",
             (int)track->track_num, data_len);
   }
 
@@ -1138,6 +1250,8 @@ static void handle_hdmv_textst (demux_plugin_t *this_gen, matroska_track_t *trac
                                 uint8_t *data, size_t data_len,
                                 int64_t data_pts, int data_duration,
                                 int input_normpos, int input_time) {
+  demux_matroska_t *this = (demux_matroska_t *)this_gen;
+
   buf_element_t *buf;
   uint32_t *val;
   char *dest;
@@ -1168,8 +1282,8 @@ static void handle_hdmv_textst (demux_plugin_t *this_gen, matroska_track_t *trac
   buf->type = track->buf_type;
   buf->decoder_flags = decoder_flags | BUF_FLAG_SPECIAL;
   buf->decoder_info[1] = BUF_SPECIAL_CHARSET_ENCODING;
-  buf->decoder_info_ptr[2] = "utf-8";
-  buf->decoder_info[2] = strlen(buf->decoder_info_ptr[2]);
+  buf->decoder_info_ptr[2] = this->class->literal_utf_8;
+  buf->decoder_info[2] = LITERAL_UTF_8_LEN;
 
   val = (uint32_t *)buf->content;
   *val++ = data_pts / 90;                    /* start time */
@@ -1235,7 +1349,8 @@ static void fill_extra_data(matroska_track_t *track, uint32_t fourcc) {
   bih->biCompression = fourcc;
   bih->biWidth = track->video_track->pixel_width;
   bih->biHeight = track->video_track->pixel_height;
-  _x_bmiheader_le2me(bih);
+  /* this is to be passed to decoder in native machine endian */
+  /* (no _x_bmiheader_le2me(bih);) */
 
   /* add bih extra data */
   memcpy(bih + 1, track->codec_private, track->codec_private_len);
@@ -1246,9 +1361,10 @@ static void fill_extra_data(matroska_track_t *track, uint32_t fourcc) {
 
 static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
   ebml_parser_t *ebml = this->ebml;
-  int next_level = 3;
+  int this_level = ebml->level;
+  int next_level = this_level;
 
-  while (next_level == 3) {
+  while (next_level == this_level) {
     ebml_elem_t elem;
 
     if (!ebml_read_elem_head(ebml, &elem))
@@ -1462,6 +1578,7 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       fill_extra_data(track, ME_FOURCC('v', 'p', '9', '0'));
       track->buf_type = BUF_VIDEO_VP9;
       init_codec = init_codec_video;
+      track->handle_content = handle_vp9;
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_HEVC)) {
       lprintf("MATROSKA_CODEC_ID_V_HEVC\n");
       fill_extra_data(track, ME_FOURCC('h', 'e', 'v', 'c'));
@@ -1660,9 +1777,10 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
 
 static int parse_tracks(demux_matroska_t *this) {
   ebml_parser_t *ebml = this->ebml;
-  int next_level = 2;
+  int this_level = ebml->level;
+  int next_level = this_level;
 
-  while (next_level == 2) {
+  while (next_level == this_level) {
     ebml_elem_t elem;
 
     if (!ebml_read_elem_head(ebml, &elem))
@@ -2009,7 +2127,7 @@ static int parse_block (demux_matroska_t *this, size_t block_size,
 
   /*gap = flags & 1;*/
   lacing = (flags >> 1) & 0x3;
-/*fprintf(stderr, "lacing: %x\n", lacing);*/
+  lprintf("lacing: %x\n", lacing);
 
   if (!find_track_by_id(this, (int)track_num, &track)) {
      xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
@@ -2053,8 +2171,11 @@ static int parse_block (demux_matroska_t *this, size_t block_size,
     size_t block_size_left;
     lprintf("no lacing\n");
 
+    if (is_key)
+      decoder_flags |= BUF_FLAG_KEYFRAME;
+
     block_size_left = (this->block_data + block_size + this->compress_maxlen) - data;
-    lprintf("size: %d, block_size: %u, block_offset: %u\n", block_size_left, block_size, this->compress_maxlen);
+    lprintf("size: %zu, block_size: %zu, block_offset: %zu\n", block_size_left, block_size, this->compress_maxlen);
 
     if (headers_len) {
       data -= headers_len;
@@ -2063,17 +2184,13 @@ static int parse_block (demux_matroska_t *this, size_t block_size,
     }
 
     if (track->handle_content != NULL) {
-      track->handle_content((demux_plugin_t *)this, track,
-                             decoder_flags,
-                             data, block_size_left,
-                             pts, xduration,
-                             normpos, pts / 90);
+      track->handle_content ((demux_plugin_t *)this, track, decoder_flags,
+        data, block_size_left, pts, xduration, normpos, pts / 90);
     } else {
-      _x_demux_send_data(track->fifo, data, block_size_left,
-                         pts, track->buf_type, decoder_flags,
-                         normpos, pts / 90,
-                         this->duration, 0);
+      _x_demux_send_data (track->fifo, data, block_size_left, pts, track->buf_type,
+        decoder_flags, normpos, pts / 90, this->duration, 0);
     }
+
   } else {
 
     size_t block_size_left;
@@ -2145,13 +2262,13 @@ static int parse_block (demux_matroska_t *this, size_t block_size,
         }
         if (first_frame_size > INT_MAX) {
           xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-                  "demux_matroska: invalid first frame size (%" PRId64 ")\n",
+                  "demux_matroska: invalid first frame size (%" PRIu64 ")\n",
                   first_frame_size);
           return 0;
         }
         data += num_len; block_size_left -= num_len;
         frame[0] = (int) first_frame_size;
-        lprintf("first frame len: %d\n", frame[0]);
+        lprintf("first frame len: %zu\n", frame[0]);
         block_size_left -= frame[0];
 
         for (i = 1; i < lace_num; i++) {
@@ -2626,7 +2743,7 @@ static int parse_top_level_head(demux_matroska_t *this, int *next_level) {
           return 0;
         break;
       case MATROSKA_ID_CLUSTER:
-        lprintf("Cluster\n");
+        lprintf("Slipping Cluster\n");
         if (!ebml_skip(ebml, &elem))
           return 0;
         ret_value = 2;
@@ -2683,23 +2800,23 @@ static int parse_top_level(demux_matroska_t *this, int *next_level) {
 
   switch (elem.id) {
     case MATROSKA_ID_SEEKHEAD:
-      lprintf("SeekHead\n");
+      lprintf("Skipping SeekHead\n");
       if (!ebml_skip(ebml, &elem))
         return 0;
       this->has_seekhead = 1;
       break;
     case MATROSKA_ID_INFO:
-      lprintf("Info\n");
+      lprintf("Skipping Info\n");
       if (!ebml_skip(ebml, &elem))
         return 0;
       break;
     case MATROSKA_ID_TRACKS:
-      lprintf("Tracks\n");
+      lprintf("Skipping Tracks\n");
       if (!ebml_skip(ebml, &elem))
         return 0;
       break;
     case MATROSKA_ID_CHAPTERS:
-      lprintf("Chapters\n");
+      lprintf("Skipping Chapters\n");
       if (!ebml_skip(ebml, &elem))
         return 0;
       break;
@@ -2711,17 +2828,17 @@ static int parse_top_level(demux_matroska_t *this, int *next_level) {
         return 0;
       break;
     case MATROSKA_ID_CUES:
-      lprintf("Cues\n");
+      lprintf("Skipping Cues\n");
       if (!ebml_skip(ebml, &elem))
         return 0;
       break;
     case MATROSKA_ID_ATTACHMENTS:
-      lprintf("Attachments\n");
+      lprintf("Skipping Attachments\n");
       if (!ebml_skip(ebml, &elem))
         return 0;
       break;
     case MATROSKA_ID_TAGS:
-      lprintf("Tags\n");
+      lprintf("Skipping Tags\n");
       if (!ebml_skip(ebml, &elem))
         return 0;
       break;
@@ -2733,6 +2850,8 @@ static int parse_top_level(demux_matroska_t *this, int *next_level) {
   }
   if (next_level)
     *next_level = ebml_get_next_level(ebml, &elem);
+
+  lprintf("parse_top_level() done\n");
   return 1;
 }
 
@@ -2812,7 +2931,12 @@ static void demux_matroska_send_headers (demux_plugin_t *this_gen) {
    */
 
   /* enter in the segment */
-  ebml_read_master (this->ebml, &this->segment);
+  if (!ebml_read_master (this->ebml, &this->segment)) {
+    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+            "demux_matroska: failed to read file header\n");
+    this->status = DEMUX_FINISHED;
+    return;
+  }
 
   /* seek back to the beginning of the segment */
   next_level = 1;
@@ -3112,6 +3236,8 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
       return NULL;
     input->seek(input, 0, SEEK_SET);
     ebml = new_ebml_parser(stream->xine, input);
+    if (!ebml)
+      goto error;
     if (!ebml_check_header(ebml))
       goto error;
   }
@@ -3126,6 +3252,10 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   }
 
   this = calloc(1, sizeof(demux_matroska_t));
+  if (!this)
+    goto error;
+
+  this->class = (demux_matroska_class_t *)class_gen;
 
   this->demux_plugin.send_headers      = demux_matroska_send_headers;
   this->demux_plugin.send_chunk        = demux_matroska_send_chunk;
@@ -3143,6 +3273,8 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   if (!ebml) {
     ebml = new_ebml_parser(stream->xine, input);
+    if (!ebml)
+      goto error;
     if (!ebml_check_header(ebml))
       goto error;
   }
@@ -3181,6 +3313,9 @@ void *demux_matroska_init_class (xine_t *xine, void *data) {
   demux_matroska_class_t     *this;
 
   this         = calloc(1, sizeof(demux_matroska_class_t));
+  if (!this)
+    return NULL;
+
   this->xine   = xine;
 
   this->demux_class.open_plugin     = open_plugin;
@@ -3193,6 +3328,8 @@ void *demux_matroska_init_class (xine_t *xine, void *data) {
   this->demux_class.extensions      = "mkv wbm webm";
   this->demux_class.dispose         = default_demux_class_dispose;
 
+  memcpy (this->literal_utf_8, "utf-8", LITERAL_UTF_8_LEN + 1);
+
   return this;
 }
 
