@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -158,17 +158,6 @@ typedef struct {
 
 } demux_mpgaudio_t ;
 
-/* demuxer class struct */
-typedef struct {
-
-  demux_class_t     demux_class;
-
-  /* class-wide, global variables here */
-
-  xine_t           *xine;
-
-} demux_mpgaudio_class_t;
-
 /*
  * Parse a mp3 frame
  * return 1 on success
@@ -438,6 +427,16 @@ exit_error:
  * Parse a Vbri header
  * return the Vbri header or NULL on error
  */
+
+static void _free_vbri_header(vbri_header_t **pp)
+{
+  if (*pp) {
+    vbri_header_t *p = *pp;
+    _x_freep(&p->toc);
+    _x_freep(pp);
+  }
+}
+
 static vbri_header_t *XINE_MALLOC parse_vbri_header(mpg_audio_frame_t *frame,
 						    uint8_t *buf, int bufsize) {
 
@@ -445,19 +444,23 @@ static vbri_header_t *XINE_MALLOC parse_vbri_header(mpg_audio_frame_t *frame,
   uint8_t *ptr = buf;
   vbri_header_t *vbri;
 
+  ptr += (32 + 4);
+
+  if ((ptr + 4) >= (buf + bufsize)) return 0;
+
   vbri = calloc(1, sizeof(vbri_header_t));
   if (!vbri)
     return NULL;
 
-  ptr += (32 + 4);
-
-  if ((ptr + 4) >= (buf + bufsize)) return 0;
   lprintf("Checking %08X\n", *ptr);
   if (_X_BE_32(ptr) == VBRI_TAG) {
     lprintf("found Vbri header\n");
     ptr += 4;
 
-    if ((ptr + 22) >= (buf + bufsize)) return 0;
+    if ((ptr + 22) >= (buf + bufsize)) {
+      free(vbri);
+      return NULL;
+    }
     vbri->version           = _X_BE_16(ptr); ptr += 2;
     vbri->delai             = _X_BE_16(ptr); ptr += 2;
     vbri->quality           = _X_BE_16(ptr); ptr += 2;
@@ -477,7 +480,10 @@ static vbri_header_t *XINE_MALLOC parse_vbri_header(mpg_audio_frame_t *frame,
     lprintf("entry_size: %d\n", vbri->entry_size);
     lprintf("entry_frames: %d\n", vbri->entry_frames);
 
-    if ((ptr + (vbri->toc_entries + 1) * vbri->entry_size) >= (buf + bufsize)) return 0;
+    if ((ptr + (vbri->toc_entries + 1) * vbri->entry_size) >= (buf + bufsize)) {
+      free (vbri);
+      return NULL;
+    }
     vbri->toc = xine_xcalloc ((vbri->toc_entries + 1), sizeof(int));
     if (!vbri->toc) {
       free (vbri);
@@ -605,6 +611,7 @@ static int parse_frame_payload(demux_mpgaudio_t *this,
   if (this->check_vbr_header) {
     this->check_vbr_header = 0;
     this->mpg_frame_start = frame_pos;
+    free(this->xing_header);
     this->xing_header = parse_xing_header(&this->cur_frame, buf->content, this->cur_frame.size);
     if (this->xing_header) {
       buf->free_buffer(buf);
@@ -612,6 +619,7 @@ static int parse_frame_payload(demux_mpgaudio_t *this,
               LOG_MODULE ": found Xing header at offset %"PRId64"\n", frame_pos);
       return 1;
     }
+    _free_vbri_header(&this->vbri_header);
     this->vbri_header = parse_vbri_header(&this->cur_frame, buf->content, this->cur_frame.size);
     if (this->vbri_header) {
       buf->free_buffer(buf);
@@ -1160,6 +1168,15 @@ static int demux_mpgaudio_get_optional_data(demux_plugin_t *this_gen,
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
+static void demux_mpgaudio_dispose (demux_plugin_t *this_gen) {
+
+  demux_mpgaudio_t *this = (demux_mpgaudio_t *) this_gen;
+
+  _free_vbri_header(&this->vbri_header);
+  _x_freep(&this->xing_header);
+  free(this);
+}
+
 static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *stream,
                                     input_plugin_t *input) {
 
@@ -1186,11 +1203,13 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   }
 
   this = calloc(1, sizeof(demux_mpgaudio_t));
+  if (!this)
+    return NULL;
 
   this->demux_plugin.send_headers      = demux_mpgaudio_send_headers;
   this->demux_plugin.send_chunk        = demux_mpgaudio_send_chunk;
   this->demux_plugin.seek              = demux_mpgaudio_seek;
-  this->demux_plugin.dispose           = default_demux_plugin_dispose;
+  this->demux_plugin.dispose           = demux_mpgaudio_dispose;
   this->demux_plugin.get_status        = demux_mpgaudio_get_status;
   this->demux_plugin.get_stream_length = demux_mpgaudio_get_stream_length;
   this->demux_plugin.get_capabilities  = demux_mpgaudio_get_capabilities;
@@ -1215,16 +1234,17 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
  */
 void *demux_mpgaudio_init_class (xine_t *xine, void *data) {
 
-  demux_mpgaudio_class_t     *this;
+  demux_class_t     *this;
 
-  this         = calloc(1, sizeof(demux_mpgaudio_class_t));
-  this->xine   = xine;
+  this         = calloc(1, sizeof(demux_class_t));
+  if (!this)
+    return NULL;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("MPEG audio demux plugin");
-  this->demux_class.identifier      = "MPEGAUDIO";
-  if( _x_decoder_available(this->xine, BUF_AUDIO_MPEG) ) {
-    this->demux_class.mimetypes =
+  this->open_plugin     = open_plugin;
+  this->description     = N_("MPEG audio demux plugin");
+  this->identifier      = "MPEGAUDIO";
+  if( _x_decoder_available(xine, BUF_AUDIO_MPEG) ) {
+    this->mimetypes =
       "audio/mpeg2: mp2: MPEG audio;"
       "audio/x-mpeg2: mp2: MPEG audio;"
       "audio/mpeg3: mp3: MPEG audio;"
@@ -1235,12 +1255,12 @@ void *demux_mpgaudio_init_class (xine_t *xine, void *data) {
       "audio/mpegurl: mp3: MPEG audio;"
       "audio/mp3: mp3: MPEG audio;"
       "audio/x-mp3: mp3: MPEG audio;";
-    this->demux_class.extensions    = "mp3 mp2 mpa mpega";
+    this->extensions    = "mp3 mp2 mpa mpega";
   } else {
-    this->demux_class.mimetypes     = NULL;
-    this->demux_class.extensions    = NULL;
+    this->mimetypes     = NULL;
+    this->extensions    = NULL;
   }
-  this->demux_class.dispose         = default_demux_class_dispose;
+  this->dispose         = default_demux_class_dispose;
 
   return this;
 }
