@@ -146,7 +146,13 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <string.h>
-#include <arpa/inet.h>
+
+#ifdef HAVE_ARPA_INET_H
+#include <arpa/inet.h>  /* htonl */
+#endif
+#ifdef _WIN32
+#include <winsock.h>    /* htonl */
+#endif
 
 #define LOG_MODULE "demux_ts"
 #define LOG_VERBOSE
@@ -755,10 +761,10 @@ static void demux_ts_update_spu_channel(demux_ts_t *this)
 #endif
     }
 
- if ((this->media[this->spu_media].type & BUF_MAJOR_MASK) == BUF_SPU_HDMV) {
-   buf->type = BUF_SPU_HDMV;
-   buf->type |= this->current_spu_channel;
- }
+  if ((this->media[this->spu_media].type & (BUF_MAJOR_MASK | BUF_DECODER_MASK)) == BUF_SPU_HDMV) {
+    buf->type = BUF_SPU_HDMV;
+    buf->type |= this->current_spu_channel;
+  }
 
   this->video_fifo->put(this->video_fifo, buf);
 }
@@ -1344,16 +1350,23 @@ static void demux_ts_buffer_pes(demux_ts_t*this, unsigned char *ts,
   }
 
   if (!m->corrupted_pes) {
+    int room = m->buf->max_size - m->buf->size;
 
-    if ((m->buf->size + len) > m->buf->max_size) {
-      m->pes_bytes_left -= m->buf->size;
-      demux_ts_send_buffer(m, 0);
-      m->buf = m->fifo->buffer_pool_alloc(m->fifo);
+    if (len > room) {
+      if (room > 0)
+        memcpy (m->buf->mem + m->buf->size, ts, room);
+      m->pes_bytes_left -= m->buf->max_size;
+      m->buf->size = m->buf->max_size;
+      demux_ts_send_buffer (m, 0);
+      m->buf = m->fifo->buffer_pool_alloc (m->fifo);
+      m->buf->decoder_flags = BUF_FLAG_MERGE;
+      memcpy (m->buf->mem, ts + room, len - room);
+      m->buf->size = len - room;
+    } else {
+      memcpy (m->buf->mem + m->buf->size, ts, len);
+      m->buf->size += len;
     }
 
-    memcpy(m->buf->mem + m->buf->size, ts, len);
-    m->buf->size += len;
-
     if (m->pes_bytes_left > 0 && m->buf->size >= m->pes_bytes_left) {
       /* PES payload complete */
       m->pes_bytes_left -= m->buf->size;
@@ -2548,12 +2561,13 @@ static int demux_ts_seek (demux_plugin_t *this_gen,
 static int demux_ts_get_stream_length (demux_plugin_t *this_gen) {
 
   demux_ts_t*this = (demux_ts_t*)this_gen;
+  int rate = this->rate;
 
-  if (this->rate)
+  if (rate)
     return (int)((int64_t) this->input->get_length (this->input)
-                 * 1000 / this->rate);
-  else
-    return 0;
+                 * 1000 / rate);
+
+  return 0;
 }
 
 
