@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2009 the xine project
+ * Copyright (C) 2008-2016 the xine project
  * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
  *
  * This file is part of xine, a free video player.
@@ -55,6 +55,8 @@
 #define LOG_MODULE "demux_vc1es"
 #define LOG_VERBOSE
 
+#include "group_video.h"
+
 #include <xine/xine_internal.h>
 #include <xine/xineutils.h>
 #include <xine/compat.h>
@@ -373,7 +375,7 @@ static demux_plugin_t *open_plugin( demux_class_t *class_gen, xine_stream_t *str
 
 
 
-static void *init_plugin( xine_t *xine, void *data )
+void *demux_vc1es_init_class ( xine_t *xine, void *data )
 {
   demux_vc1_es_class_t     *this;
 
@@ -389,18 +391,3 @@ static void *init_plugin( xine_t *xine, void *data )
   return this;
 }
 
-
-/*
- * exported plugin catalog entry
- */
-static const demuxer_info_t demux_info_vc1es = {
-  0                       /* priority */
-};
-
-
-
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 27, "vc1es", XINE_VERSION_CODE, &demux_info_vc1es, init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
