@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2011 the xine project
+ * Copyright (C) 2004-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -161,7 +161,7 @@ static int open_vmd_file(demux_vmd_t *this) {
   this->frame_count *= 2;
 
   raw_frame_table_size = this->frame_count * BYTES_PER_FRAME_RECORD;
-  raw_frame_table = xine_xmalloc(raw_frame_table_size);
+  raw_frame_table = malloc(raw_frame_table_size);
   if (this->input->read(this->input, raw_frame_table, raw_frame_table_size) !=
     raw_frame_table_size) {
     free(raw_frame_table);
@@ -416,6 +416,10 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   demux_vmd_t    *this;
 
   this         = calloc(1, sizeof(demux_vmd_t));
+  if (!this) {
+    return NULL;
+  }
+
   this->stream = stream;
   this->input  = input;
 
@@ -456,6 +460,9 @@ void *demux_vmd_init_plugin (xine_t *xine, void *data) {
   demux_vmd_class_t     *this;
 
   this = calloc(1, sizeof(demux_vmd_class_t));
+  if (!this) {
+    return NULL;
+  }
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.description     = N_("Sierra VMD file demux plugin");
