@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2004 the xine project
+ * Copyright (C) 2003-2016 the xine project
  * Copyright (C) 2003 Jeroen Asselman <j.asselman@itsec-ps.nl>
  *
  * This file is part of xine, a free video player.
@@ -39,6 +39,8 @@
 #define LOG
 */
 
+#include "group_video.h"
+
 #include <xine/xine_internal.h>
 #include <xine/xineutils.h>
 #include <xine/demux.h>
@@ -76,7 +78,7 @@ static int switch_buf(demux_yuv_frames_t *this , buf_element_t *buf){
   if (this->seek_flag) {
     this->seek_flag = 0;
     _x_demux_control_newpts(this->stream, buf->pts, BUF_FLAG_SEEK);
-  } else if (abs(this->last_pts - buf->pts) > WRAP_THRESHOLD) {
+  } else if (llabs(this->last_pts - buf->pts) > WRAP_THRESHOLD) {
     _x_demux_control_newpts(this->stream, buf->pts, 0);
   }
 
@@ -224,7 +226,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
 /*
  * demuxer class
  */
-static void *init_class (xine_t *xine, void *data) {
+void *demux_yuv_frames_init_class (xine_t *xine, void *data) {
   demux_yuv_frames_class_t     *this;
 
   this = calloc(1, sizeof(demux_yuv_frames_class_t));
@@ -239,19 +241,8 @@ static void *init_class (xine_t *xine, void *data) {
   return this;
 }
 
-/*
- * exported plugin catalog entry
- */
-static const demuxer_info_t demux_info_yuv_frames = {
-  0                       /* priority */
-};
-
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 27, "yuv_frames", XINE_VERSION_CODE, &demux_info_yuv_frames, init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
-
 /*
  * vim:sw=3:sts=3:
  */
+
+
