@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2016 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -35,6 +35,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <pthread.h>
 #include <fcntl.h>
 #include <errno.h>
 
@@ -231,7 +232,7 @@ static spu_decoder_t *dxr3_spudec_open_plugin(spu_decoder_class_t *class_gen, xi
     CONF_KEY, 0, CONF_NAME, CONF_HELP, 10, NULL, NULL);
 
   pthread_mutex_lock(&this->dxr3_vo->spu_device_lock);
-  if (this->dxr3_vo->fd_spu)
+  if (this->dxr3_vo->fd_spu >= 0)
     this->fd_spu = this->dxr3_vo->fd_spu;
   else {
     /* open dxr3 spu device */
@@ -376,7 +377,10 @@ static void dxr3_spudec_decode_data(spu_decoder_t *this_gen, buf_element_t *buf)
 	this->button_filter = 1;
 	pthread_mutex_lock(&this->dxr3_vo->spu_device_lock);
         dxr3_spu_button(this->fd_spu, NULL);
-        write(this->fd_spu, empty_spu, sizeof(empty_spu));
+        if (write(this->fd_spu, empty_spu, sizeof(empty_spu)) != sizeof(empty_spu)) {
+          xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+                  _("dxr3_decode_spu: spu device write failed (%s)\n"), strerror(errno));
+        }
 	pthread_mutex_unlock(&this->dxr3_vo->spu_device_lock);
       }
     }
@@ -557,10 +561,13 @@ static void dxr3_spudec_dispose(spu_decoder_t *this_gen)
   pthread_mutex_lock(&this->dxr3_vo->spu_device_lock);
   /* clear any remaining spu */
   dxr3_spu_button(this->fd_spu, NULL);
-  write(this->fd_spu, empty_spu, sizeof(empty_spu));
+  if (write(this->fd_spu, empty_spu, sizeof(empty_spu)) != sizeof(empty_spu)) {
+    xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+            _("dxr3_decode_spu: spu device write failed (%s)\n"), strerror(errno));
+  }
   close(this->fd_spu);
-  this->fd_spu = 0;
-  this->dxr3_vo->fd_spu = 0;
+  this->fd_spu = -1;
+  this->dxr3_vo->fd_spu = -1;
   pthread_mutex_unlock(&this->dxr3_vo->spu_device_lock);
 
   dxr3_spudec_clear_nav_list(this);
