@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2016 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -201,13 +201,15 @@ static int lavc_on_update_format(dxr3_driver_t *drv, dxr3_frame_t *frame)
     10, NULL, NULL);
   }
 
-  lprintf("lavc -> bitrate %d  \n", this->context->bit_rate);
+  lprintf("lavc -> bitrate %" PRId64 "  \n", (int64_t)this->context->bit_rate);
 
   this->context->width  = frame->vo_frame.pitches[0];
   this->context->height = frame->oheight;
 
   this->context->gop_size = 0; /*intra frames only */
+#if defined(LIBAVCODEC_VERSION_MAJOR) && LIBAVCODEC_VERSION_MAJOR < 58
   this->context->me_method = ME_ZERO; /*motion estimation type*/
+#endif
 
   this->context->time_base.den = 90000;
   if (frame->vo_frame.duration > 90000 / 24)
