@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -40,6 +40,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <pthread.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <math.h>
@@ -215,6 +216,7 @@ static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const v
   int encoder, confnum;
   static const char *available_encoders[SUPPORTED_ENCODER_COUNT + 2];
   plugin_node_t *node = NULL; /* unused inside dxr3_lavc_init () anyway... */
+  int fd_control;
 
   static const char *const videoout_modes[] = {
     "letterboxed tv", "widescreen tv",
@@ -230,8 +232,20 @@ static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const v
   */
   if (class->instance) return NULL;
 
+  snprintf(tmpstr, sizeof(tmpstr), "/dev/em8300-%d", class->devnum);
+  llprintf(LOG_VID, "Entering video init, devname = %s.\n", tmpstr);
+  if ((fd_control = xine_open_cloexec(tmpstr, O_WRONLY)) < 0) {
+    xprintf(class->xine, XINE_VERBOSITY_LOG,
+            _("video_out_dxr3: Failed to open control device %s (%s)\n"), tmpstr, strerror(errno));
+    return NULL;
+  }
+
+
   this = calloc(1, sizeof(dxr3_driver_t));
-  if (!this) return NULL;
+  if (!this) {
+    close(fd_control);
+    return NULL;
+  }
 
   this->vo_driver.get_capabilities     = dxr3_get_capabilities;
   this->vo_driver.alloc_frame          = dxr3_alloc_frame;
@@ -253,6 +267,9 @@ static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const v
   _x_vo_scale_init(&this->scale, 0, 0, config);
   _x_alphablend_init(&this->alphablend_extra_data, class->xine);
 
+  this->fd_control = fd_control;
+  this->fd_spu = -1;
+
   this->class                          = class;
   this->swap_fields                    = config->register_bool(config,
     "dxr3.encoding.swap_fields", 0, _("swap odd and even lines"),
@@ -270,21 +287,12 @@ static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const v
     _("Enabling this option will utilise a smoother play mode for non-MPEG content."),
     20, dxr3_update_enhanced_mode, this);
 
-  snprintf(tmpstr, sizeof(tmpstr), "/dev/em8300-%d", class->devnum);
-  llprintf(LOG_VID, "Entering video init, devname = %s.\n", tmpstr);
-  if ((this->fd_control = xine_open_cloexec(tmpstr, O_WRONLY)) < 0) {
-
-    xprintf(this->class->xine, XINE_VERBOSITY_LOG,
-	    _("video_out_dxr3: Failed to open control device %s (%s)\n"), tmpstr, strerror(errno));
-
-    return 0;
-  }
-
   snprintf (tmpstr, sizeof(tmpstr), "/dev/em8300_mv-%d", class->devnum);
   if ((this->fd_video = xine_open_cloexec(tmpstr, O_WRONLY | O_SYNC )) < 0) {
     xprintf(this->class->xine, XINE_VERBOSITY_LOG,
 	    _("video_out_dxr3: Failed to open video device %s (%s)\n"), tmpstr, strerror(errno));
-    return 0;
+    dxr3_dispose(&this->vo_driver);
+    return NULL;
   }
   /* close now and and let the decoder/encoder reopen if they want */
   close(this->fd_video);
@@ -329,20 +337,23 @@ static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const v
     if ((strcmp(available_encoders[encoder], "libavcodec") == 0) && !dxr3_lavc_init(this, node)) {
       xprintf(this->class->xine, XINE_VERBOSITY_LOG,
 	      _("video_out_dxr3: Mpeg encoder libavcodec failed to init.\n"));
-      return 0;
+      dxr3_dispose(&this->vo_driver);
+      return NULL;
     }
 #ifdef HAVE_LIBRTE
     if ((strcmp(available_encoders[encoder], "rte") == 0) && !dxr3_rte_init(this)) {
       xprintf(this->class->xine, XINE_VERBOSITY_LOG,
 	      _("video_out_dxr3: Mpeg encoder rte failed to init.\n"));
-      return 0;
+      dxr3_dispose(&this->vo_driver);
+      return NULL;
     }
 #endif
 #ifdef HAVE_LIBFAME
     if ((strcmp(available_encoders[encoder], "fame") == 0) && !dxr3_fame_init(this)) {
       xprintf(this->class->xine, XINE_VERBOSITY_LOG,
 	      _("video_out_dxr3: Mpeg encoder fame failed to init.\n"));
-      return 0;
+      dxr3_dispose(&this->vo_driver);
+      return NULL;
     }
 #endif
     if (strcmp(available_encoders[encoder], "none") == 0)
@@ -831,7 +842,7 @@ static void dxr3_overlay_end(vo_driver_t *this_gen, vo_frame_t *frame_gen)
   pthread_mutex_lock(&this->spu_device_lock);
 
   /* try to open the dxr3 spu device */
-  if (!this->fd_spu) {
+  if (this->fd_spu < 0) {
     snprintf (tmpstr, sizeof(tmpstr), "/dev/em8300_sp-%d", this->class->devnum);
     if ((this->fd_spu = xine_open_cloexec(tmpstr, O_WRONLY)) < 0) {
       xprintf(this->class->xine, XINE_VERBOSITY_DEBUG,
@@ -851,7 +862,10 @@ static void dxr3_overlay_end(vo_driver_t *this_gen, vo_frame_t *frame_gen)
       0x00, 0x01, 0x00, 0x20, 0x02, 0xFF };
     /* just clear any previous spu */
     dxr3_spu_button(this->fd_spu, NULL);
-    write(this->fd_spu, empty_spu, sizeof(empty_spu));
+    if (write(this->fd_spu, empty_spu, sizeof(empty_spu)) != sizeof(empty_spu)) {
+      xprintf(this->class->xine, XINE_VERBOSITY_DEBUG,
+              "video_out_dxr3: spu device write failed (%s)\n", strerror(errno));
+    }
     pthread_mutex_unlock(&this->spu_device_lock);
     return;
   }
@@ -1220,9 +1234,10 @@ static void dxr3_dispose(vo_driver_t *this_gen)
     this->enc->on_close(this);
   if(this->overlay_enabled)
     ioctl(this->fd_control, EM8300_IOCTL_OVERLAY_SETMODE, &val);
-  close(this->fd_control);
+  if (this->fd_control >= 0)
+    close(this->fd_control);
   pthread_mutex_lock(&this->spu_device_lock);
-  if (this->fd_spu) {
+  if (this->fd_spu >= 0) {
     static const uint8_t empty_spu[] = {
       0x00, 0x26, 0x00, 0x08, 0x80, 0x00, 0x00, 0x80,
       0x00, 0x00, 0x00, 0x20, 0x01, 0x03, 0x00, 0x00,
@@ -1231,7 +1246,10 @@ static void dxr3_dispose(vo_driver_t *this_gen)
       0x00, 0x01, 0x00, 0x20, 0x02, 0xFF };
     /* clear any remaining spu */
     dxr3_spu_button(this->fd_spu, NULL);
-    write(this->fd_spu, empty_spu, sizeof(empty_spu));
+    if (write(this->fd_spu, empty_spu, sizeof(empty_spu)) != sizeof(empty_spu)) {
+      xprintf(this->class->xine, XINE_VERBOSITY_DEBUG,
+              "video_out_dxr3: spu device write failed (%s)\n", strerror(errno));
+    }
     close(this->fd_spu);
   }
   pthread_mutex_unlock(&this->spu_device_lock);
