@@ -96,10 +96,10 @@ typedef struct {
   int             xine_playlist_size;
 
   /* config */
-  char           *mountpoint;
-  char           *device;
-  char           *language;
-  char           *country;
+  const char     *mountpoint;
+  const char     *device;
+  const char     *language;
+  const char     *country;
   int             region;
   int             parental;
 } bluray_input_class_t;
@@ -324,13 +324,15 @@ static void draw_bitmap(xine_osd_t *osd, const BD_OVERLAY * const ov)
     size_t pixels = (size_t)ov->w * ov->h;
     uint8_t *img = malloc(pixels);
 
-    for (i = 0; i < pixels; i += rlep->len, rlep++) {
-      memset(img + i, rlep->color, rlep->len);
+    if (img) {
+      for (i = 0; i < pixels; i += rlep->len, rlep++) {
+        memset(img + i, rlep->color, rlep->len);
+      }
+
+      xine_osd_draw_bitmap(osd, img, ov->x, ov->y, ov->w, ov->h, NULL);
+
+      free(img);
     }
-
-    xine_osd_draw_bitmap(osd, img, ov->x, ov->y, ov->w, ov->h, NULL);
-
-    free(img);
   }
 }
 
@@ -1427,9 +1429,9 @@ static void bluray_plugin_dispose (input_plugin_t *this_gen)
   osd_buf_destroy(&this->osd_buf);
 #endif
 
-  free (this->mrl);
-  free (this->disc_root);
-  free (this->disc_name);
+  _x_freep (&this->mrl);
+  _x_freep (&this->disc_root);
+  _x_freep (&this->disc_name);
 
   free (this);
 }
@@ -1819,10 +1821,9 @@ static void free_xine_playlist(bluray_input_class_t *this)
     int i;
     for (i = 0; i < this->xine_playlist_size; i++) {
       MRL_ZERO(this->xine_playlist[i]);
-      free(this->xine_playlist[i]);
+      _x_freep(&this->xine_playlist[i]);
     }
-    free(this->xine_playlist);
-    this->xine_playlist = NULL;
+    _x_freep (&this->xine_playlist);
   }
 
   this->xine_playlist_size = 0;
@@ -1931,10 +1932,10 @@ static void *bluray_init_plugin (xine_t *xine, void *data)
                               0, mountpoint_change_cb, (void *) this);
   this->device =
     config->register_filename(config, "media.bluray.device",
-                              BLURAY_PATH, XINE_CONFIG_STRING_IS_DIRECTORY_NAME,
+                              BLURAY_PATH, XINE_CONFIG_STRING_IS_DEVICE_NAME,
                               _("device used for BluRay playback"),
                               _("The path to the device "
-                                "which you intend to use for playing BluRy discs."),
+                                "which you intend to use for playing BluRay discs."),
                               0, device_change_cb, (void *) this);
 
   /* Player settings */
@@ -2010,7 +2011,7 @@ static void *bd_init_plugin (xine_t *xine, void *data)
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, INPUT_PLUGIN_IFACE_VERSION, "BLURAY", XINE_VERSION_CODE, NULL, bluray_init_plugin },
-  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, INPUT_PLUGIN_IFACE_VERSION, "BD",     XINE_VERSION_CODE, NULL, bd_init_plugin },
+  { PLUGIN_INPUT, INPUT_PLUGIN_IFACE_VERSION, "BLURAY", XINE_VERSION_CODE, NULL, bluray_init_plugin },
+  { PLUGIN_INPUT, INPUT_PLUGIN_IFACE_VERSION, "BD",     XINE_VERSION_CODE, NULL, bd_init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
