@@ -45,17 +45,24 @@
 
 #ifdef HAVE_SYS_IOCTL_H
 #  include <sys/ioctl.h>
-#else
-/* for WIN32 */
+#endif
+#ifdef WIN32
+#  include <windows.h>
 #  include <windef.h>
 #  include <winioctl.h>
 #endif
 
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
 
 #include <signal.h>
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
 
 #include <basedir.h>
 
@@ -302,8 +309,8 @@ typedef struct {
   int                  cache_last;
 
 #ifdef WIN32
-    HANDLE h_device_handle;                         /* vcd device descriptor */
-  long  hASPI;
+  HANDLE  h_device_handle;   /* vcd device descriptor */
+  HMODULE hASPI;             /* wnaspi32.dll */
   short i_sid;
   long  (*lpSendCommand)( void* );
 #endif
@@ -1405,7 +1412,7 @@ static void _cdda_mkdir_safe(xine_t *xine, char *path) {
     WIN32_FIND_DATA FileData;
 
     // Get the proper directory path
-    sprintf(szDir, "%s\\*", path);
+    snprintf(szDir, sizeof(szDir), "%s\\*", path);
 
     // Get the first file
     hList = FindFirstFile(szDir, &FileData);
@@ -1785,7 +1792,7 @@ static int _cdda_cddb_retrieve(cdda_input_plugin_t *this) {
      * that most people don't like.
      */
     memset(&buffer, 0, sizeof(buffer));
-    sprintf(buffer, "cddb hello unknown localhost xine %s\n", VERSION);
+    snprintf(buffer, sizeof(buffer), "cddb hello unknown localhost xine %s\n", VERSION);
     if ((err = _cdda_cddb_send_command(this, buffer)) <= 0) {
       xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
 	      "input_cdda: error while sending cddb hello command.\n");
@@ -1805,7 +1812,7 @@ static int _cdda_cddb_retrieve(cdda_input_plugin_t *this) {
     /* For UTF-8 support - use protocol 6 */
 
     memset(&buffer, 0, sizeof(buffer));
-    sprintf(buffer, "proto %d\n",CDDB_PROTOCOL);
+    snprintf(buffer, sizeof(buffer), "proto %d\n", CDDB_PROTOCOL);
     if ((err = _cdda_cddb_send_command(this, buffer)) <= 0) {
       xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
 	      "input_cdda: error while sending cddb protocol command.\n");
@@ -1969,14 +1976,14 @@ static void _cdda_cdindex(cdda_input_plugin_t *this, cdrom_toc *toc) {
 
   sha160_init (&sha);
 
-  sprintf (temp, "%02X%02X%08X",
+  snprintf (temp, sizeof(temp), "%02X%02X%08X",
     toc->first_track,
     toc->last_track - toc->ignore_last_track,
     toc->leadout_track.first_frame); /* + 150 */
   sha160_update (&sha, temp, 12);
 
   for (i = toc->first_track; i <= toc->last_track - toc->ignore_last_track; i++) {
-    sprintf (temp, "%08X", toc->toc_entries[i - 1].first_frame);
+    snprintf (temp, sizeof(temp), "%08X", toc->toc_entries[i - 1].first_frame);
     sha160_update (&sha, temp, 8);
   }
 
@@ -2072,8 +2079,8 @@ static int cdda_open(cdda_input_plugin_t *this_gen,
     this_gen->fd = -1;
     this_gen->h_device_handle = NULL;
     this_gen->i_sid = 0;
-    this_gen->hASPI = 0;
-    this_gen->lpSendCommand = 0;
+    this_gen->hASPI = NULL;
+    this_gen->lpSendCommand = NULL;
   }
   else
       return -1;
@@ -2108,10 +2115,10 @@ static int cdda_open(cdda_input_plugin_t *this_gen,
       hASPI = LoadLibrary( "wnaspi32.dll" );
       if( hASPI != NULL )
 	{
-	  lpGetSupport = GetProcAddress( hASPI,
-						   "GetASPI32SupportInfo" );
-	  lpSendCommand = GetProcAddress( hASPI,
-						    "SendASPI32Command" );
+          lpGetSupport = (long (*)(void))GetProcAddress( hASPI,
+                                         "GetASPI32SupportInfo" );
+          lpSendCommand = (long (*)(void*))GetProcAddress( hASPI,
+                                         "SendASPI32Command" );
 	}
 
       if( hASPI == NULL || lpGetSupport == NULL || lpSendCommand == NULL )
@@ -2182,7 +2189,7 @@ static int cdda_open(cdda_input_plugin_t *this_gen,
                       ( srbGDEVBlock.SRB_DeviceType == DTYPE_CDROM ) )
 		    {
                       this_gen->i_sid = MAKEWORD( i, j );
-                      this_gen->hASPI = (long)hASPI;
+                      this_gen->hASPI = hASPI;
                       this_gen->lpSendCommand = lpSendCommand;
 
                       lprintf( "using aspi layer" );
@@ -2238,8 +2245,8 @@ static int cdda_close(cdda_input_plugin_t *this_gen) {
      CloseHandle( this_gen->h_device_handle );
   this_gen->h_device_handle = NULL;
   if( this_gen->hASPI )
-      FreeLibrary( (HMODULE)this_gen->hASPI );
-  this_gen->hASPI = (long)NULL;
+      FreeLibrary( this_gen->hASPI );
+  this_gen->hASPI = NULL;
 #endif /* WIN32 */
 
   return 0;
@@ -2597,6 +2604,7 @@ static const char * const * cdda_class_get_autoplay_list (input_class_t *this_ge
       lprintf("cdda_class_get_autoplay_list: opening >%s< failed %s\n",
               this->cdda_device, strerror(errno));
       if (ip != this->ip) free(ip);
+      free_cdrom_toc(toc);
       return NULL;
     }
 
@@ -2615,6 +2623,7 @@ static const char * const * cdda_class_get_autoplay_list (input_class_t *this_ge
 
   if ( err < 0 ) {
     if (ip != this->ip) free(ip);
+    free_cdrom_toc(toc);
     return NULL;
   }
 
