@@ -673,7 +673,7 @@ static xine_mrl_t **file_input_class_get_dir (input_class_t *this_gen, const cha
   if(strcasecmp(current_dir, "/"))
     snprintf(current_dir_slashed, sizeof(current_dir_slashed), "%s/", current_dir);
   else
-    sprintf(current_dir_slashed, "/");
+    strcpy(current_dir_slashed, "/");
 
   /*
    * Ooch!
@@ -943,6 +943,7 @@ static void file_input_class_dispose (input_class_t *this_gen) {
   config_values_t     *config = this->xine->config;
 
   config->unregister_callback(config, "media.files.origin_path");
+  config->unregister_callback(config, "media.files.show_hidden_files");
 
   while(this->mrls_allocated_entries) {
     MRL_ZERO(this->mrls[this->mrls_allocated_entries - 1]);
