@@ -29,14 +29,21 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <string.h>
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
-#include <ctype.h>
-
-#ifndef WIN32
+#endif
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
+#include <ctype.h>
 #include <errno.h>
-#endif /* WIN32 */
+
+#ifdef WIN32
+#include <winsock.h>
+#endif
 
 #include <sys/time.h>
 
@@ -50,6 +57,7 @@
 #include <xine/xineutils.h>
 #include <xine/input_plugin.h>
 #include "net_buf_ctrl.h"
+#include "group_network.h"
 #include "http_helper.h"
 
 #define BUFSIZE                 1024
@@ -117,14 +125,14 @@ typedef struct {
   xine_t           *xine;
   config_values_t  *config;
 
-  char             *proxyhost;
+  const char       *proxyhost;
   char             *proxyhost_env;
   int               proxyport;
   int               proxyport_env;
 
-  char             *proxyuser;
-  char             *proxypassword;
-  char             *noproxylist;
+  const char       *proxyuser;
+  const char       *proxypassword;
+  const char       *noproxylist;
 } http_input_class_t;
 
 static void proxy_host_change_cb (void *this_gen, xine_cfg_entry_t *cfg) {
@@ -606,6 +614,7 @@ static void http_plugin_dispose (input_plugin_t *this_gen ) {
   _x_freep (&this->user);
   _x_freep (&this->password);
   _x_freep (&this->uri);
+  _x_freep (&this->mime_type);
   free (this);
 }
 
@@ -1036,13 +1045,20 @@ static input_plugin_t *http_class_get_instance (input_class_t *cls_gen, xine_str
 
 static void http_class_dispose (input_class_t *this_gen) {
   http_input_class_t  *this = (http_input_class_t *) this_gen;
+  config_values_t     *config = this->config;
+
+  config->unregister_callback(config, "media.network.http_proxy_host");
+  config->unregister_callback(config, "media.network.http_proxy_port");
+  config->unregister_callback(config, "media.network.http_proxy_user");
+  config->unregister_callback(config, "media.network.http_proxy_password");
+  config->unregister_callback(config, "media.network.http_no_proxy");
 
   _x_freep(&this->proxyhost_env);
 
   free (this);
 }
 
-static void *init_class (xine_t *xine, void *data) {
+void *input_http_init_class (xine_t *xine, void *data) {
   http_input_class_t  *this;
   config_values_t     *config;
   char                *proxy_env;
@@ -1117,12 +1133,4 @@ static void *init_class (xine_t *xine, void *data) {
   return this;
 }
 
-/*
- * exported plugin catalog entry
- */
 
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 18, "http", XINE_VERSION_CODE, NULL, init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
