@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2016 the xine project
+ * Copyright (C) 2002-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -26,9 +26,15 @@
 
 #include <unistd.h>
 #include <stdio.h>
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -49,14 +55,6 @@
 #include "mmsh.h"
 #include "net_buf_ctrl.h"
 
-/*
- * not very nice but maybe still better than the previous solution
- * (rebuild and link demuxers/asfheader.c from here, and let newer
- * automake disagree about where to store asfheader.o) ...
- */
-#undef LOG_MODULE
-#include "demuxers/asfheader.c"
-#undef LOG_MODULE
 #define LOG_MODULE "input_mms"
 
 #define PROTOCOL_UNDEFINED 0
