@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2009 the xine project
+ * Copyright (C) 2002-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -26,9 +26,15 @@
 
 #include <unistd.h>
 #include <stdio.h>
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -47,16 +53,10 @@
 
 #include "pnm.h"
 #include "net_buf_ctrl.h"
+#include "group_network.h"
 
 #define BUFSIZE 4096
 
-typedef struct {
-
-  input_class_t     input_class;
-
-  xine_t           *xine;
-} pnm_input_class_t;
-
 typedef struct {
   input_plugin_t   input_plugin;
 
@@ -271,32 +271,20 @@ static input_plugin_t *pnm_class_get_instance (input_class_t *cls_gen, xine_stre
  * pnm input plugin class stuff
  */
 
-static void *init_class (xine_t *xine, void *data) {
+void *input_pnm_init_class (xine_t *xine, void *data) {
 
-  pnm_input_class_t  *this;
+  input_class_t  *this;
 
-  this = calloc(1, sizeof (pnm_input_class_t));
+  this = calloc(1, sizeof (input_class_t));
 
-  this->xine   = xine;
-
-  this->input_class.get_instance       = pnm_class_get_instance;
-  this->input_class.identifier         = "pnm";
-  this->input_class.description        = N_("pnm streaming input plugin");
-  this->input_class.get_dir            = NULL;
-  this->input_class.get_autoplay_list  = NULL;
-  this->input_class.dispose            = default_input_class_dispose;
-  this->input_class.eject_media        = NULL;
+  this->get_instance       = pnm_class_get_instance;
+  this->identifier         = "pnm";
+  this->description        = N_("pnm streaming input plugin");
+  this->get_dir            = NULL;
+  this->get_autoplay_list  = NULL;
+  this->dispose            = default_input_class_dispose;
+  this->eject_media        = NULL;
 
   return this;
 }
 
-/*
- * exported plugin catalog entry
- */
-
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_INPUT, 18, "pnm", XINE_VERSION_CODE, NULL, init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
-
