@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2011 the xine project
+ * Copyright (C) 2000-2017 the xine project
  * March 2003 - Miguel Freitas
  * This plugin was sponsored by 1Control
  *
@@ -169,10 +169,7 @@ typedef struct {
 
   input_class_t     input_class;
 
-  xine_t           *xine;
-  config_values_t  *config;
-
-  char             *devname;
+  const char       *devname;
 
 } pvr_input_class_t;
 
@@ -607,7 +604,10 @@ static int pvr_break_rec_page (pvr_input_plugin_t *this) {
       this->play_fd = -1;
     }
 
-    remove(filename);
+    if (remove(filename) < 0) {
+      xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+              "input_pvr: error removing pvr file (%s)\n", filename);
+    }
     free(filename);
   }
   return 1;
@@ -629,12 +629,12 @@ static int pvr_rec_file(pvr_input_plugin_t *this) {
       return 0;
   }
   pos = (off_t)(this->rec_blk - this->page_block[this->rec_page]) * PVR_BLOCK_SIZE;
-  if( lseek (this->rec_fd, pos, SEEK_SET) != pos ) {
-    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
-	    "input_pvr: error setting position for writing %" PRIdMAX "\n", (intmax_t)pos);
-    return 0;
-  }
   if( this->rec_fd != -1 ) {
+    if( lseek (this->rec_fd, pos, SEEK_SET) != pos ) {
+      xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+              "input_pvr: error setting position for writing %" PRIdMAX "\n", (intmax_t)pos);
+      return 0;
+    }
     if( write(this->rec_fd, this->data, PVR_BLOCK_SIZE) < PVR_BLOCK_SIZE ) {
       xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
 	      "input_pvr: short write to pvr file (out of disk space?)\n");
@@ -895,7 +895,10 @@ static void pvr_finish_recording (pvr_input_plugin_t *this) {
       if( this->save_page == (unsigned)-1 || i < this->save_page ) {
         lprintf("erasing old pvr file (%s)\n", src_filename);
 
-        remove(src_filename);
+        if (remove(src_filename) < 0) {
+          xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+                  "input_pvr: error removing pvr file (%s)\n", src_filename);
+        }
       } else {
 
         if( !this->save_name || !strlen(this->save_name) )
@@ -905,7 +908,10 @@ static void pvr_finish_recording (pvr_input_plugin_t *this) {
 
         lprintf("moving (%s) to (%s)\n", src_filename, dst_filename);
 
-        rename(src_filename,dst_filename);
+        if (rename(src_filename, dst_filename) < 0) {
+          xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+                  "input_pvr: error renaming pvr file (%s->%s)\n", src_filename, dst_filename);
+        }
         free(dst_filename);
       }
       free(src_filename);
@@ -1123,7 +1129,10 @@ static void pvr_event_handler (pvr_input_plugin_t *this) {
 
                 lprintf("moving (%s) to (%s)\n", src_filename, dst_filename);
 
-                rename(src_filename,dst_filename);
+                if (rename(src_filename, dst_filename) < 0) {
+                  xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+                          "input_pvr: error renaming pvr file (%s->%s)\n", src_filename, dst_filename);
+                }
                 free(dst_filename);
                 free(src_filename);
               }
@@ -1156,6 +1165,7 @@ static void pvr_event_handler (pvr_input_plugin_t *this) {
        close(this->dev_fd);
        this->dev_fd = xine_open_cloexec(this->class->devname, O_RDWR);
        if (this->dev_fd == -1) {
+         pthread_mutex_unlock(&this->dev_lock);
          xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
 		 _("input_pvr: error opening device %s\n"), this->class->devname );
          return;
@@ -1376,13 +1386,9 @@ static void pvr_plugin_dispose (input_plugin_t *this_gen ) {
 
   pvr_finish_recording(this);
 
-  free (this->mrl);
-
-  if (this->tmp_prefix)
-    free (this->tmp_prefix);
-
-  if (this->save_prefix)
-    free (this->save_prefix);
+  _x_freep (&this->mrl);
+  _x_freep (&this->tmp_prefix);
+  _x_freep (&this->save_prefix);
 
   ite = xine_list_front (this->saved_shows);
   while (ite) {
@@ -1401,6 +1407,9 @@ static int pvr_plugin_open (input_plugin_t *this_gen ) {
   int                  err;
   struct ivtv_ioctl_codec codec;
 
+  _x_assert(this->dev_fd == -1);
+  _x_assert(this->pvr_running == 0);
+
   this->session = 0;
   this->rec_fd = -1;
   this->play_fd = -1;
@@ -1437,12 +1446,14 @@ static int pvr_plugin_open (input_plugin_t *this_gen ) {
   }
 
   /* register our own scr provider */
+  _x_assert(this->scr == NULL);
   time = this->stream->xine->clock->get_current_time(this->stream->xine->clock);
   this->scr = pvrscr_init();
   this->scr->scr.start(&this->scr->scr, time);
   this->stream->xine->clock->register_scr(this->stream->xine->clock, &this->scr->scr);
   this->scr_tunning = 0;
 
+  _x_assert(this->event_queue == NULL);
   this->event_queue = xine_event_new_queue (this->stream);
 
   /* enable resample method */
@@ -1454,7 +1465,12 @@ static int pvr_plugin_open (input_plugin_t *this_gen ) {
 			     NULL, pvr_loop, this)) != 0) {
     xprintf (this->stream->xine, XINE_VERBOSITY_NONE,
 	     "input_pvr: can't create new thread (%s)\n", strerror(err));
-    _x_abort();
+
+    this->pvr_running = 0;
+    close(this->dev_fd);
+    this->dev_fd = -1;
+
+    return 0;
   }
 
   return 1;
@@ -1544,10 +1560,7 @@ static void *init_plugin (xine_t *xine, void *data) {
 
   this = calloc(1, sizeof (pvr_input_class_t));
 
-  this->xine   = xine;
-  this->config = xine->config;
-
-  this->devname = this->config->register_filename(this->config,
+  this->devname = xine->config->register_filename(xine->config,
 				    "media.wintv_pvr.device",
 				    PVR_DEVICE, XINE_CONFIG_STRING_IS_DEVICE_NAME,
 				    _("device used for WinTV-PVR 250/350 (pvr plugin)"),
