@@ -126,11 +126,6 @@ static const int tv_standard_values[] = { VIDEO_MODE_AUTO, VIDEO_MODE_PAL, VIDEO
 
 typedef struct pvrscr_s pvrscr_t;
 
-typedef struct {
-  input_class_t            input_class;
-  xine_t                  *xine;
-} v4l_input_class_t;
-
 typedef struct {
   input_plugin_t           input_plugin;
 
@@ -1897,20 +1892,18 @@ static input_plugin_t *v4l_class_get_radio_instance (input_class_t *cls_gen,
  */
 static void *init_video_class (xine_t *xine, void *data)
 {
-  v4l_input_class_t  *this;
+  input_class_t      *this;
   config_values_t    *config = xine->config;
 
-  this = calloc(1, sizeof (v4l_input_class_t));
+  this = calloc(1, sizeof (input_class_t));
 
-  this->xine                           = xine;
-
-  this->input_class.get_instance       = v4l_class_get_video_instance;
-  this->input_class.identifier         = "v4l";
-  this->input_class.description        = N_("v4l tv input plugin");
-  this->input_class.get_dir            = NULL;
-  this->input_class.get_autoplay_list  = NULL;
-  this->input_class.dispose            = default_input_class_dispose;
-  this->input_class.eject_media        = NULL;
+  this->get_instance       = v4l_class_get_video_instance;
+  this->identifier         = "v4l";
+  this->description        = N_("v4l tv input plugin");
+  this->get_dir            = NULL;
+  this->get_autoplay_list  = NULL;
+  this->dispose            = default_input_class_dispose;
+  this->eject_media        = NULL;
 
   config->register_filename (config, "media.video4linux.video_device",
 			   VIDEO_DEV, XINE_CONFIG_STRING_IS_DEVICE_NAME,
@@ -1935,20 +1928,18 @@ static void *init_video_class (xine_t *xine, void *data)
 
 static void *init_radio_class (xine_t *xine, void *data)
 {
-  v4l_input_class_t  *this;
-  config_values_t    *config = xine->config;
+  input_class_t *this;
+  config_values_t *config = xine->config;
 
-  this = calloc(1, sizeof (v4l_input_class_t));
+  this = calloc (1, sizeof (input_class_t));
 
-  this->xine                           = xine;
-
-  this->input_class.get_instance       = v4l_class_get_radio_instance;
-  this->input_class.identifier         = "v4l";
-  this->input_class.description        = N_("v4l radio input plugin");
-  this->input_class.get_dir            = NULL;
-  this->input_class.get_autoplay_list  = NULL;
-  this->input_class.dispose            = default_input_class_dispose;
-  this->input_class.eject_media        = NULL;
+  this->get_instance       = v4l_class_get_radio_instance;
+  this->identifier         = "v4l";
+  this->description        = N_("v4l radio input plugin");
+  this->get_dir            = NULL;
+  this->get_autoplay_list  = NULL;
+  this->dispose            = default_input_class_dispose;
+  this->eject_media        = NULL;
 
   config->register_filename (config, "media.video4linux.radio_device",
 			   RADIO_DEV, XINE_CONFIG_STRING_IS_DEVICE_NAME,
