@@ -30,9 +30,15 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <assert.h>
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
 #include <string.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -182,6 +188,7 @@ static void rtsp_send_request(rtsp_t *s, const char *type, const char *what) {
 
   buf = _x_asprintf("%s %s %s",type, what, rtsp_protocol_version);
   rtsp_put(s,buf);
+  free(buf);
 
   if (s->auth) {
     rtsp_put(s, s->auth);
@@ -211,6 +218,7 @@ static void rtsp_schedule_standard(rtsp_t *s) {
     char *buf;
     buf = _x_asprintf("Session: %s", s->session);
     rtsp_schedule_field(s, buf);
+    free(buf);
   }
 }
 /*
@@ -257,6 +265,7 @@ static int rtsp_get_answers(rtsp_t *s) {
         if (strcmp(tmp, s->session)) {
           xprintf(s->stream->xine, XINE_VERBOSITY_DEBUG,
 		  "rtsp: warning: setting NEW session: %s\n", tmp);
+          free(s->session);
           s->session=strdup(tmp);
         }
       } else
@@ -268,7 +277,7 @@ static int rtsp_get_answers(rtsp_t *s) {
     }
     *answer_ptr=answer;
     answer_ptr++;
-  } while ((strlen(answer)!=0) && (++ans_count < MAX_FIELDS));
+  } while ((strlen(answer)!=0) && (++ans_count < MAX_FIELDS-1));
 
   s->cseq++;
 
@@ -447,8 +456,8 @@ static void rtsp_basicauth (const char *user, const char *password, char** dest)
 rtsp_t *rtsp_connect(xine_stream_t *stream, const char *mrl, const char *user_agent) {
 
   rtsp_t *s = malloc(sizeof(rtsp_t));
-  char *mrl_ptr=strdup(mrl);
-  char *slash, *colon, *amp;
+  const char *mrl_ptr = mrl;
+  const char *slash, *colon, *amp;
   int hostend, i;
   size_t pathbegin;
 
@@ -568,11 +577,11 @@ rtsp_t *rtsp_connect(xine_stream_t *stream, const char *mrl, const char *user_ag
 void rtsp_close(rtsp_t *s) {
 
   if (s->server_state) close(s->s); /* TODO: send a TEAROFF */
-  if (s->path) free(s->path);
-  if (s->host) free(s->host);
-  if (s->mrl) free(s->mrl);
-  if (s->session) free(s->session);
-  if (s->user_agent) free(s->user_agent);
+  free(s->path);
+  free(s->host);
+  free(s->mrl);
+  free(s->session);
+  free(s->user_agent);
   free(s->auth);
   rtsp_free_answers(s);
   rtsp_unschedule_all(s);
@@ -584,12 +593,11 @@ void rtsp_close(rtsp_t *s) {
  * after the first matched tag. returns NULL if no match found.
  */
 
-char *rtsp_search_answers(rtsp_t *s, const char *tag) {
+const char *rtsp_search_answers(rtsp_t *s, const char *tag) {
 
   char **answer;
   char *ptr;
 
-  if (!s->answers) return NULL;
   answer=s->answers;
 
   while (*answer) {
@@ -613,8 +621,7 @@ char *rtsp_search_answers(rtsp_t *s, const char *tag) {
 
 void rtsp_set_session(rtsp_t *s, const char *id) {
 
-  if (s->session) free(s->session);
-
+  free(s->session);
   s->session=strdup(id);
 
 }
@@ -679,7 +686,6 @@ void rtsp_unschedule_all(rtsp_t *s) {
 
   char **ptr;
 
-  if (!s->scheduled) return;
   ptr=s->scheduled;
 
   while (*ptr) {
@@ -696,7 +702,6 @@ void rtsp_free_answers(rtsp_t *s) {
 
   char **answer;
 
-  if (!s->answers) return;
   answer=s->answers;
 
   while (*answer) {
