@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2004 the xine project
+ * Copyright (C) 2002-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -32,9 +32,15 @@
 
 #include <unistd.h>
 #include <stdio.h>
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -582,7 +588,7 @@ static int mms_tcp_connect(mms_t *this) {
    */
   lprintf("try to connect to %s on port %d \n", this->host, this->port);
   this->s = _x_io_tcp_connect (this->stream, this->host, this->port);
-  if (this->s == -1) {
+  if (this->s < 0) {
     xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
              "failed to connect '%s'\n", this->host);
     return 1;
@@ -669,7 +675,7 @@ static int mms_choose_best_streams(mms_t *this) {
  */
 mms_t *mms_connect (xine_stream_t *stream, const char *url, int bandwidth) {
 #ifdef USE_ICONV
-  iconv_t url_conv;
+  iconv_t url_conv = (iconv_t)-1;
 #else
   int     url_conv = 0;
 #endif
@@ -902,6 +908,10 @@ mms_t *mms_connect (xine_stream_t *stream, const char *url, int bandwidth) {
   return this;
 
 fail:
+#ifdef USE_ICONV
+  string_utf16_close(url_conv);
+#endif
+
   if (this->s != -1)
     close (this->s);
   if (this->url)
