@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2009 the xine project
+ * Copyright (C) 2002-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -34,9 +34,15 @@
 
 #include <unistd.h>
 #include <stdio.h>
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -44,7 +50,6 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <time.h>
-#include <assert.h>
 
 #define LOG_MODULE "mmsh"
 #define LOG_VERBOSE
@@ -494,7 +499,7 @@ static int mmsh_tcp_connect(mmsh_t *this) {
 
   this->s = _x_io_tcp_connect (this->stream, this->host, this->port);
 
-  if (this->s == -1) {
+  if (this->s < 0) {
     xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
              "libmmsh: failed to connect '%s'\n", this->host);
     return 1;
@@ -677,18 +682,13 @@ fail:
   lprintf("mmsh_connect: failed\n" );
   if (this->s != -1)
     close(this->s);
-  if (this->url)
-    free(this->url);
-  if (this->proto)
-    free(this->proto);
-  if (this->host)
-    free(this->host);
-  if (this->user)
-    free(this->user);
-  if (this->password)
-    free(this->password);
-  if (this->uri)
-    free(this->uri);
+
+  free(this->url);
+  free(this->proto);
+  free(this->host);
+  free(this->user);
+  free(this->password);
+  free(this->uri);
 
   free(this);
 
@@ -866,22 +866,18 @@ void mmsh_close (mmsh_t *this) {
 
   if (this->s != -1)
     close(this->s);
-  if (this->url)
-    free (this->url);
-  if (this->proto)
-    free(this->proto);
-  if (this->host)
-    free(this->host);
-  if (this->user)
-    free(this->user);
-  if (this->password)
-    free(this->password);
-  if (this->uri)
-    free(this->uri);
+
   if (this->asf_header)
     asf_header_delete(this->asf_header);
-  if (this)
-    free (this);
+
+  free(this->url);
+  free(this->proto);
+  free(this->host);
+  free(this->user);
+  free(this->password);
+  free(this->uri);
+
+  free(this);
 }
 
 
