@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2011 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -69,7 +69,9 @@ void __ctype_b(void) { exit(0); }
 #endif
 
 void _x_real_codecs_init(xine_t *const xine) {
-  const char *real_codecs_path = NULL;
+#ifdef LOG
+  const char *real_codecs_path;
+#endif
 #ifdef REAL_CODEC_PATH
   const char *const default_real_codecs_path = REAL_CODEC_PATH;
 #else
@@ -132,7 +134,9 @@ void _x_real_codecs_init(xine_t *const xine) {
 
 #endif
 
+#ifdef LOG
   real_codecs_path =
+#endif
     xine->config->register_filename (xine->config, "decoder.external.real_codecs_path",
 				     default_real_codecs_path,
 				     XINE_CONFIG_STRING_IS_DIRECTORY_NAME,
@@ -146,7 +150,9 @@ void _x_real_codecs_init(xine_t *const xine) {
 				       "how to install the codecs."),
 				     10, NULL, NULL);
 
+#ifdef LOG
   lprintf ("real codecs path : %s\n", real_codecs_path);
+#endif
 }
 
 void *_x_real_codec_open(xine_stream_t *const stream, const char *const path,
