@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2013 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -178,7 +178,9 @@ static int load_syms_linux (realdec_decoder_t *this, const char *codec_name, con
 static int init_codec (realdec_decoder_t *this, buf_element_t *buf) {
 
   /* unsigned int* extrahdr = (unsigned int*) (buf->content+28); */
+#ifdef LOG
   int           result;
+#endif
   rv_init_t     init_data = {11, 0, 0, 0, 0, 0, 1, 0}; /* rv30 */
 
   switch (buf->type) {
@@ -252,9 +254,14 @@ static int init_codec (realdec_decoder_t *this, buf_element_t *buf) {
 
   this->context = NULL;
 
-  result = this->rvyuv_init (&init_data, &this->context);
+#ifdef LOG
+  result =
+#endif
+    this->rvyuv_init (&init_data, &this->context);
 
+#ifdef LOG
   lprintf ("init result: %d\n", result);
+#endif
 
   /* setup rv30 codec (codec sub-type and image dimensions): */
   if ((init_data.format>=0x20200002) && (buf->type != BUF_VIDEO_RV40)) {
@@ -349,8 +356,9 @@ static void realdec_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
       lprintf ("special buffer (%d bytes)\n", buf->size);
 
       if (buf->decoder_info[1] == BUF_SPECIAL_RV_CHUNK_TABLE) {
-
+#ifdef LOG
         int            result;
+#endif
         vo_frame_t    *img;
 
         transform_out_t transform_out;
@@ -387,15 +395,18 @@ static void realdec_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
                       2*(buf->decoder_info[2]+1)*sizeof(uint32_t));
 #endif
 
-        result = this->rvyuv_transform (this->chunk_buffer,
+#ifdef LOG
+        result =
+#endif
+          this->rvyuv_transform (this->chunk_buffer,
                                         this->frame_buffer,
                                         &transform_in,
                                         &transform_out,
                                         this->context);
 
+#ifdef LOG
         lprintf ("transform result: %08x\n", result);
         lprintf ("transform_out:\n");
-#ifdef LOG
         xine_hexdump ((uint8_t *) &transform_out, 5 * 4);
 #endif
 
@@ -501,6 +512,9 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen,
   realdec_decoder_t *this ;
 
   this = (realdec_decoder_t *) calloc(1, sizeof(realdec_decoder_t));
+  if (!this) {
+    return NULL;
+  }
 
   this->video_decoder.decode_data         = realdec_decode_data;
   this->video_decoder.flush               = realdec_flush;
@@ -526,6 +540,9 @@ void *init_realvdec (xine_t *xine, void *data) {
   real_class_t       *this;
 
   this = (real_class_t *) calloc(1, sizeof(real_class_t));
+  if (!this) {
+    return NULL;
+  }
 
   this->decoder_class.open_plugin     = open_plugin;
   this->decoder_class.identifier      = "realvdec";
