@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2010 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -29,6 +29,7 @@
 #endif
 
 #include <stdio.h>
+#include <pthread.h>
 
 #include <xine/xine_internal.h>
 #include <xine/xineutils.h>
@@ -196,8 +197,7 @@ static int upmix_port_open(xine_audio_port_t *port_gen, xine_stream_t *stream,
        (float)rate, &this->sub->k, this->sub->w[0])) ||
        (-1 == szxform(s_param[1].a, s_param[1].b, Q, this->sub->fc,
        (float)rate, &this->sub->k, this->sub->w[1]))) {
-    free(this->sub);
-    this->sub=NULL;
+    _x_freep(&this->sub);
     pthread_mutex_unlock (&this->lock);
     return 0;
   }
@@ -373,7 +373,7 @@ static void upmix_dispose(post_plugin_t *this_gen)
   post_plugin_upmix_t *this = (post_plugin_upmix_t *)this_gen;
 
   if (_x_post_dispose(this_gen)) {
-    if (this->sub) free(this->sub);
+    _x_freep(&this->sub);
     free(this);
   }
 }
@@ -423,7 +423,7 @@ static post_plugin_t *upmix_open_plugin(post_class_t *class_gen, int inputs,
 /* plugin class initialization function */
 void *upmix_init_plugin(xine_t *xine, void *data)
 {
-  post_class_upmix_t *class = (post_class_upmix_t *)xine_xmalloc(sizeof(post_class_upmix_t));
+  post_class_upmix_t *class = calloc(1, sizeof(post_class_upmix_t));
 
   if (!class)
     return NULL;
