@@ -8,6 +8,10 @@
 */
 /*---------------------------------------------------------------------------*/
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "goom_config_param.h"
 #include <string.h>
 
