@@ -7,6 +7,10 @@
  * (c)2000-2003, by iOS-software.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -24,7 +28,7 @@
 #include "sound_tester.h"
 #include "goom_plugin_info.h"
 #include "goom_fx.h"
-#include "goomsl.h"
+/*#include "goomsl.h"*/
 
 #include <xine/xine_internal.h>
 
@@ -103,7 +107,7 @@ PluginInfo *goom_init (guint32 resx, guint32 resy)
                                          GML_HLINE, 0, GML_BLACK,
                                          GML_CIRCLE, 0.2f * (float) goomInfo->screen.height, GML_RED);
     
-    gfont_load ();
+    goomInfo->font = gfont_load ();
  
     /* goom_set_main_script(goomInfo, goomInfo->main_script_str); */
     
@@ -618,7 +622,7 @@ guint32 *goom_update (PluginInfo *goomInfo, gint16 data[2][512],
             
             if (fps > 0) {
                 sprintf (text, "%2.0f fps", fps);
-                goom_draw_text (goomInfo->p1,goomInfo->screen.width,goomInfo->screen.height,
+                goom_draw_text (goomInfo->font, goomInfo->p1,goomInfo->screen.width,goomInfo->screen.height,
                                 10, 24, text, 1, 0);
             }
             
@@ -632,12 +636,12 @@ guint32 *goom_update (PluginInfo *goomInfo, gint16 data[2][512],
             }
             
             if (goomInfo->update.timeOfTitleDisplay) {
-                goom_draw_text (goomInfo->p1,goomInfo->screen.width,goomInfo->screen.height,
+                goom_draw_text (goomInfo->font, goomInfo->p1,goomInfo->screen.width,goomInfo->screen.height,
                                 goomInfo->screen.width / 2, goomInfo->screen.height / 2 + 7, goomInfo->update.titleText,
                                 ((float) (190 - goomInfo->update.timeOfTitleDisplay) / 10.0f), 1);
                 goomInfo->update.timeOfTitleDisplay--;
                 if (goomInfo->update.timeOfTitleDisplay < 4)
-                    goom_draw_text (goomInfo->p2,goomInfo->screen.width,goomInfo->screen.height,
+                    goom_draw_text (goomInfo->font, goomInfo->p2,goomInfo->screen.width,goomInfo->screen.height,
                                     goomInfo->screen.width / 2, goomInfo->screen.height / 2 + 7, goomInfo->update.titleText,
                                     ((float) (190 - goomInfo->update.timeOfTitleDisplay) / 10.0f), 1);
             }
@@ -772,7 +776,20 @@ void goom_close (PluginInfo *goomInfo)
     goomInfo->star_fx.free(&goomInfo->star_fx);
     goomInfo->tentacles_fx.free(&goomInfo->tentacles_fx);
     goomInfo->zoomFilter_fx.free(&goomInfo->zoomFilter_fx);
-    
+
+    /*
+    if (goomInfo->scanner)
+      gsl_free(goomInfo->scanner);
+    if (goomInfo->main_scanner)
+      gsl_free(goomInfo->main_scanner);
+    */
+
+    gfont_unload(&goomInfo->font);
+
+    free(goomInfo->params);
+    free(goomInfo->visuals);
+    free(goomInfo->sound.params.params);
+
     free(goomInfo);
 }
 
@@ -844,7 +861,7 @@ void update_message (PluginInfo *goomInfo, char *message) {
     
     if (message) {
         int i=1,j=0;
-	strcpy(goomInfo->update_message.message, message);
+        strlcpy(goomInfo->update_message.message, message, sizeof(goomInfo->update_message.message));
         for (j=0;goomInfo->update_message.message[j];j++)
             if (goomInfo->update_message.message[j]=='\n')
                 i++;
@@ -881,7 +898,7 @@ void update_message (PluginInfo *goomInfo, char *message) {
                 pos = (int)goomInfo->screen.height / 2;
             pos += 7;
             
-            goom_draw_text(goomInfo->p1,goomInfo->screen.width,goomInfo->screen.height,
+            goom_draw_text(goomInfo->font, goomInfo->p1,goomInfo->screen.width,goomInfo->screen.height,
                            goomInfo->screen.width/2, pos,
                            message,
                            ecart,
