@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 the xine project
+ * Copyright (C) 2003-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -127,7 +127,7 @@ static int32_t        expand_overlay_add_event(video_overlay_manager_t *this_gen
 
 void *expand_init_plugin(xine_t *xine, void *data)
 {
-  post_class_t *class = (post_class_t *)xine_xmalloc(sizeof(post_class_t));
+  post_class_t *class = calloc(1, sizeof(post_class_t));
 
   if (!class)
     return NULL;
