@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2009 the xine project
+ * Copyright (C) 2005-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -45,7 +45,7 @@ static int            fill_draw(vo_frame_t *frame, xine_stream_t *stream);
 
 void *fill_init_plugin(xine_t *xine, void *data)
 {
-  post_class_t *class = (post_class_t *)xine_xmalloc(sizeof(post_class_t));
+  post_class_t *class = calloc(1, sizeof(post_class_t));
 
   if (!class)
     return NULL;
