@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -407,8 +407,7 @@ static void fftgraph_dispose(post_plugin_t *this_gen)
 
     this->metronom->exit(this->metronom);
 
-    if(this->buf.mem)
-      free(this->buf.mem);
+    _x_freep(&this->buf.mem);
     free(this);
   }
 }
@@ -459,7 +458,7 @@ static post_plugin_t *fftgraph_open_plugin(post_class_t *class_gen, int inputs,
 /* plugin class initialization function */
 void *fftgraph_init_plugin(xine_t *xine, void *data)
 {
-  post_class_fftgraph_t *class = (post_class_fftgraph_t *)xine_xmalloc(sizeof(post_class_fftgraph_t));
+  post_class_fftgraph_t *class = calloc(1, sizeof(post_class_fftgraph_t));
 
   if (!class)
     return NULL;
