@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * Copyright (C) Christian Vogler
  *               cvogler@gradient.cis.upenn.edu - December 2001
@@ -461,7 +461,7 @@ static void get_font_metrics(osd_renderer_t *renderer,
   *maxw = 0;
   *maxh = 0;
 
-  renderer->set_font(testc, (char *) fontname, font_size);
+  renderer->set_font(testc, fontname, font_size);
   renderer->set_encoding(testc, "iso-8859-1");
   for (c = 32; c < 256; c++) {
     int tw, th;
@@ -572,7 +572,7 @@ static void ccrow_set_attributes(cc_renderer_t *renderer, cc_row_t *this,
     fontname = cap_info->italic_font;
   else
     fontname = cap_info->font;
-  renderer->osd_renderer->set_font(renderer->cap_display, (char *) fontname,
+  renderer->osd_renderer->set_font(renderer->cap_display, fontname,
 				   cap_info->font_size);
 }
 
@@ -960,7 +960,11 @@ cc_renderer_t *cc_renderer_open(osd_renderer_t *osd_renderer,
 				metronom_t *metronom, cc_state_t *cc_state,
 				int video_width, int video_height)
 {
-  cc_renderer_t *this = calloc(1, sizeof (cc_renderer_t));
+  cc_renderer_t *this;
+
+  this = calloc(1, sizeof (cc_renderer_t));
+  if (!this)
+    return NULL;
 
   this->osd_renderer = osd_renderer;
   this->metronom = metronom;
@@ -1477,7 +1481,12 @@ void decode_cc(cc_decoder_t *this, uint8_t *buffer, uint32_t buf_len,
 
 cc_decoder_t *cc_decoder_open(cc_state_t *cc_state)
 {
-  cc_decoder_t *this = calloc(1, sizeof (cc_decoder_t));
+  cc_decoder_t *this;
+
+  this = calloc(1, sizeof (cc_decoder_t));
+  if (!this)
+    return NULL;
+
   /* configfile stuff */
   this->cc_state = cc_state;
 
