@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2010 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -18,6 +18,10 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #define LOG_MODULE "libspucmml"
 #define LOG_VERBOSE
 /*
@@ -286,7 +290,7 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
 	lprintf ("found title: \"%s\"\n", data.str);
 
 	/* set xine meta-info */
-	_x_meta_info_set(this->stream, XINE_META_INFO_TITLE, strdup(data.str));
+	_x_meta_info_set(this->stream, XINE_META_INFO_TITLE, data.str);
 
 	/* and push out a new event signifying the title update on the event
 	 * queue */
@@ -306,8 +310,10 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
 
         /* found the <a> tag: grab its value and its href property */
 
-        if (clip_node->data)
+        if (clip_node->data) {
+          free(anchor_text);
           anchor_text = strdup (clip_node->data);
+        }
 
         for (href_property = clip_node->props; href_property != NULL; href_property = href_property->next) {
           if (strcasecmp (href_property->name, "href") == 0) {
@@ -410,6 +416,10 @@ static void spudec_discontinuity (spu_decoder_t *this_gen) {
 
 static void spudec_dispose (spu_decoder_t *this_gen) {
   spucmml_decoder_t *this = (spucmml_decoder_t *) this_gen;
+  config_values_t   *config = this->class->xine->config;
+
+  config->unregister_callback(config, "subtitles.separate.font");
+  config->unregister_callback(config, "subtitles.separate.vertical_offset");
 
   if (this->event_queue)
     xine_event_dispose_queue (this->event_queue);
@@ -441,7 +451,11 @@ static void update_osd_font(void *this_gen, xine_cfg_entry_t *entry)
 
 static spu_decoder_t *spucmml_class_open_plugin (spu_decoder_class_t *class_gen, xine_stream_t *stream) {
   spucmml_class_t *class = (spucmml_class_t *)class_gen;
-  spucmml_decoder_t *this = (spucmml_decoder_t *) calloc(1, sizeof(spucmml_decoder_t));
+  spucmml_decoder_t *this;
+
+  this = (spucmml_decoder_t *) calloc(1, sizeof(spucmml_decoder_t));
+  if (!this)
+    return NULL;
 
   this->spu_decoder.decode_data         = spudec_decode_data;
   this->spu_decoder.reset               = spudec_reset;
@@ -489,13 +503,26 @@ static void update_src_encoding(void *this_gen, xine_cfg_entry_t *entry)
   printf("libspucmml: spu_src_encoding = %s\n", this->src_encoding );
 }
 
+static void spu_decoder_class_dispose(spu_decoder_class_t *this_gen)
+{
+  spucmml_class_t *this = (spucmml_class_t *)this_gen;
+
+  this->xine->config->unregister_callback(this->xine->config, "subtitles.separate.src_encoding");
+
+  free(this);
+}
+
 static void *init_spu_decoder_plugin (xine_t *xine, void *data) {
-  spucmml_class_t *this = (spucmml_class_t *) calloc(1, sizeof(spucmml_class_t));
+  spucmml_class_t *this;
+
+  this = (spucmml_class_t *) calloc(1, sizeof(spucmml_class_t));
+  if (!this)
+    return NULL;
 
   this->class.open_plugin      = spucmml_class_open_plugin;
   this->class.identifier       = "spucmml";
   this->class.description      = N_("CMML subtitle decoder plugin");
-  this->class.dispose          = default_spu_decoder_class_dispose;
+  this->class.dispose          = spu_decoder_class_dispose;
 
   this->xine                   = xine;
 
