@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2010 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * Copyright (C) James Courtier-Dutton James@superbug.demon.co.uk - July 2001
  *
@@ -28,10 +28,9 @@
 
 #include <stdlib.h>
 #include <unistd.h>
+#include <pthread.h>
 #include <string.h>
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
 
 #include <xine/xine_internal.h>
 #include <xine/buffer.h>
@@ -197,13 +196,13 @@ static void spudec_dispose (spu_decoder_t *this_gen) {
       ovl_manager->free_handle(ovl_manager,
 			       this->spudec_stream_state[i].overlay_handle);
     this->spudec_stream_state[i].overlay_handle = -1;
-    free (this->spudec_stream_state[i].ra_seq.buf);
+    _x_freep (&this->spudec_stream_state[i].ra_seq.buf);
   }
 
   spudec_clear_nav_list(this);
   pthread_mutex_destroy(&this->nav_pci_lock);
 
-  free (this->event.object.overlay);
+  _x_freep (&this->event.object.overlay);
   free (this);
 }
 
@@ -306,12 +305,9 @@ static void spudec_set_button (spu_decoder_t *this_gen, int32_t button, int32_t
             overlay_event->vpts);
 #endif
     ovl_manager->add_event (ovl_manager, (void *)overlay_event);
-    free(overlay_event);
-    free(overlay);
-  } else {
-    free(overlay_event);
-    free(overlay);
   }
+  free(overlay_event);
+  free(overlay);
   return;
 }
 
@@ -320,6 +316,8 @@ static spu_decoder_t *open_plugin (spu_decoder_class_t *class_gen, xine_stream_t
   spudec_decoder_t *this ;
 
   this = (spudec_decoder_t *) calloc(1, sizeof (spudec_decoder_t));
+  if (!this)
+    return NULL;
 
   this->spu_decoder.decode_data         = spudec_decode_data;
   this->spu_decoder.reset               = spudec_reset;
@@ -328,11 +326,14 @@ static spu_decoder_t *open_plugin (spu_decoder_class_t *class_gen, xine_stream_t
   this->spu_decoder.get_interact_info   = spudec_get_interact_info;
   this->spu_decoder.set_button          = spudec_set_button;
   this->stream                          = stream;
-  this->class                           = (spudec_class_t *) class_gen;
 
   this->menu_handle = -1;
   this->buttonN = 1;
   this->event.object.overlay = calloc(1, sizeof(vo_overlay_t));
+  if (!this->event.object.overlay) {
+    free(this);
+    return NULL;
+  }
 
   pthread_mutex_init(&this->nav_pci_lock, NULL);
   this->pci_cur.pci.hli.hl_gi.hli_ss  = 0;
@@ -358,14 +359,18 @@ static spu_decoder_t *open_plugin (spu_decoder_class_t *class_gen, xine_stream_t
 
 static void *init_plugin (xine_t *xine, void *data) {
 
-  spudec_class_t *this;
+  spu_decoder_class_t *this;
 
-  this = calloc(1, sizeof (spudec_class_t));
+  this = calloc(1, sizeof (spu_decoder_class_t));
+  if (!this)
+    return NULL;
 
-  this->decoder_class.open_plugin     = open_plugin;
-  this->decoder_class.identifier      = "spudec";
-  this->decoder_class.description     = N_("DVD/VOB SPU decoder plugin");
-  this->decoder_class.dispose         = default_spu_decoder_class_dispose;
+  this = calloc(1, sizeof (spu_decoder_class_t));
+
+  this->open_plugin     = open_plugin;
+  this->identifier      = "spudec";
+  this->description     = N_("DVD/VOB SPU decoder plugin");
+  this->dispose         = default_spu_decoder_class_dispose;
 
   lprintf ("libspudec:init_plugin called\n");
   return this;
