@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2014 the xine project
+ * Copyright (C) 2010-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -25,6 +25,11 @@
  * - Implement support for teletext based subtitles
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <time.h>
 #include <pthread.h>
 #include <errno.h>
 
@@ -62,8 +67,7 @@ static void sparse_array_delete (sparse_array_t *sa) {
   sa->sorted_entries =
   sa->used_entries   =
   sa->max_entries    = 0;
-  free (sa->entries);
-  sa->entries        = NULL;
+  _x_freep (&sa->entries);
 }
 
 static int _sparse_array_find (sparse_array_t *sa, uint32_t key, uint32_t *pos) {
@@ -337,24 +341,18 @@ static void update_region (dvb_spu_decoder_t * this, int region_id, int region_w
 
   /* reject invalid sizes and set some limits ! */
   if ( region_width<=0 || region_height<=0 || region_width>SPU_MAX_WIDTH || region_height>SPU_MAX_HEIGHT ) {
-    if ( reg->img ) {
-      free( reg->img );
-      reg->img = NULL;
-    }
+    _x_freep( &reg->img );
     lprintf("rejected region %d = %dx%d\n", region_id, region_width, region_height );
     return;
   }
 
   if ( reg->width*reg->height<region_width*region_height ) {
     lprintf("update size of region %d = %dx%d\n", region_id, region_width, region_height);
-    if ( reg->img ) {
-      free( reg->img );
-      reg->img = NULL;
-    }
+    _x_freep( &reg->img );
   }
 
   if ( !reg->img ) {
-    if ( !(reg->img=xine_xmalloc(region_width*region_height)) ) {
+    if ( !(reg->img = calloc(1, region_width * region_height)) ) {
       lprintf( "can't allocate mem for region %d\n", region_id );
       return;
     }
@@ -984,7 +982,7 @@ static void* dvbsub_timer_func(void *this_gen)
     {
       /* We timed out, and no-one changed the timeout underneath us.
          Hide the OSD, then wait until we're signalled. */
-      if(this && this->stream && this->stream->osd_renderer)
+      if(this->stream && this->stream->osd_renderer)
       {
 	for ( i=0; i<this->dvbsub->max_regions; i++ ) {
 	  if ( this->dvbsub->regions[i].osd ) {
@@ -1233,43 +1231,49 @@ static void spudec_discontinuity (spu_decoder_t * this_gen)
   /* do nothing */
 }
 
-static void spudec_dispose (spu_decoder_t * this_gen)
+static void spudec_dispose_internal (dvb_spu_decoder_t * this, int thread_running)
 {
-  dvb_spu_decoder_t *this = (dvb_spu_decoder_t *) this_gen;
   int i;
 
-  pthread_cancel(this->dvbsub_timer_thread);
-  pthread_join(this->dvbsub_timer_thread, NULL);
+  if (thread_running) {
+    pthread_cancel(this->dvbsub_timer_thread);
+    pthread_join(this->dvbsub_timer_thread, NULL);
+  }
   pthread_mutex_destroy(&this->dvbsub_osd_mutex);
   pthread_cond_destroy(&this->dvbsub_restart_timeout);
 
-  if(this->spu_descriptor){
-    free(this->spu_descriptor);
-    this->spu_descriptor=NULL;
-  }
+  _x_freep(&this->spu_descriptor);
 
   for ( i=0; i<MAX_REGIONS; i++ ) {
-    if ( this->dvbsub->regions[i].img )
-      free( this->dvbsub->regions[i].img );
+    _x_freep( &this->dvbsub->regions[i].img );
     if ( this->dvbsub->regions[i].osd )
       this->stream->osd_renderer->free_object( this->dvbsub->regions[i].osd );
   }
 
-  if (this->pes_pkt)
-    free (this->pes_pkt);
+  _x_freep (&this->pes_pkt);
 
   if (this->dvbsub) {
     sparse_array_delete (&this->dvbsub->object_pos);
-    free (this->dvbsub);
+    _x_freep (&this->dvbsub);
   }
 
   free (this);
 }
 
+static void spudec_dispose (spu_decoder_t * this_gen)
+{
+  dvb_spu_decoder_t *this = (dvb_spu_decoder_t *) this_gen;
+  spudec_dispose_internal(this, 1);
+}
+
 static spu_decoder_t *dvb_spu_class_open_plugin (spu_decoder_class_t * class_gen, xine_stream_t * stream)
 {
-  dvb_spu_decoder_t *this = calloc(1, sizeof (dvb_spu_decoder_t));
   dvb_spu_class_t *class = (dvb_spu_class_t *)class_gen;
+  dvb_spu_decoder_t *this;
+
+  this = calloc(1, sizeof (dvb_spu_decoder_t));
+  if (!this)
+    return NULL;
 
 #define YUVA(r, g, b, a) (clut_t) { COMPUTE_V(r, g, b), COMPUTE_U(r, g, b), COMPUTE_V(r, g, b), a }
 #define GETBIT(s, v1, v2, tr) \
@@ -1309,10 +1313,16 @@ static spu_decoder_t *dvb_spu_class_open_plugin (spu_decoder_class_t * class_gen
   this->class = class;
   this->stream = stream;
 
-  this->pes_pkt = calloc(65, 1024);
-  this->spu_descriptor = calloc(1, sizeof(spu_dvb_descriptor_t));
+  pthread_mutex_init(&this->dvbsub_osd_mutex, NULL);
+  pthread_cond_init(&this->dvbsub_restart_timeout, NULL);
 
-  this->dvbsub = calloc(1, sizeof (dvbsub_func_t));
+  this->pes_pkt        = calloc(65, 1024);
+  this->spu_descriptor = calloc(1, sizeof(spu_dvb_descriptor_t));
+  this->dvbsub         = calloc(1, sizeof (dvbsub_func_t));
+  if (!this->pes_pkt || !this->spu_descriptor || !this->dvbsub) {
+    spudec_dispose_internal(this, 0);
+    return NULL;
+  }
 
   int i;
   for (i = 0; i < MAX_REGIONS; i++) {
@@ -1336,11 +1346,14 @@ static spu_decoder_t *dvb_spu_class_open_plugin (spu_decoder_class_t * class_gen
 
   sparse_array_new (&this->dvbsub->object_pos);
 
-  pthread_mutex_init(&this->dvbsub_osd_mutex, NULL);
-  pthread_cond_init(&this->dvbsub_restart_timeout, NULL);
   this->dvbsub_hide_timeout.tv_nsec = 0;
   this->dvbsub_hide_timeout.tv_sec = time(NULL);
-  pthread_create(&this->dvbsub_timer_thread, NULL, dvbsub_timer_func, this);
+
+  if (pthread_create(&this->dvbsub_timer_thread, NULL, dvbsub_timer_func, this)) {
+    xprintf (this->class->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": pthread_create() failed\n");
+    spudec_dispose_internal(this, 0);
+    return NULL;
+  }
 
   return (spu_decoder_t *) this;
 }
@@ -1359,6 +1372,8 @@ static void *init_spu_decoder_plugin (xine_t * xine, void *data)
 
   dvb_spu_class_t *this;
   this = calloc(1, sizeof (dvb_spu_class_t));
+  if (!this)
+    return NULL;
 
   this->class.open_plugin = dvb_spu_class_open_plugin;
   this->class.identifier  = "spudvb";
