@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * Copyright (C) 2009 Petri Hintukainen <phintuka@users.sourceforge.net>
  *
@@ -182,8 +182,8 @@ struct presentation_segment_s {
 static void free_subtitle_object(void *ptr)
 {
   if (ptr) {
-    free(((subtitle_object_t*)ptr)->rle);
-    free(((subtitle_object_t*)ptr)->raw_data);
+    _x_freep(&((subtitle_object_t*)ptr)->rle);
+    _x_freep(&((subtitle_object_t*)ptr)->raw_data);
     free(ptr);
   }
 }
@@ -229,8 +229,7 @@ static segment_buffer_t *segbuf_init(void)
 
 static void segbuf_dispose(segment_buffer_t *buf)
 {
-  if (buf->buf)
-    free (buf->buf);
+  _x_freep (&buf->buf);
   free (buf);
 }
 
@@ -272,10 +271,7 @@ static void segbuf_fill(segment_buffer_t *buf, uint8_t *data, size_t len)
 {
   if (buf->len + len > buf->data_size) {
     buf->data_size = buf->len + len;
-    if (buf->buf)
-      buf->buf = realloc(buf->buf, buf->data_size);
-    else
-      buf->buf = malloc(buf->data_size);
+    buf->buf = realloc(buf->buf, buf->data_size);
   }
 
   memcpy(buf->buf + buf->len, data, len);
@@ -291,7 +287,8 @@ static int segbuf_segment_complete(segment_buffer_t *buf)
 
 static void segbuf_skip_segment(segment_buffer_t *buf)
 {
-  if (segbuf_segment_complete (buf)) {
+  _x_assert (segbuf_segment_complete (buf));
+
     buf->len -= buf->segment_len + 3;
     if (buf->len > 0)
       memmove(buf->buf, buf->buf + buf->segment_len + 3, buf->len);
@@ -299,11 +296,6 @@ static void segbuf_skip_segment(segment_buffer_t *buf)
     segbuf_parse_segment_header(buf);
 
     XINE_HDMV_TRACE("  skip_segment: %zd bytes left\n", buf->len);
-  } else {
-    XINE_HDMV_ERROR("  skip_segment: ERROR - %zd bytes queued, %d required\n",
-                    buf->len, buf->segment_len);
-    segbuf_reset (buf);
-  }
 }
 
 /*
@@ -346,6 +338,7 @@ static uint32_t segbuf_get_u24(segment_buffer_t *buf)
 
 static subtitle_clut_t *segbuf_decode_palette(segment_buffer_t *buf)
 {
+  subtitle_clut_t *clut;
   uint8_t palette_id             = segbuf_get_u8 (buf);
   uint8_t palette_version_number = segbuf_get_u8 (buf);
 
@@ -365,7 +358,11 @@ static subtitle_clut_t *segbuf_decode_palette(segment_buffer_t *buf)
                   entries, palette_id, palette_version_number);
 
   /* convert to xine-lib clut */
-  subtitle_clut_t *clut = calloc(1, sizeof(subtitle_clut_t));
+
+  clut = calloc(1, sizeof(subtitle_clut_t));
+  if (!clut)
+    return NULL;
+
   clut->id = palette_id;
 
   for (i = 0; i < entries; i++) {
@@ -462,9 +459,13 @@ static subtitle_object_t *segbuf_decode_object(segment_buffer_t *buf, subtitle_o
                   object_id, version, seq_desc);
 
   if (seq_desc & 0x80) {
+    subtitle_object_t *obj;
+
     /* new object (first-in-sequence flag set) */
 
-    subtitle_object_t *obj = calloc(1, sizeof(subtitle_object_t));
+    obj = calloc(1, sizeof(subtitle_object_t));
+    if (!obj)
+      return NULL;
 
     obj->id       = object_id;
     obj->data_len = segbuf_get_u24(buf);
@@ -485,8 +486,7 @@ static subtitle_object_t *segbuf_decode_object(segment_buffer_t *buf, subtitle_o
       XINE_HDMV_TRACE("    object length %d bytes, have only %zd bytes -> missing %d bytes\n",
                       obj->data_len, segbuf_data_length(buf), obj->data_len - (int)segbuf_data_length(buf));
 
-      if (obj->raw_data)
-        free(obj->raw_data);
+      _x_freep(&obj->raw_data);
 
       /* store partial RLE data in HDMV format */
       obj->raw_data_len  = segbuf_data_length(buf);
@@ -543,8 +543,7 @@ static subtitle_object_t *segbuf_decode_object(segment_buffer_t *buf, subtitle_o
     }
 
     /* free decode buffer */
-    free(objects->raw_data);
-    objects->raw_data      = NULL;
+    _x_freep(&objects->raw_data);
     objects->raw_data_len  = 0;
     objects->raw_data_size = 0;
   }
@@ -554,9 +553,13 @@ static subtitle_object_t *segbuf_decode_object(segment_buffer_t *buf, subtitle_o
 
 static window_def_t *segbuf_decode_window_definition(segment_buffer_t *buf)
 {
-  window_def_t *wnd = calloc(1, sizeof(window_def_t));
+  window_def_t *wnd;
+  uint8_t a;
+  wnd = calloc(1, sizeof(window_def_t));
+  if (!wnd)
+    return NULL;
 
-  uint8_t a   = segbuf_get_u8 (buf);
+  a           = segbuf_get_u8 (buf);
   wnd->id     = segbuf_get_u8 (buf);
   wnd->xpos   = segbuf_get_u16 (buf);
   wnd->ypos   = segbuf_get_u16 (buf);
@@ -596,11 +599,16 @@ static int segbuf_decode_composition_descriptor(segment_buffer_t *buf, compositi
 
 static composition_object_t *segbuf_decode_composition_object(segment_buffer_t *buf)
 {
-  composition_object_t *cobj = calloc(1, sizeof(composition_object_t));
+  composition_object_t *cobj;
+  uint8_t tmp;
+
+  cobj = calloc(1, sizeof(composition_object_t));
+  if (!cobj)
+    return NULL;
 
   cobj->object_id_ref    = segbuf_get_u16 (buf);
   cobj->window_id_ref    = segbuf_get_u8 (buf);
-  uint8_t tmp            = segbuf_get_u8 (buf);
+  tmp                    = segbuf_get_u8 (buf);
   cobj->cropped_flag     = !!(tmp & 0x80);
   cobj->forced_flag      = !!(tmp & 0x40);
   cobj->xpos             = segbuf_get_u16 (buf);
@@ -628,9 +636,13 @@ static composition_object_t *segbuf_decode_composition_object(segment_buffer_t *
 
 static presentation_segment_t *segbuf_decode_presentation_segment(segment_buffer_t *buf)
 {
-  presentation_segment_t *seg = calloc(1, sizeof(presentation_segment_t));
+  presentation_segment_t *seg;
   int                     index;
 
+  seg = calloc(1, sizeof(presentation_segment_t));
+  if (!seg)
+    return NULL;
+
   segbuf_decode_video_descriptor (buf);
   segbuf_decode_composition_descriptor (buf, &seg->comp_descr);
 
@@ -662,7 +674,9 @@ static rle_elem_t *copy_crop_rle(subtitle_object_t *obj, composition_object_t *c
   /* TODO: cropping (w,h sized image from pos x,y) */
 
   rle_elem_t *rle = calloc (obj->num_rle, sizeof(rle_elem_t));
-  memcpy (rle, obj->rle, obj->num_rle * sizeof(rle_elem_t));
+  if (rle) {
+    memcpy (rle, obj->rle, obj->num_rle * sizeof(rle_elem_t));
+  }
   return rle;
 }
 
@@ -671,14 +685,9 @@ static rle_elem_t *copy_crop_rle(subtitle_object_t *obj, composition_object_t *c
  * xine plugin
  */
 
-typedef struct {
-  spu_decoder_class_t decoder_class;
-} spuhdmv_class_t;
-
 typedef struct spuhdmv_decoder_s {
   spu_decoder_t    spu_decoder;
 
-  spuhdmv_class_t  *class;
   xine_stream_t    *stream;
 
   segment_buffer_t *buf;
@@ -813,10 +822,13 @@ static int show_overlay(spuhdmv_decoder_t *this, composition_object_t *cobj, uns
   overlay.width     = obj->width;
   overlay.height    = obj->height;
 
-  overlay.rle       = copy_crop_rle (obj, cobj);
   overlay.num_rle   = obj->num_rle;
   overlay.data_size = obj->num_rle * sizeof(rle_elem_t);
 
+  overlay.rle       = copy_crop_rle (obj, cobj);
+  if (!overlay.rle)
+    return -1;
+
   /* */
 
   overlay.x = /*wnd->xpos +*/ cobj->xpos;
@@ -869,10 +881,7 @@ static void hide_overlays(spuhdmv_decoder_t *this, int64_t pts)
     metronom_t              *metronom    = this->stream->metronom;
 
     event.object.handle = this->overlay_handles[i];
-    if (this)
-      event.vpts = metronom->got_spu_packet (metronom, pts);
-    else
-      event.vpts = 0;
+    event.vpts = metronom->got_spu_packet (metronom, pts);
     event.event_type = OVERLAY_EVENT_HIDE;
     event.object.overlay = NULL;
     ovl_manager->add_event (ovl_manager, (void *)&event);
@@ -1038,6 +1047,14 @@ static spu_decoder_t *open_plugin (spu_decoder_class_t *class_gen, xine_stream_t
   spuhdmv_decoder_t *this;
 
   this = (spuhdmv_decoder_t *) calloc(1, sizeof (spuhdmv_decoder_t));
+  if (!this)
+    return NULL;
+
+  this->buf = segbuf_init();
+  if (!this->buf) {
+    free(this);
+    return NULL;
+  }
 
   this->spu_decoder.decode_data         = spudec_decode_data;
   this->spu_decoder.reset               = spudec_reset;
@@ -1046,9 +1063,6 @@ static spu_decoder_t *open_plugin (spu_decoder_class_t *class_gen, xine_stream_t
   this->spu_decoder.get_interact_info   = NULL;
   this->spu_decoder.set_button          = NULL;
   this->stream                          = stream;
-  this->class                           = (spuhdmv_class_t *) class_gen;
-
-  this->buf = segbuf_init();
 
   memset(this->overlay_handles, 0xff, sizeof(this->overlay_handles)); /* --> -1 */
 
@@ -1057,14 +1071,16 @@ static spu_decoder_t *open_plugin (spu_decoder_class_t *class_gen, xine_stream_t
 
 static void *init_plugin (xine_t *xine, void *data)
 {
-  spuhdmv_class_t *this;
+  spu_decoder_class_t *this;
 
-  this = calloc(1, sizeof (spuhdmv_class_t));
+  this = calloc(1, sizeof (spu_decoder_class_t));
+  if (!this)
+    return NULL;
 
-  this->decoder_class.open_plugin = open_plugin;
-  this->decoder_class.identifier  = "spuhdmv";
-  this->decoder_class.description = "HDMV/BluRay bitmap SPU decoder plugin";
-  this->decoder_class.dispose     = default_spu_decoder_class_dispose;
+  this->open_plugin = open_plugin;
+  this->identifier  = "spuhdmv";
+  this->description = "HDMV/BluRay bitmap SPU decoder plugin";
+  this->dispose     = default_spu_decoder_class_dispose;
 
   return this;
 }
