@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -37,8 +37,6 @@
 #include <unistd.h>
 #include <string.h>
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
 #include <ctype.h>
 
 #define LOG_MODULE "demux_sputext"
@@ -102,6 +100,7 @@ typedef struct demux_sputext_class_s {
 
   demux_class_t      demux_class;
 
+  xine_t            *xine;
   int                max_timeout;  /* default timeout of hidding subtitles */
 
 } demux_sputext_class_t;
@@ -1304,10 +1303,10 @@ static void demux_sputext_dispose (demux_plugin_t *this_gen) {
 
   for (i = 0; i < this->num; i++) {
     for (l = 0; l < this->subtitles[i].lines; l++)
-      free(this->subtitles[i].text[l]);
+      _x_freep(&this->subtitles[i].text[l]);
   }
-  free(this->subtitles);
-  free(this->encoding);
+  _x_freep(&this->subtitles);
+  _x_freep(&this->encoding);
   free(this);
 }
 
@@ -1402,6 +1401,9 @@ static demux_plugin_t *open_demux_plugin (demux_class_t *class_gen, xine_stream_
   lprintf("open_plugin() called\n");
 
   this = calloc(1, sizeof (demux_sputext_t));
+  if (!this)
+    return NULL;
+
   this->stream = stream;
   this->input  = input;
 
@@ -1473,6 +1475,15 @@ static void config_timeout_cb(void *this_gen, xine_cfg_entry_t *entry) {
   this->max_timeout = entry->num_value;
 }
 
+static void sputext_demux_class_dispose(demux_class_t *this_gen)
+{
+  demux_sputext_class_t *this = (demux_sputext_class_t *)this_gen;
+
+  this->xine->config->unregister_callback(this->xine->config, "subtitles.separate.timeout");
+
+  free(this);
+}
+
 void *init_sputext_demux_class (xine_t *xine, void *data) {
 
   demux_sputext_class_t *this ;
@@ -1480,6 +1491,8 @@ void *init_sputext_demux_class (xine_t *xine, void *data) {
   lprintf("initializing\n");
 
   this = calloc(1, sizeof (demux_sputext_class_t));
+  if (!this)
+    return NULL;
 
   this->demux_class.open_plugin     = open_demux_plugin;
   this->demux_class.description     = N_("sputext demuxer plugin");
@@ -1488,7 +1501,9 @@ void *init_sputext_demux_class (xine_t *xine, void *data) {
   /* "text/plain: asc txt sub srt: VIDEO subtitles;" */
   this->demux_class.mimetypes       = NULL;
   this->demux_class.extensions      = "asc txt sub srt smi ssa ass";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  this->demux_class.dispose         = sputext_demux_class_dispose;
+
+  this->xine = xine;
 
   /*
    * Some subtitling formats, namely AQT and Subrip09, define the end of a
