@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2013 the xine project
+ * Copyright (C) 2003-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -32,10 +32,19 @@
 #include <errno.h>
 #include <pthread.h>
 
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
 #include <resolv.h>
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
+#ifdef WIN32
+#include <winsock.h>
+#endif
 
 #define LOG_MODULE "input_vdr"
 #define LOG_VERBOSE
@@ -136,6 +145,7 @@ struct vdr_input_plugin_s
   pthread_cond_t      trick_speed_mode_cond;
 
   pthread_t           rpc_thread;
+  int                 rpc_thread_created;
   int                 rpc_thread_shutdown;
   pthread_mutex_t     rpc_thread_shutdown_lock;
   pthread_cond_t      rpc_thread_shutdown_cond;
@@ -144,6 +154,7 @@ struct vdr_input_plugin_s
   pthread_t           metronom_thread;
   pthread_mutex_t     metronom_thread_lock;
   int64_t             metronom_thread_request;
+  int                 metronom_thread_created;
   int                 metronom_thread_reply;
   pthread_cond_t      metronom_thread_request_cond;
   pthread_cond_t      metronom_thread_reply_cond;
@@ -559,12 +570,10 @@ break;
 */
       if (this->osd_buffer_size < this->cur_size)
       {
-        if (this->osd_buffer)
-          free(this->osd_buffer);
-
+        free(this->osd_buffer);
         this->osd_buffer_size = 0;
 
-        this->osd_buffer = xine_xmalloc(this->cur_size);
+        this->osd_buffer = calloc(1, this->cur_size);
         if (!this->osd_buffer)
           return -1;
 
@@ -601,7 +610,7 @@ break;
               int y;
 
               if (src_stride == dst_stride)
-                xine_fast_memcpy(dst, src, src_stride * data->height);
+                xine_fast_memcpy(dst, src, src_stride * (size_t)data->height);
               else
               {
                 for (y = 0; y < data->height; y++)
@@ -1395,7 +1404,7 @@ fprintf(stderr, "C =============================================\n");
   {
     off_t skip = this->cur_size - this->cur_done;
 
-    lprintf("func: %d, skipping: %lld\n", this->cur_func, skip);
+    lprintf("func: %d, skipping: %" PRId64 "\n", this->cur_func, (int64_t)skip);
 
     while (skip > BUF_SIZE)
     {
@@ -1681,8 +1690,8 @@ static off_t vdr_plugin_seek(input_plugin_t *this_gen, off_t offset, int origin)
 {
   vdr_input_plugin_t *this = (vdr_input_plugin_t *)this_gen;
 
-  lprintf("seek %lld offset, %d origin...\n",
-          offset, origin);
+  lprintf("seek %" PRId64 " offset, %d origin...\n",
+          (int64_t)offset, origin);
 
   if ((origin == SEEK_CUR) && (offset >= 0))
   {
@@ -1702,7 +1711,8 @@ static off_t vdr_plugin_seek(input_plugin_t *this_gen, off_t offset, int origin)
       if (this->curpos <= this->preview_size)
         this->curpos = offset;
       else
-        lprintf("cannot seek back! (%lld > %lld)\n", this->curpos, offset);
+        lprintf("cannot seek back! (%" PRId64 " > %" PRId64 ")\n",
+                (int64_t)this->curpos, (int64_t)offset);
     }
     else
     {
@@ -1760,7 +1770,7 @@ static void vdr_plugin_dispose(input_plugin_t *this_gen)
   if (this->event_queue)
     xine_event_dispose_queue(this->event_queue);
 
-  if (this->rpc_thread)
+  if (this->rpc_thread_created)
   {
     struct timespec abstime;
     int ms_to_time_out = 10000;
@@ -1804,7 +1814,7 @@ static void vdr_plugin_dispose(input_plugin_t *this_gen)
   pthread_cond_destroy(&this->rpc_thread_shutdown_cond);
   pthread_mutex_destroy(&this->rpc_thread_shutdown_lock);
 
-  if (this->metronom_thread)
+  if (this->metronom_thread_created)
   {
     xprintf(this->stream->xine, XINE_VERBOSITY_LOG, _("%s: joining metronom thread ...\n"), LOG_MODULE);
 
@@ -1910,13 +1920,13 @@ static inline const char *mrl_to_host (const char *mrl)
 static int vdr_plugin_open_fifo_mrl(input_plugin_t *this_gen)
 {
   vdr_input_plugin_t *this = (vdr_input_plugin_t *)this_gen;
-  char *filename = (char *)mrl_to_fifo (this->mrl);
+  const char *fifoname = mrl_to_fifo (this->mrl);
 
-  if(!strcmp(filename, "/")) {
-	  filename = (char *)VDR_ABS_FIFO_DIR "/stream";
+  if(!strcmp(fifoname, "/")) {
+    fifoname = VDR_ABS_FIFO_DIR "/stream";
   }
 
-  filename = strdup(filename);
+  char *filename = strdup(fifoname);
 
   _x_mrl_unescape (filename);
   this->fh = xine_open_cloexec(filename, O_RDONLY | O_NONBLOCK);
@@ -2054,6 +2064,7 @@ static int vdr_plugin_open_socket(vdr_input_plugin_t *this, struct hostent *host
     xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
             _("%s: failed to connect to port %d (%s)\n"), LOG_MODULE, port,
             strerror(errno));
+    close(fd);
 
     return -1;
   }
@@ -2198,6 +2209,7 @@ static int vdr_plugin_open(input_plugin_t *this_gen)
 
       return 0;
     }
+    this->metronom_thread_created = 1;
 
     this->rpc_thread_shutdown = 0;
 
@@ -2215,6 +2227,7 @@ static int vdr_plugin_open(input_plugin_t *this_gen)
 
       return 0;
     }
+    this->rpc_thread_created = 1;
   }
 
   /*
@@ -2390,7 +2403,8 @@ fprintf(stderr, "A =============================================\n");
     if (!this->vpts_offset_queue
       || this->vpts_offset_queue_tail->vpts < vpts)
     {
-      vdr_vpts_offset_t *curr = (vdr_vpts_offset_t *)calloc(1, sizeof (vdr_vpts_offset_t));
+      vdr_vpts_offset_t *curr = calloc(1, sizeof (vdr_vpts_offset_t));
+      if (curr) {
       curr->vpts = vpts;
       curr->offset = vpts_offset;
 
@@ -2401,6 +2415,7 @@ fprintf(stderr, "A =============================================\n");
         this->vpts_offset_queue_tail->next = curr;
         this->vpts_offset_queue_tail = curr;
       }
+      }
     }
   }
   else
@@ -2629,9 +2644,9 @@ static input_plugin_t *vdr_class_get_instance(input_class_t *cls_gen, xine_strea
   char               *mrl = strdup(data);
 
   if (!strncasecmp(mrl, "vdr:/", 5))
-    lprintf("filename '%s'\n", mrl_to_path (mrl));
+    lprintf("filename '%s'\n", mrl_to_fifo (mrl));
   else if (!strncasecmp(mrl, "netvdr:/", 5))
-    lprintf("host '%s'\n", mrl_to_socket (mrl));
+    lprintf("host '%s'\n", mrl_to_host (mrl));
   else
   {
     free(mrl);
@@ -2644,7 +2659,11 @@ static input_plugin_t *vdr_class_get_instance(input_class_t *cls_gen, xine_strea
    * => create plugin instance
    */
 
-  this = (vdr_input_plugin_t *)xine_xmalloc(sizeof (vdr_input_plugin_t));
+  this = calloc(1, sizeof (vdr_input_plugin_t));
+  if (!this) {
+    free(mrl);
+    return NULL;
+  }
 
   this->stream     = stream;
   this->curpos     = 0;
@@ -2760,7 +2779,10 @@ void *vdr_input_init_plugin(xine_t *xine, void *data)
 
   lprintf("init_class\n");
 
-  this = (vdr_input_class_t *)xine_xmalloc(sizeof (vdr_input_class_t));
+  this = calloc(1, sizeof (vdr_input_class_t));
+  if (!this) {
+    return NULL;
+  }
 
   this->xine = xine;
 
