@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2013 the xine project
+ * Copyright (C) 2006-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -29,8 +29,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
 #include <unistd.h>
 #include <stdio.h>
 
