@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2016 the xine project
+ * Copyright (C) 2003-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -28,6 +28,7 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <unistd.h>
+#include <pthread.h>
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -123,6 +124,13 @@ static vo_frame_t *caca_alloc_frame(vo_driver_t *this_gen) {
   if (!frame)
     return NULL;
 
+  /* colorspace converter for this frame */
+  frame->yuv2rgb = this->yuv2rgb_factory->create_converter (this->yuv2rgb_factory);
+  if (!frame->yuv2rgb) {
+    free(frame);
+    return NULL;
+  }
+
   pthread_mutex_init(&frame->vo_frame.mutex, NULL);
 
   frame->vo_frame.proc_slice = NULL;
@@ -131,10 +139,6 @@ static vo_frame_t *caca_alloc_frame(vo_driver_t *this_gen) {
   frame->vo_frame.dispose = caca_dispose_frame;
   frame->vo_frame.driver = this_gen;
 
-  /* colorspace converter for this frame */
-  frame->yuv2rgb =
-    this->yuv2rgb_factory->create_converter (this->yuv2rgb_factory);
-
   return (vo_frame_t*) frame;
 }
 
@@ -151,7 +155,7 @@ static void caca_update_frame_format (vo_driver_t *this_gen, vo_frame_t *img,
     xine_freep_aligned (&frame->vo_frame.base[1]);
     xine_freep_aligned (&frame->vo_frame.base[2]);
 
-    free (frame->pixmap_d); frame->pixmap_d = NULL;
+    _x_freep (&frame->pixmap_d);
 
     if (frame->pixmap_s) {
       cucul_free_dither (frame->pixmap_s);
@@ -249,9 +253,10 @@ static void caca_get_property_min_max (vo_driver_t *this_gen,
 static void caca_dispose_driver (vo_driver_t *this_gen) {
   caca_driver_t *this = (caca_driver_t*) this_gen;
   this->yuv2rgb_factory->dispose (this->yuv2rgb_factory);
-     caca_free_display(this->dp);
-    cucul_free_canvas(this->cv);
+  caca_free_display(this->dp);
+  cucul_free_canvas(this->cv);
 
+  free(this);
 }
 
 static int caca_redraw_needed (vo_driver_t *this_gen) {
@@ -283,6 +288,11 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *vi
   this->vo_driver.dispose              = caca_dispose_driver;
 
   this->yuv2rgb_factory = yuv2rgb_factory_init(MODE_32_RGB, 0, NULL);
+  if (!this->yuv2rgb_factory) {
+    xprintf (this->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": yuv2rgb initialization failed\n");
+    free(this);
+    return NULL;
+  }
   this->yuv2rgb_factory->set_csc_levels(this->yuv2rgb_factory, 0, 128, 128, CM_DEFAULT);
 
   if (dp) {
