@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2011 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -25,6 +25,8 @@
 #include "config.h"
 #endif
 
+#define XINE_NEED_WIN32_VISUAL
+
 typedef unsigned char boolean;
 
 #include <windows.h>
@@ -42,16 +44,13 @@ typedef unsigned char boolean;
 #include <xine/video_out.h>
 #include <xine/xine_internal.h>
 
-#include <xine/xine_internal.h>
 #include "yuv2rgb.h"
 
-#define NEW_YUV 1
-
 /* Set to 1 for RGB support */
 #define RGB_SUPPORT          0
 
-#define BORDER_SIZE	     8
-#define IMGFMT_NATIVE	     4
+#define BORDER_SIZE          8
+#define IMGFMT_NATIVE        4
 
 /*****************************************************************************
  * DirectDraw GUIDs.
@@ -60,7 +59,7 @@ typedef unsigned char boolean;
  *****************************************************************************/
 #if 1
 static const GUID xine_IID_IDirectDraw = {
-	0x6C14DB80,0xA733,0x11CE,{0xA5,0x21,0x00,0x20,0xAF,0x0B,0xE5,0x60}
+        0x6C14DB80,0xA733,0x11CE,{0xA5,0x21,0x00,0x20,0xAF,0x0B,0xE5,0x60}
 };
 #ifdef IID_IDirectDraw
 #  undef IID_IDirectDraw
@@ -70,7 +69,7 @@ static const GUID xine_IID_IDirectDraw = {
 
 #if 0
 static const GUID IID_IDirectDraw2 = {
-	0xB3A6F3E0,0x2B43,0x11CF,{0xA2,0xDE,0x00,0xAA,0x00,0xB9,0x33,0x56}
+        0xB3A6F3E0,0x2B43,0x11CF,{0xA2,0xDE,0x00,0xAA,0x00,0xB9,0x33,0x56}
 };
 #ifdef IID_IDirectDraw2
 #  undef IID_IDirectDraw2
@@ -80,7 +79,7 @@ static const GUID IID_IDirectDraw2 = {
 
 #if 0
 static const GUID IID_IDirectDraw4 = {
-	0x9C59509A,0x39BD,0x11D1,{0x8C,0x4A,0x00,0xC0,0x4F,0xD9,0x30,0xC5}
+        0x9C59509A,0x39BD,0x11D1,{0x8C,0x4A,0x00,0xC0,0x4F,0xD9,0x30,0xC5}
 };
 #ifdef IID_IDirectDraw4
 #  undef IID_IDirectDraw4
@@ -97,11 +96,11 @@ static const GUID IID_IDirectDraw4 = {
 typedef struct {
   vo_frame_t               vo_frame;
   uint8_t                 *buffer;
-  int			   format;
-  uint32_t			   width;
-  uint32_t			   height;
-  int		           size;
-  double	           ratio;
+  int                      format;
+  uint32_t                         width;
+  uint32_t                         height;
+  int                      size;
+  double                   ratio;
 } win32_frame_t;
 
 /* -----------------------------------------
@@ -117,31 +116,34 @@ typedef enum {
 } vo_directx_hwaccel_enum;
 
 typedef struct {
-  vo_driver_t		   vo_driver;
-  win32_visual_t          *win32_visual;
+  vo_driver_t              vo_driver;
+
+  const win32_visual_t    *win32_visual;
+  HWND                     WndHnd;
+  RECT                     WndRect;
 
   xine_t                  *xine;
 
-  LPDIRECTDRAW		   ddobj;	    /* direct draw object */
-  LPDIRECTDRAWSURFACE	   primary;	    /* primary dd surface */
-  LPDIRECTDRAWSURFACE	   secondary;	    /* secondary dd surface  */
-  LPDIRECTDRAWCLIPPER	   ddclipper;	    /* dd clipper object */
-  uint8_t *		   contents;	    /* secondary contents */
+  LPDIRECTDRAW             ddobj;           /* direct draw object */
+  LPDIRECTDRAWSURFACE      primary;         /* primary dd surface */
+  LPDIRECTDRAWSURFACE      secondary;       /* secondary dd surface  */
+  LPDIRECTDRAWCLIPPER      ddclipper;       /* dd clipper object */
+  uint8_t *                contents;        /* secondary contents */
   win32_frame_t           *current;         /* current frame */
 
-  int			   req_format;	    /* requested frame format */
-  int			   act_format;	    /* actual frame format */
-  uint32_t			   width;	    /* frame with */
-  uint32_t			   height;	    /* frame height */
-  double		   ratio;	    /* frame ratio */
+  int                      req_format;      /* requested frame format */
+  int                      act_format;      /* actual frame format */
+  uint32_t                         width;           /* frame with */
+  uint32_t                         height;          /* frame height */
+  double                   ratio;           /* frame ratio */
   vo_directx_hwaccel_enum  hwaccel;         /* requested level of HW acceleration */
 
   yuv2rgb_factory_t       *yuv2rgb_factory; /* used for format conversion */
   yuv2rgb_t               *yuv2rgb;         /* used for format conversion */
-  int			   mode;	    /* rgb mode */
-  int		           bytespp;	    /* rgb bits per pixel */
+  int                      mode;            /* rgb mode */
+  int                      bytespp;         /* rgb bits per pixel */
   DDPIXELFORMAT primary_pixel_format;
-  DDSURFACEDESC	ddsd; /* set by Lock(), used during display_frame */
+  DDSURFACEDESC ddsd; /* set by Lock(), used during display_frame */
   alphablend_t             alphablend_extra_data;
 } win32_driver_t;
 
@@ -151,7 +153,6 @@ typedef struct {
   xine_t                  *xine;
 } directx_class_t;
 
-char *config_hwaccel_values[] = {"full", "scale", "none", NULL };
 
 /* -----------------------------------------
  *
@@ -163,30 +164,43 @@ char *config_hwaccel_values[] = {"full", "scale", "none", NULL };
 /* Display formatted error message in
  * popup message box.*/
 
-static void Error( HWND hwnd, LPSTR szfmt, ... )
+static void Error( HWND hwnd, LPCSTR szfmt, ... )
 {
   char tempbuff[ 256 ];
   *tempbuff = 0;
-  wvsprintf(	&tempbuff[ strlen( tempbuff ) ], szfmt, ( char * )( &szfmt + 1 ) );
+  wvsprintf(    &tempbuff[ strlen( tempbuff ) ], szfmt, ( char * )( &szfmt + 1 ) );
   MessageBox( hwnd, tempbuff, "Error", MB_ICONERROR | MB_OK | MB_APPLMODAL | MB_SYSTEMMODAL );
 }
 
 /* Update our drivers current knowledge
  * of our windows video out posistion */
 
-static void UpdateRect( win32_visual_t * win32_visual )
+static void UpdateRect( HWND WndHnd, RECT *rect, const win32_visual_t * win32_visual )
 {
   if( win32_visual->FullScreen )
     {
-      SetRect( &win32_visual->WndRect, 0, 0,
-	       GetSystemMetrics( SM_CXSCREEN ),
-	       GetSystemMetrics( SM_CYSCREEN ) );
+      SetRect( rect, 0, 0,
+               GetSystemMetrics( SM_CXSCREEN ),
+               GetSystemMetrics( SM_CYSCREEN ) );
     }
   else
     {
-      GetClientRect( win32_visual->WndHnd, &win32_visual->WndRect );
-      ClientToScreen( win32_visual->WndHnd, ( POINT * ) &win32_visual->WndRect );
-      ClientToScreen( win32_visual->WndHnd, ( POINT * ) &win32_visual->WndRect + 1 );
+      POINT p0, p1;
+
+      GetClientRect( WndHnd, rect );
+
+      p0.x = rect->left;
+      p0.y = rect->top;
+      p1.x = rect->right;
+      p1.y = rect->bottom;
+
+      ClientToScreen( WndHnd, &p0);
+      ClientToScreen( WndHnd, &p1);
+
+      rect->left   = p0.x;
+      rect->top    = p0.y;
+      rect->right  = p1.x;
+      rect->bottom = p1.y;
     }
 }
 
@@ -200,9 +214,9 @@ static void UpdateRect( win32_visual_t * win32_visual )
 
 static boolean CreatePrimary( win32_driver_t * win32_driver )
 {
-  LPDIRECTDRAW			ddobj;
-  DDSURFACEDESC			ddsd;
-  HRESULT					result;
+  LPDIRECTDRAW                  ddobj;
+  DDSURFACEDESC                 ddsd;
+  HRESULT                                       result;
 
   /* create direct draw object */
 
@@ -216,7 +230,7 @@ static boolean CreatePrimary( win32_driver_t * win32_driver )
 
   /* set cooperative level */
 
-  result = IDirectDraw_SetCooperativeLevel( ddobj, win32_driver->win32_visual->WndHnd, DDSCL_NORMAL );
+  result = IDirectDraw_SetCooperativeLevel( ddobj, win32_driver->WndHnd, DDSCL_NORMAL );
   if( result != DD_OK )
     {
       Error( 0, "SetCooperativeLevel : error 0x%lx", result );
@@ -261,7 +275,7 @@ static boolean CreatePrimary( win32_driver_t * win32_driver )
 
   /* associate our clipper with our window */
 
-  result = IDirectDrawClipper_SetHWnd( win32_driver->ddclipper, 0, win32_driver->win32_visual->WndHnd );
+  result = IDirectDrawClipper_SetHWnd( win32_driver->ddclipper, 0, win32_driver->WndHnd );
   if( result != DD_OK )
     {
       Error( 0, "ddclipper->SetHWnd : error 0x%lx", result );
@@ -279,7 +293,7 @@ static boolean CreatePrimary( win32_driver_t * win32_driver )
 
   /* store our objects in our visual struct */
 
-  UpdateRect( win32_driver->win32_visual );
+  UpdateRect( win32_driver->WndHnd, &win32_driver->WndRect, win32_driver->win32_visual );
 
   return 1;
 }
@@ -316,9 +330,9 @@ static boolean CreateSecondary( win32_driver_t * win32_driver, int width, int he
   /* store our reqested format,
    * width and height */
 
-  win32_driver->req_format	= format;
-  win32_driver->width			= width;
-  win32_driver->height		= height;
+  win32_driver->req_format      = format;
+  win32_driver->width                   = width;
+  win32_driver->height          = height;
 
   /* if we already have a secondary
    * surface then release it */
@@ -440,6 +454,12 @@ static void Destroy( win32_driver_t * win32_driver )
   if( win32_driver->ddobj )
     IDirectDraw_Release( win32_driver->ddobj );
 
+  if (win32_driver->yuv2rgb)
+    win32_driver->yuv2rgb->dispose(win32_driver->yuv2rgb);
+
+  if (win32_driver->yuv2rgb_factory)
+    win32_driver->yuv2rgb_factory->dispose(win32_driver->yuv2rgb_factory);
+
   _x_alphablend_free(&win32_driver->alphablend_extra_data);
 
   free( win32_driver );
@@ -452,8 +472,8 @@ static void Destroy( win32_driver_t * win32_driver )
 
 static boolean CheckPixelFormat( win32_driver_t * win32_driver )
 {
-  DDPIXELFORMAT	ddpf;
-  HRESULT			result;
+  DDPIXELFORMAT ddpf;
+  HRESULT                       result;
 
   /* get the pixel format of our primary surface */
 
@@ -489,36 +509,36 @@ static boolean CheckPixelFormat( win32_driver_t * win32_driver )
   if( ddpf.dwRGBBitCount == 32 )
     {
       if( ddpf.dwRBitMask == 0xff0000 )
-	win32_driver->mode = MODE_32_RGB;
+        win32_driver->mode = MODE_32_RGB;
       else
-	win32_driver->mode = MODE_32_BGR;
+        win32_driver->mode = MODE_32_BGR;
     }
 
   if( ddpf.dwRGBBitCount == 24 )
     {
       if( ddpf.dwRBitMask == 0xff0000 )
-	win32_driver->mode = MODE_24_RGB;
+        win32_driver->mode = MODE_24_RGB;
       else
-	win32_driver->mode = MODE_24_BGR;
+        win32_driver->mode = MODE_24_BGR;
     }
 
   if( ddpf.dwRGBBitCount == 16 )
     {
       if( ddpf.dwRBitMask == 0xf800 )
-	win32_driver->mode = MODE_16_RGB;
+        win32_driver->mode = MODE_16_RGB;
       else
-	win32_driver->mode = MODE_16_BGR;
+        win32_driver->mode = MODE_16_BGR;
     }
 
   if( ddpf.dwRGBBitCount == 15 )
     {
       if( ddpf.dwRBitMask == 0x7C00 )
-	win32_driver->mode = MODE_15_RGB;
+        win32_driver->mode = MODE_15_RGB;
       else
-	win32_driver->mode = MODE_15_BGR;
+        win32_driver->mode = MODE_15_BGR;
     }
 
-	lprintf("win32 mode: %u\n", win32_driver->mode);
+        lprintf("win32 mode: %u\n", win32_driver->mode);
   return TRUE;
 }
 
@@ -533,12 +553,12 @@ static boolean CheckPixelFormat( win32_driver_t * win32_driver )
 
 static LPDIRECTDRAWSURFACE CreateBMP( win32_driver_t * win32_driver, int resource )
 {
-  LPDIRECTDRAWSURFACE	bmp_surf;
-  DDSURFACEDESC	bmp_ddsd;
-  HBITMAP		bmp_hndl;
-  BITMAP		bmp_head;
-  HDC			hdc_dds;
-  HDC			hdc_mem;
+  LPDIRECTDRAWSURFACE   bmp_surf;
+  DDSURFACEDESC bmp_ddsd;
+  HBITMAP               bmp_hndl;
+  BITMAP                bmp_head;
+  HDC                   hdc_dds;
+  HDC                   hdc_mem;
 
   /* load our bitmap from a resource */
 
@@ -578,7 +598,7 @@ static LPDIRECTDRAWSURFACE CreateBMP( win32_driver_t * win32_driver, int resourc
    * into our dd surface */
 
   BitBlt( hdc_dds, 0, 0, bmp_head.bmWidth, bmp_head.bmHeight,
-	  hdc_mem, 0, 0, SRCCOPY );
+          hdc_mem, 0, 0, SRCCOPY );
 
   /* clean up */
 
@@ -596,14 +616,14 @@ static LPDIRECTDRAWSURFACE CreateBMP( win32_driver_t * win32_driver, int resourc
  * is supported. */
 
 static boolean Overlay( LPDIRECTDRAWSURFACE src_surface, RECT * src_rect,
-		 LPDIRECTDRAWSURFACE dst_surface, RECT * dst_rect,
-		 COLORREF color_key )
+                 LPDIRECTDRAWSURFACE dst_surface, RECT * dst_rect,
+                 COLORREF color_key )
 {
-  DWORD			dw_color_key;
-  DDPIXELFORMAT		ddpf;
-  DDOVERLAYFX		ddofx;
-  int			flags;
-  HRESULT		result;
+  DWORD                 dw_color_key;
+  DDPIXELFORMAT         ddpf;
+  DDOVERLAYFX           ddofx;
+  int                   flags;
+  HRESULT               result;
 
   /* compute the colorkey pixel value from the RGB value we've got/
    * NOTE : based on videolan colorkey code */
@@ -635,17 +655,17 @@ static boolean Overlay( LPDIRECTDRAWSURFACE src_surface, RECT * src_rect,
   if( result != DD_OK )
     {
       if( result == DDERR_SURFACELOST )
-	{
-	  IDirectDrawSurface_Restore( src_surface );
-	  IDirectDrawSurface_Restore( dst_surface );
+        {
+          IDirectDrawSurface_Restore( src_surface );
+          IDirectDrawSurface_Restore( dst_surface );
 
-	  IDirectDrawSurface_UpdateOverlay( src_surface, src_rect, dst_surface, dst_rect, flags, &ddofx );
-	}
+          IDirectDrawSurface_UpdateOverlay( src_surface, src_rect, dst_surface, dst_rect, flags, &ddofx );
+        }
       else
-	{
-	  Error( 0, "IDirectDrawSurface_UpdateOverlay : error 0x%lx. You can try disable hardware acceleration (option video.directx.hwaccel).", result );
-	  return FALSE;
-	}
+        {
+          Error( 0, "IDirectDrawSurface_UpdateOverlay : error 0x%lx. You can try disable hardware acceleration (option video.directx.hwaccel).", result );
+          return FALSE;
+        }
     }
 
   return TRUE;
@@ -657,10 +677,10 @@ static boolean Overlay( LPDIRECTDRAWSURFACE src_surface, RECT * src_rect,
  * not supported. */
 
 static boolean BltCopy( LPDIRECTDRAWSURFACE src_surface, RECT * src_rect,
-		 LPDIRECTDRAWSURFACE dst_surface, RECT * dst_rect )
+                 LPDIRECTDRAWSURFACE dst_surface, RECT * dst_rect )
 {
-  DDSURFACEDESC	ddsd_target;
-  HRESULT	result;
+  DDSURFACEDESC ddsd_target;
+  HRESULT       result;
 
   memset( &ddsd_target, 0, sizeof( ddsd_target ) );
   ddsd_target.dwSize = sizeof( ddsd_target );
@@ -671,17 +691,17 @@ static boolean BltCopy( LPDIRECTDRAWSURFACE src_surface, RECT * src_rect,
   if( result != DD_OK )
     {
       if( result != DDERR_SURFACELOST )
-	{
-	  IDirectDrawSurface_Restore( src_surface );
-	  IDirectDrawSurface_Restore( dst_surface );
+        {
+          IDirectDrawSurface_Restore( src_surface );
+          IDirectDrawSurface_Restore( dst_surface );
 
-	  IDirectDrawSurface_Blt( dst_surface, dst_rect, src_surface, src_rect, DDBLT_WAIT, 0 );
-	}
+          IDirectDrawSurface_Blt( dst_surface, dst_rect, src_surface, src_rect, DDBLT_WAIT, 0 );
+        }
       else
-	{
-	  Error( 0, "IDirectDrawSurface_Blt : error 0x%lx", result );
-	  return FALSE;
-	}
+        {
+          Error( 0, "IDirectDrawSurface_Blt : error 0x%lx", result );
+          return FALSE;
+        }
     }
 
   return TRUE;
@@ -695,21 +715,21 @@ static boolean BltCopy( LPDIRECTDRAWSURFACE src_surface, RECT * src_rect,
 
 static boolean DisplayFrame( win32_driver_t * win32_driver )
 {
-  int					view_width;
-  int					view_height;
-  int					scaled_width;
-  int					scaled_height;
-  int					screen_width;
-  int					screen_height;
-  RECT					clipped;
-  RECT					centered;
+  int                                   view_width;
+  int                                   view_height;
+  int                                   scaled_width;
+  int                                   scaled_height;
+  int                                   screen_width;
+  int                                   screen_height;
+  RECT                                  clipped;
+  RECT                                  centered;
 
   /* aspect ratio calculations */
 
   /* TODO : account for screen ratio as well */
 
-  view_width	= win32_driver->win32_visual->WndRect.right - win32_driver->win32_visual->WndRect.left;
-  view_height	= win32_driver->win32_visual->WndRect.bottom - win32_driver->win32_visual->WndRect.top;
+  view_width    = win32_driver->WndRect.right - win32_driver->WndRect.left;
+  view_height   = win32_driver->WndRect.bottom - win32_driver->WndRect.top;
 
   if( view_width / win32_driver->ratio < view_height )
     {
@@ -724,15 +744,15 @@ static boolean DisplayFrame( win32_driver_t * win32_driver )
 
   /* center our overlay in our view frame */
 
-  centered.left = ( view_width - scaled_width ) / 2 + win32_driver->win32_visual->WndRect.left;
+  centered.left = ( view_width - scaled_width ) / 2 + win32_driver->WndRect.left;
   centered.right = centered.left + scaled_width;
-  centered.top = ( view_height - scaled_height ) / 2 + win32_driver->win32_visual->WndRect.top;
+  centered.top = ( view_height - scaled_height ) / 2 + win32_driver->WndRect.top;
   centered.bottom = centered.top + scaled_height;
 
   /* clip our overlay if it is off screen */
 
-  screen_width	= GetSystemMetrics( SM_CXSCREEN );
-  screen_height	= GetSystemMetrics( SM_CYSCREEN );
+  screen_width  = GetSystemMetrics( SM_CXSCREEN );
+  screen_height = GetSystemMetrics( SM_CYSCREEN );
 
   if( centered.left < 0 )
     {
@@ -800,8 +820,8 @@ static boolean DisplayFrame( win32_driver_t * win32_driver )
 
 static void * Lock( win32_driver_t * win32_driver, void * surface )
 {
-  LPDIRECTDRAWSURFACE	lock_surface = ( LPDIRECTDRAWSURFACE ) surface;
-  HRESULT		result;
+  LPDIRECTDRAWSURFACE   lock_surface = ( LPDIRECTDRAWSURFACE ) surface;
+  HRESULT               result;
 
   if( !surface )
     return 0;
@@ -816,16 +836,16 @@ static void * Lock( win32_driver_t * win32_driver, void * surface )
       result = IDirectDrawSurface_Lock( lock_surface, 0, &win32_driver->ddsd, DDLOCK_WAIT | DDLOCK_NOSYSLOCK, 0 );
 
       if( result != DD_OK )
-	return 0;
+        return 0;
 
     }
   else if( result != DD_OK )
     {
       if( result == DDERR_GENERIC )
-	{
-	  Error( 0, "surface->Lock : error, DDERR_GENERIC" );
-	  exit( 1 );
-	}
+        {
+          Error( 0, "surface->Lock : error, DDERR_GENERIC" );
+          exit( 1 );
+        }
     }
 
   return win32_driver->ddsd.lpSurface;
@@ -922,7 +942,7 @@ static vo_frame_t * win32_alloc_frame( vo_driver_t * vo_driver )
 
 
 static void win32_update_frame_format( vo_driver_t * vo_driver, vo_frame_t * vo_frame, uint32_t width,
-				       uint32_t height, double ratio, int format, int flags )
+                                       uint32_t height, double ratio, int format, int flags )
 {
   win32_driver_t  *win32_driver = ( win32_driver_t * ) vo_driver;
   win32_frame_t   *win32_frame  = ( win32_frame_t * ) vo_frame;
@@ -930,9 +950,9 @@ static void win32_update_frame_format( vo_driver_t * vo_driver, vo_frame_t * vo_
   /*printf("vo_out_directx : win32_update_frame_format() - width = %d, height=%d, ratio_code=%d, format=%d, flags=%d\n", width, height, ratio_code, format, flags);*/
 
 
-  if( ( win32_frame->format	!= format	) ||
-      ( win32_frame->width	!= width	) ||
-      ( win32_frame->height	!= height	) )
+  if( ( win32_frame->format     != format       ) ||
+      ( win32_frame->width      != width        ) ||
+      ( win32_frame->height     != height       ) )
     {
       /* free our allocated memory */
 
@@ -940,46 +960,46 @@ static void win32_update_frame_format( vo_driver_t * vo_driver, vo_frame_t * vo_
 
       /* create new render buffer */
       if( format == XINE_IMGFMT_YV12 )
-	{
-	  win32_frame->vo_frame.pitches[0] = 8*((width + 7) / 8);
-	  win32_frame->vo_frame.pitches[1] = 8*((width + 15) / 16);
-	  win32_frame->vo_frame.pitches[2] = 8*((width + 15) / 16);
+        {
+          win32_frame->vo_frame.pitches[0] = 8*((width + 7) / 8);
+          win32_frame->vo_frame.pitches[1] = 8*((width + 15) / 16);
+          win32_frame->vo_frame.pitches[2] = 8*((width + 15) / 16);
 
-	  win32_frame->vo_frame.base[0] = malloc(win32_frame->vo_frame.pitches[0] * height);
-	  win32_frame->vo_frame.base[1] = malloc(win32_frame->vo_frame.pitches[1] * ((height+1)/2));
-	  win32_frame->vo_frame.base[2] = malloc(win32_frame->vo_frame.pitches[2] * ((height+1)/2));
+          win32_frame->vo_frame.base[0] = malloc(win32_frame->vo_frame.pitches[0] * height);
+          win32_frame->vo_frame.base[1] = malloc(win32_frame->vo_frame.pitches[1] * ((height+1)/2));
+          win32_frame->vo_frame.base[2] = malloc(win32_frame->vo_frame.pitches[2] * ((height+1)/2));
 
-	  win32_frame->size = win32_frame->vo_frame.pitches[0] * height * 2;
-	}
+          win32_frame->size = win32_frame->vo_frame.pitches[0] * height * 2;
+        }
       else if( format == XINE_IMGFMT_YUY2 )
-	{
-	  win32_frame->vo_frame.pitches[0] = 8*((width + 3) / 4);
+        {
+          win32_frame->vo_frame.pitches[0] = 8*((width + 3) / 4);
 
-	  win32_frame->vo_frame.base[0] = malloc(win32_frame->vo_frame.pitches[0] * height * 2);
-	  win32_frame->vo_frame.base[1] = NULL;
-	  win32_frame->vo_frame.base[2] = NULL;
+          win32_frame->vo_frame.base[0] = malloc(win32_frame->vo_frame.pitches[0] * height * 2);
+          win32_frame->vo_frame.base[1] = NULL;
+          win32_frame->vo_frame.base[2] = NULL;
 
-	  win32_frame->size = win32_frame->vo_frame.pitches[0] * height * 2;
-	}
+          win32_frame->size = win32_frame->vo_frame.pitches[0] * height * 2;
+        }
 #if RGB_SUPPORT
       else if( format == IMGFMT_RGB )
-	{
-	  win32_frame->size   = width * height * 3;
-	  win32_frame->buffer = malloc( win32_frame->size );
-	  vo_frame->base[0]   = win32_frame->buffer;
-	}
+        {
+          win32_frame->size   = width * height * 3;
+          win32_frame->buffer = malloc( win32_frame->size );
+          vo_frame->base[0]   = win32_frame->buffer;
+        }
 #endif
       else
-	{
-	  xprintf (win32_driver->xine, XINE_VERBOSITY_DEBUG,
-		   "vo_out_directx : !!! unsupported image format %04x !!!\n", format );
-	  exit (1);
-	}
+        {
+          xprintf (win32_driver->xine, XINE_VERBOSITY_DEBUG,
+                   "vo_out_directx : !!! unsupported image format %04x !!!\n", format );
+          exit (1);
+        }
 
-      win32_frame->format	= format;
-      win32_frame->width	= width;
-      win32_frame->height	= height;
-      win32_frame->ratio	= ratio;
+      win32_frame->format       = format;
+      win32_frame->width        = width;
+      win32_frame->height       = height;
+      win32_frame->ratio        = ratio;
     }
 
 }
@@ -993,9 +1013,9 @@ static void win32_display_frame( vo_driver_t * vo_driver, vo_frame_t * vo_frame
   /* if the required width, height or format has changed
    * then recreate the secondary buffer */
 
-  if( ( win32_driver->req_format	!= win32_frame->format	) ||
-      ( win32_driver->width		!= win32_frame->width	) ||
-      ( win32_driver->height		!= win32_frame->height	) )
+  if( ( win32_driver->req_format        != win32_frame->format  ) ||
+      ( win32_driver->width             != win32_frame->width   ) ||
+      ( win32_driver->height            != win32_frame->height  ) )
     {
       CreateSecondary( win32_driver, win32_frame->width, win32_frame->height, win32_frame->format );
     }
@@ -1026,64 +1046,46 @@ static void win32_display_frame( vo_driver_t * vo_driver, vo_frame_t * vo_frame
        * pixel format ... this is slow */
 
       if( win32_driver->req_format == XINE_IMGFMT_YV12 )
-	{
-	  /* convert from yv12 to native
-	   * screen pixel format */
+        {
+          /* convert from yv12 to native
+           * screen pixel format */
 
-#if NEW_YUV
-	  win32_driver->yuv2rgb->configure( win32_driver->yuv2rgb,
-					    win32_driver->width, win32_driver->height,
-					    win32_frame->vo_frame.pitches[0], win32_frame->vo_frame.pitches[1],
-					    win32_driver->width, win32_driver->height,
-					    win32_driver->width * win32_driver->bytespp);
-#else
-	  yuv2rgb_setup( win32_driver->yuv2rgb,
-			 win32_driver->width, win32_driver->height,
-			 win32_frame->vo_frame.pitches[0], win32_frame->vo_frame.pitches[1],
-			 win32_driver->width, win32_driver->height,
-			 win32_driver->width * win32_driver->bytespp );
+          win32_driver->yuv2rgb->configure( win32_driver->yuv2rgb,
+                                            win32_driver->width, win32_driver->height,
+                                            win32_frame->vo_frame.pitches[0], win32_frame->vo_frame.pitches[1],
+                                            win32_driver->width, win32_driver->height,
+                                            win32_driver->width * win32_driver->bytespp);
 
-#endif
-
-	  win32_driver->yuv2rgb->yuv2rgb_fun( win32_driver->yuv2rgb,
-					      win32_driver->contents,
-					      win32_frame->vo_frame.base[0],
-					      win32_frame->vo_frame.base[1],
-					      win32_frame->vo_frame.base[2] );
-	}
+          win32_driver->yuv2rgb->yuv2rgb_fun( win32_driver->yuv2rgb,
+                                              win32_driver->contents,
+                                              win32_frame->vo_frame.base[0],
+                                              win32_frame->vo_frame.base[1],
+                                              win32_frame->vo_frame.base[2] );
+        }
 
       if( win32_driver->req_format == XINE_IMGFMT_YUY2 )
-	{
-	  /* convert from yuy2 to native
-	   * screen pixel format */
-#if NEW_YUV
-	  win32_driver->yuv2rgb->configure( win32_driver->yuv2rgb,
-					    win32_driver->width, win32_driver->height,
-					    win32_frame->vo_frame.pitches[0], win32_frame->vo_frame.pitches[0] / 2,
-					    win32_driver->width, win32_driver->height,
-					    win32_driver->width * win32_driver->bytespp );
-#else
+        {
+          /* convert from yuy2 to native
+           * screen pixel format */
+          win32_driver->yuv2rgb->configure( win32_driver->yuv2rgb,
+                                            win32_driver->width, win32_driver->height,
+                                            win32_frame->vo_frame.pitches[0], win32_frame->vo_frame.pitches[0] / 2,
+                                            win32_driver->width, win32_driver->height,
+                                            win32_driver->width * win32_driver->bytespp );
 
-	  yuv2rgb_setup( win32_driver->yuv2rgb,
-			 win32_driver->width, win32_driver->height,
-			 win32_frame->vo_frame.pitches[0], win32_frame->vo_frame.pitches[0] / 2,
-			 win32_driver->width, win32_driver->height,
-			 win32_driver->width * win32_driver->bytespp );
-
-#endif
-	  win32_driver->yuv2rgb->yuy22rgb_fun( win32_driver->yuv2rgb,
-					       win32_driver->contents,
-					       win32_frame->vo_frame.base[0] );
-	}
+          win32_driver->yuv2rgb->yuy22rgb_fun( win32_driver->yuv2rgb,
+                                               win32_driver->contents,
+                                               win32_frame->vo_frame.base[0] );
+        }
 
 #if RGB_SUPPORT
       if( win32_driver->req_format == IMGFMT_RGB )
-	{
-	  /* convert from 24 bit rgb to native
-	   * screen pixel format */
+        {
+          /* convert from 24 bit rgb to native
+           * screen pixel format */
 
-	  /* TODO : rgb2rgb conversion */
-	}
+          /* TODO : rgb2rgb conversion */
+        }
 #endif
     }
   else
@@ -1097,7 +1099,7 @@ static void win32_display_frame( vo_driver_t * vo_driver, vo_frame_t * vo_frame
     uint8_t * dst = (uint8_t *)win32_driver->contents;
 
     switch(win32_frame->format)
-	{
+        {
       case XINE_IMGFMT_YV12:
         src = frame->base[0];
         for (line = 0; line < frame->height ; line++){
@@ -1119,18 +1121,18 @@ static void win32_display_frame( vo_driver_t * vo_driver, vo_frame_t * vo_frame
           src += vo_frame->pitches[1];
           dst += win32_driver->ddsd.lPitch/2;
         }
-	  break;
+          break;
 
-	case XINE_IMGFMT_YUY2:
-	default:
+        case XINE_IMGFMT_YUY2:
+        default:
       src = frame->base[0];
       for (line = 0; line < frame->height ; line++){
-	    xine_fast_memcpy( dst, src, frame->width*2);
-	    src += vo_frame->pitches[0];
-	    dst += win32_driver->ddsd.lPitch;
-	  }
-	  break;
-	}
+            xine_fast_memcpy( dst, src, frame->width*2);
+            src += vo_frame->pitches[0];
+            dst += win32_driver->ddsd.lPitch;
+          }
+          break;
+        }
   }
 
   /* unlock the surface  */
@@ -1163,9 +1165,9 @@ static void win32_overlay_blend( vo_driver_t * vo_driver, vo_frame_t * vo_frame,
   if( vo_overlay->rle )
     {
       if( vo_frame->format == XINE_IMGFMT_YV12 )
-	_x_blend_yuv( win32_frame->vo_frame.base, vo_overlay, win32_frame->width, win32_frame->height, win32_frame->vo_frame.pitches, &win32_driver->alphablend_extra_data );
+        _x_blend_yuv( win32_frame->vo_frame.base, vo_overlay, win32_frame->width, win32_frame->height, win32_frame->vo_frame.pitches, &win32_driver->alphablend_extra_data );
       else
-	_x_blend_yuy2( win32_frame->vo_frame.base[0], vo_overlay, win32_frame->width, win32_frame->height, win32_frame->vo_frame.pitches[0], &win32_driver->alphablend_extra_data );
+        _x_blend_yuy2( win32_frame->vo_frame.base[0], vo_overlay, win32_frame->width, win32_frame->height, win32_frame->vo_frame.pitches[0], &win32_driver->alphablend_extra_data );
     }
 }
 
@@ -1178,6 +1180,19 @@ static int win32_get_property( vo_driver_t * vo_driver, int property )
 
 static int win32_set_property( vo_driver_t * vo_driver, int property, int value )
 {
+  win32_driver_t *win32_driver = (win32_driver_t *)vo_driver;
+
+  if (property == VO_PROP_DISCARD_FRAMES) {
+    if (value == -1) {
+      value = 0;
+      if (win32_driver->current) {
+        win32_driver->current->vo_frame.free (&win32_driver->current->vo_frame);
+        win32_driver->current = NULL;
+        value = 1;
+      }
+    }
+  }
+
   return value;
 }
 
@@ -1195,7 +1210,7 @@ static int win32_gui_data_exchange( vo_driver_t * vo_driver, int data_type, void
     {
 
     case GUI_WIN32_MOVED_OR_RESIZED:
-      UpdateRect( win32_driver->win32_visual );
+      UpdateRect( win32_driver->WndHnd, &win32_driver->WndRect, win32_driver->win32_visual );
       DisplayFrame( win32_driver );
       break;
     case XINE_GUI_SEND_DRAWABLE_CHANGED:
@@ -1203,24 +1218,24 @@ static int win32_gui_data_exchange( vo_driver_t * vo_driver, int data_type, void
       HRESULT result;
       HWND newWndHnd = (HWND) data;
 
-	  /* set cooperative level */
-	  result = IDirectDraw_SetCooperativeLevel( win32_driver->ddobj, newWndHnd, DDSCL_NORMAL );
+          /* set cooperative level */
+          result = IDirectDraw_SetCooperativeLevel( win32_driver->ddobj, newWndHnd, DDSCL_NORMAL );
       if( result != DD_OK )
       {
         Error( 0, "SetCooperativeLevel : error 0x%lx", result );
         return 0;
       }
       /* associate our clipper with new window */
-	  result = IDirectDrawClipper_SetHWnd( win32_driver->ddclipper, 0, newWndHnd );
+          result = IDirectDrawClipper_SetHWnd( win32_driver->ddclipper, 0, newWndHnd );
       if( result != DD_OK )
       {
         Error( 0, "ddclipper->SetHWnd : error 0x%lx", result );
         return 0;
       }
       /* store our objects in our visual struct */
-	  win32_driver->win32_visual->WndHnd = newWndHnd;
-	  /* update video area and redraw current frame */
-      UpdateRect( win32_driver->win32_visual );
+      win32_driver->WndHnd = newWndHnd;
+      /* update video area and redraw current frame */
+      UpdateRect( win32_driver->WndHnd, &win32_driver->WndRect, win32_driver->win32_visual );
       DisplayFrame( win32_driver );
       break;
     }
@@ -1251,40 +1266,49 @@ static void win32_exit( vo_driver_t * vo_driver )
 {
   win32_driver_t  *win32_driver = ( win32_driver_t * ) vo_driver;
 
-  free(win32_driver->win32_visual);
-
   Destroy( win32_driver );
 }
 
-static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *win32_visual)
+static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *visual)
      /*vo_driver_t *init_video_out_plugin( config_values_t * config, void * win32_visual )*/
 {
-  directx_class_t *class = (directx_class_t *)class_gen;
-  win32_driver_t  *win32_driver = calloc(1, sizeof(win32_driver_t));
+  static const char * const config_hwaccel_values[] = {"full", "scale", "none", NULL };
+  const win32_visual_t *win32_visual = visual;
+  directx_class_t      *class = (directx_class_t *)class_gen;
+  win32_driver_t       *win32_driver;
 
+  win32_driver = calloc(1, sizeof(win32_driver_t));
+  if (!win32_driver || !visual)
+    return NULL;
 
   _x_alphablend_init(&win32_driver->alphablend_extra_data, class->xine);
 
   win32_driver->xine = class->xine;
 
+  win32_driver->WndHnd         = win32_visual->WndHnd;
+  win32_driver->WndRect.left   = win32_visual->WndRect.left;
+  win32_driver->WndRect.right  = win32_visual->WndRect.right;
+  win32_driver->WndRect.top    = win32_visual->WndRect.top;
+  win32_driver->WndRect.bottom = win32_visual->WndRect.bottom;
+
   /* Make sure that the DirectX drivers are available and present! */
   /* Not complete yet */
 
-  win32_driver->win32_visual			= (win32_visual_t *)win32_visual;
-  win32_driver->vo_driver.get_capabilities	= win32_get_capabilities;
-  win32_driver->vo_driver.alloc_frame		= win32_alloc_frame ;
-  win32_driver->vo_driver.update_frame_format	= win32_update_frame_format;
-  win32_driver->vo_driver.display_frame		= win32_display_frame;
-  win32_driver->vo_driver.overlay_blend		= win32_overlay_blend;
-  win32_driver->vo_driver.get_property		= win32_get_property;
-  win32_driver->vo_driver.set_property		= win32_set_property;
-  win32_driver->vo_driver.get_property_min_max	= win32_get_property_min_max;
-  win32_driver->vo_driver.gui_data_exchange	= win32_gui_data_exchange;
-  win32_driver->vo_driver.dispose		= win32_exit;
+  win32_driver->win32_visual                    = win32_visual;
+  win32_driver->vo_driver.get_capabilities      = win32_get_capabilities;
+  win32_driver->vo_driver.alloc_frame           = win32_alloc_frame ;
+  win32_driver->vo_driver.update_frame_format   = win32_update_frame_format;
+  win32_driver->vo_driver.display_frame         = win32_display_frame;
+  win32_driver->vo_driver.overlay_blend         = win32_overlay_blend;
+  win32_driver->vo_driver.get_property          = win32_get_property;
+  win32_driver->vo_driver.set_property          = win32_set_property;
+  win32_driver->vo_driver.get_property_min_max  = win32_get_property_min_max;
+  win32_driver->vo_driver.gui_data_exchange     = win32_gui_data_exchange;
+  win32_driver->vo_driver.dispose               = win32_exit;
   win32_driver->vo_driver.redraw_needed         = win32_redraw_needed;
 
   win32_driver->hwaccel = class->config->register_enum(class->config,
-    "video.directx.hwaccel", 0, config_hwaccel_values,
+    "video.directx.hwaccel", 0, (char**)config_hwaccel_values,
     _("HW acceleration level"),
     _("Possible values (default full):\n\n"
 "full: full acceleration\n"
@@ -1303,12 +1327,17 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *wi
       return NULL;
     }
 
-#if (NEW_YUV)
   win32_driver->yuv2rgb_factory = yuv2rgb_factory_init( win32_driver->mode, 0, 0 );
+  if (!win32_driver->yuv2rgb_factory) {
+    xprintf (win32_driver->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": yuv2rgb initialization failed\n");
+    Destroy (win32_driver);
+    return NULL;
+  }
   win32_driver->yuv2rgb = win32_driver->yuv2rgb_factory->create_converter(win32_driver->yuv2rgb_factory);
-#else
-  win32_driver->yuv2rgb = yuv2rgb_init( win32_driver->mode, 0, 0 );
-#endif
+  if (!win32_driver->yuv2rgb) {
+      Destroy( win32_driver );
+      return NULL;
+  }
 
   return ( vo_driver_t * ) win32_driver;
 }
@@ -1321,6 +1350,8 @@ static void *init_class (xine_t *xine, void *visual_gen) {
    * from this point on, nothing should go wrong anymore
    */
   directx = calloc(1, sizeof (directx_class_t));
+  if (!directx)
+    return NULL;
 
   directx->driver_class.open_plugin     = open_plugin;
   directx->driver_class.identifier      = "DirectX";
