@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2016 the xine project and Fredrik Noring
+ * Copyright (C) 2000-2017 the xine project and Fredrik Noring
  *
  * This file is part of xine, a free video player.
  *
@@ -148,7 +148,6 @@ typedef struct fb_driver_s
 typedef struct
 {
   video_driver_class_t driver_class;
-  config_values_t     *config;
   xine_t              *xine;
 } fb_class_t;
 
@@ -206,6 +205,8 @@ static void fb_frame_dispose(vo_frame_t *vo_img)
 {
   fb_frame_t *frame = (fb_frame_t *)vo_img;
 
+  frame->yuv2rgb->dispose (frame->yuv2rgb);
+
   if(!frame->this->use_zero_copy)
      free(frame->data);
   free(frame);
@@ -224,6 +225,13 @@ static vo_frame_t *fb_alloc_frame(vo_driver_t *this_gen)
   if(!frame)
     return NULL;
 
+  /* colorspace converter for this frame */
+  frame->yuv2rgb = this->yuv2rgb_factory->create_converter(this->yuv2rgb_factory);
+  if (!frame->yuv2rgb) {
+    free(frame);
+    return NULL;
+  }
+
   memcpy(&frame->sc, &this->sc, sizeof(vo_scale_t));
 
   pthread_mutex_init(&frame->vo_frame.mutex, NULL);
@@ -237,10 +245,6 @@ static vo_frame_t *fb_alloc_frame(vo_driver_t *this_gen)
 
   frame->this = this;
 
-  /* colorspace converter for this frame */
-  frame->yuv2rgb =
-    this->yuv2rgb_factory->create_converter(this->yuv2rgb_factory);
-
   if(this->use_zero_copy)
   {
     frame->yoffset = this->used_num_buffers * this->fb_var.yres;
@@ -589,6 +593,23 @@ static int fb_set_property(vo_driver_t *this_gen, int property, int value)
 
   switch(property)
   {
+    case VO_PROP_DISCARD_FRAMES:
+      if (value == -1) {
+        int n = 0;
+        if (this->old_frame) {
+          this->old_frame->vo_frame.free (&this->old_frame->vo_frame);
+          this->old_frame = NULL;
+          n++;
+        }
+        if (this->cur_frame) {
+          this->cur_frame->vo_frame.free (&this->cur_frame->vo_frame);
+          this->cur_frame = NULL;
+          n++;
+        }
+        value = n;
+      }
+      break;
+
     case VO_PROP_ASPECT_RATIO:
       if(value>=XINE_VO_ASPECT_NUM_RATIOS)
 	value = XINE_VO_ASPECT_AUTO;
@@ -664,11 +685,17 @@ static void fb_dispose(vo_driver_t *this_gen)
 {
   fb_driver_t *this = (fb_driver_t *)this_gen;
 
-  munmap(this->video_mem_base, this->mem_size);
-  close(this->fd);
+  if (this->video_mem_base != (void *)-1)
+    munmap(this->video_mem_base, this->mem_size);
+
+  if (this->fd >= 0)
+    close(this->fd);
 
   _x_alphablend_free(&this->alphablend_extra_data);
 
+  if (this->yuv2rgb_factory)
+    this->yuv2rgb_factory->dispose (this->yuv2rgb_factory);
+
   free(this);
 }
 
@@ -874,6 +901,8 @@ static int setup_yuv2rgb(fb_driver_t *this, config_values_t *config,
   this->yuv2rgb_factory = yuv2rgb_factory_init(this->yuv2rgb_mode,
 					       this->yuv2rgb_swap,
 					       this->yuv2rgb_cmap);
+  if (!this->yuv2rgb_factory)
+    return 0;
   this->yuv2rgb_factory->set_csc_levels (this->yuv2rgb_factory,
                                          this->yuv2rgb_brightness,
                                          this->yuv2rgb_contrast,
@@ -948,20 +977,18 @@ static vo_driver_t *fb_open_plugin(video_driver_class_t *class_gen,
   config_values_t *config;
   fb_driver_t *this;
   fb_class_t *class;
-  fb_visual_t *visual = NULL;
-
-  if (visual_gen) {
-    visual = (fb_visual_t *) visual_gen;
-  }
+  const fb_visual_t *visual = (const fb_visual_t *)visual_gen;
 
   class = (fb_class_t *)class_gen;
-  config = class->config;
+  config = class->xine->config;
 
   /* allocate plugin struct */
   this = calloc(1, sizeof(fb_driver_t));
   if(!this)
     return NULL;
 
+  this->video_mem_base = (void*)-1;
+
   _x_alphablend_init(&this->alphablend_extra_data, class->xine);
 
   register_callbacks(this);
@@ -1026,13 +1053,16 @@ static vo_driver_t *fb_open_plugin(video_driver_class_t *class_gen,
   /* mmap whole video memory */
   this->mem_size = this->fb_fix.smem_len;
   this->video_mem_base = mmap(0, this->mem_size, PROT_READ | PROT_WRITE,
-			      MAP_SHARED, this->fd, 0);
+                              MAP_SHARED, this->fd, 0);
+  if (this->video_mem_base == (void*)-1)
+    goto error;
+
   return &this->vo_driver;
+
 error:
   xprintf(class->xine, XINE_VERBOSITY_DEBUG,
           "video_out_fb: Unable to configure fb device, aborting: %s\n", strerror(errno));
-
-  free(this);
+  fb_dispose(&this->vo_driver);
   return 0;
 }
 
@@ -1045,7 +1075,6 @@ static void *fb_init_class(xine_t *xine, void *visual_gen)
   this->driver_class.description     = N_("Xine video output plugin using the Linux frame buffer device");
   this->driver_class.dispose         = default_video_driver_class_dispose;
 
-  this->config          = xine->config;
   this->xine            = xine;
 
   return this;
