@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2016 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -208,6 +208,7 @@ typedef struct {
 
   /* color matrix switching */
   int                cm_yuv2rgb, cm_fragprog, cm_state;
+  uint8_t            cm_lut[32];
 
   /* Frame state */
   opengl_frame_t    *frame[NUM_FRAMES_BACKLOG];
@@ -246,10 +247,20 @@ typedef struct {
 
 
 /* import common color matrix stuff */
+#define CM_LUT
 #define CM_DRIVER_T opengl_driver_t
 #include "color_matrix.c"
 
-extern const int32_t Inverse_Table_6_9[8][4]; /* from yuv2rgb.c */
+static const int32_t Inverse_Table_6_9[8][4] = {
+  {117504, 138453, 13954, 34903}, /* no sequence_display_extension */
+  {117504, 138453, 13954, 34903}, /* ITU-R Rec. 709 (1990) */
+  {104597, 132201, 25675, 53279}, /* unspecified */
+  {104597, 132201, 25675, 53279}, /* reserved */
+  {104448, 132798, 24759, 53109}, /* FCC */
+  {104597, 132201, 25675, 53279}, /* ITU-R Rec. 624-4 System B, G */
+  {104597, 132201, 25675, 53279}, /* SMPTE 170M */
+  {117579, 136230, 16907, 35559}  /* SMPTE 240M (1987) */
+};
 
 /*
  * Render functions
@@ -687,7 +698,7 @@ static int render_help_verify_ext (opengl_driver_t *this, const char *ext) {
   int ret = 0;
   const size_t l = strlen (ext);
   const char *e;
-  for (e = (char *) this->gl_exts; e && *e; e = strchr (e, ' ')) {
+  for (e = (const char *) this->gl_exts; e && *e; e = strchr (e, ' ')) {
     while (isspace (*e))
       e++;
     if (strncmp (e, ext, l) == 0 && (e[l] == 0 || e[l] == ' ')) {
@@ -1058,6 +1069,10 @@ static void render_gfx_vinfo (opengl_driver_t *this) {
     glxAttrib[9] = GLX_DOUBLEBUFFER;
   else
     glxAttrib[9] = None;
+
+  if (this->vinfo)
+    XFree(this->vinfo);
+
   this->vinfo = glXChooseVisual (this->display, this->screen, glxAttrib);
   CHECKERR ("choosevisual");
 }
@@ -1114,7 +1129,6 @@ static void *render_run (opengl_driver_t *this) {
     case RENDER_DRAW:
       this->render_action = RENDER_NONE;
       this->render_frame_changed = 0;
-      pthread_mutex_unlock (&this->render_action_mutex);
       if (this->context && frame) {
 	/* update fragprog if color matrix changed */
 	if (this->render_fun_id == 0) {
@@ -1122,9 +1136,11 @@ static void *render_run (opengl_driver_t *this) {
 	  if (cm != this->cm_fragprog) {
 	    this->cm_fragprog = cm;
 	    this->render_action = RENDER_SETUP;
+            pthread_mutex_unlock (&this->render_action_mutex);
 	    break;
 	  }
 	}
+        pthread_mutex_unlock (&this->render_action_mutex);
 	XLockDisplay (this->display);
 	CHECKERR ("pre-render");
 	ret = 1;
@@ -1145,6 +1161,8 @@ static void *render_run (opengl_driver_t *this) {
 	  if (render->fallback != -1 && this->gl_exts)
 	    this->config->update_num (this->config, "video.output.opengl_renderer", render->fallback);
 	}
+      } else {
+        pthread_mutex_unlock (&this->render_action_mutex);
       }
       break;
 
@@ -1361,6 +1379,7 @@ static void opengl_frame_dispose (vo_frame_t *vo_img) {
   xine_free_aligned (frame->vo_frame.base[1]);
   xine_free_aligned (frame->vo_frame.base[2]);
   xine_free_aligned (frame->rgb);
+  pthread_mutex_destroy (&frame->vo_frame.mutex);
   free (frame);
 }
 
@@ -1373,6 +1392,15 @@ static vo_frame_t *opengl_alloc_frame (vo_driver_t *this_gen) {
   if (!frame)
     return NULL;
 
+  /*
+   * colorspace converter for this frame
+   */
+  frame->yuv2rgb = this->yuv2rgb_factory->create_converter (this->yuv2rgb_factory);
+  if (!frame->yuv2rgb) {
+    free(frame);
+    return NULL;
+  }
+
   pthread_mutex_init (&frame->vo_frame.mutex, NULL);
 
   /*
@@ -1384,11 +1412,6 @@ static vo_frame_t *opengl_alloc_frame (vo_driver_t *this_gen) {
   frame->vo_frame.dispose    = opengl_frame_dispose;
   frame->vo_frame.driver     = this_gen;
 
-  /*
-   * colorspace converter for this frame
-   */
-  frame->yuv2rgb = this->yuv2rgb_factory->create_converter (this->yuv2rgb_factory);
-
   return (vo_frame_t *) frame;
 }
 
@@ -1716,6 +1739,20 @@ static int opengl_set_property (vo_driver_t *this_gen,
   opengl_driver_t *this = (opengl_driver_t *) this_gen;
 
   switch (property) {
+
+  case VO_PROP_DISCARD_FRAMES:
+    if (value == -1) {
+      int i, n = 0;
+      for (i = NUM_FRAMES_BACKLOG - 1; i >= 0; i--) {
+        if (this->frame[i]) {
+          this->frame[i]->vo_frame.free (&this->frame[i]->vo_frame);
+          this->frame[i] = NULL;
+          n++;
+        }
+      }
+      value = n;
+    }
+    break;
   case VO_PROP_ASPECT_RATIO:
     if (value>=XINE_VO_ASPECT_NUM_RATIOS)
       value = XINE_VO_ASPECT_AUTO;
@@ -1871,15 +1908,16 @@ static int opengl_gui_data_exchange (vo_driver_t *this_gen,
   return 0;
 }
 
-static void opengl_dispose (vo_driver_t *this_gen) {
-  opengl_driver_t *this = (opengl_driver_t *) this_gen;
+static void opengl_dispose_internal (opengl_driver_t *this, int thread_running) {
   int i;
 
-  pthread_mutex_lock    (&this->render_action_mutex);
-  this->render_action = RENDER_EXIT;
-  pthread_cond_signal   (&this->render_action_cond);
-  pthread_mutex_unlock  (&this->render_action_mutex);
-  pthread_join          (this->render_thread, NULL);
+  if (thread_running) {
+    pthread_mutex_lock    (&this->render_action_mutex);
+    this->render_action = RENDER_EXIT;
+    pthread_cond_signal   (&this->render_action_cond);
+    pthread_mutex_unlock  (&this->render_action_mutex);
+    pthread_join          (this->render_thread, NULL);
+  }
   pthread_mutex_destroy (&this->render_action_mutex);
   pthread_cond_destroy  (&this->render_action_cond);
   pthread_cond_destroy  (&this->render_return_cond);
@@ -1898,11 +1936,20 @@ static void opengl_dispose (vo_driver_t *this_gen) {
     XUnlockDisplay (this->display);
   }
 
+  if (this->vinfo) {
+    XFree(this->vinfo);
+  }
+
   _x_alphablend_free(&this->alphablend_extra_data);
 
   free (this);
 }
 
+static void opengl_dispose (vo_driver_t *this_gen) {
+  opengl_driver_t *this = (opengl_driver_t *) this_gen;
+  opengl_dispose_internal(this, 1);
+}
+
 static void opengl_cb_render_fun (void *this_gen, xine_cfg_entry_t *entry) {
   opengl_driver_t *this = (opengl_driver_t *) this_gen;
   pthread_mutex_lock (&this->render_action_mutex);
@@ -1922,7 +1969,7 @@ static void opengl_cb_default (void *val_gen, xine_cfg_entry_t *entry) {
 static vo_driver_t *opengl_open_plugin (video_driver_class_t *class_gen, const void *visual_gen) {
   opengl_class_t       *class   = (opengl_class_t *) class_gen;
   config_values_t      *config  = class->xine->config;
-  x11_visual_t         *visual  = (x11_visual_t *) visual_gen;
+  const x11_visual_t   *visual  = (const x11_visual_t *) visual_gen;
   opengl_driver_t      *this;
   const char          **render_fun_names;
   int                   i;
@@ -1932,6 +1979,13 @@ static vo_driver_t *opengl_open_plugin (video_driver_class_t *class_gen, const v
   if (!this)
     return NULL;
 
+  this->yuv2rgb_factory = yuv2rgb_factory_init (YUV_FORMAT, YUV_SWAP_MODE, NULL);
+  if (!this->yuv2rgb_factory) {
+    xprintf (this->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": yuv2rgb initialization failed\n");
+    free(this);
+    return NULL;
+  }
+
   this->display		    = visual->display;
   this->screen		    = visual->screen;
 
@@ -1973,8 +2027,6 @@ static vo_driver_t *opengl_open_plugin (video_driver_class_t *class_gen, const v
 
   cm_init (this);
 
-  this->yuv2rgb_factory = yuv2rgb_factory_init (YUV_FORMAT, YUV_SWAP_MODE, NULL);
-
   XLockDisplay (this->display);
   this->xoverlay = x11osd_create (this->xine, this->display, this->screen,
                                   this->drawable, X11OSD_SHAPED);
@@ -2006,6 +2058,7 @@ static vo_driver_t *opengl_open_plugin (video_driver_class_t *class_gen, const v
 						 "Environment_Mapped_Torus\n"
 						 "Show images reflected in a spinning torus. Way cool =)"),
 					       10, opengl_cb_render_fun, this);
+  free(render_fun_names);
   this->render_min_fps = config->register_range (config,
 						 "video.output.opengl_min_fps",
 						 20, 1, 120,
@@ -2024,7 +2077,11 @@ static vo_driver_t *opengl_open_plugin (video_driver_class_t *class_gen, const v
   pthread_mutex_init (&this->render_action_mutex, NULL);
   pthread_cond_init  (&this->render_action_cond, NULL);
   pthread_cond_init  (&this->render_return_cond, NULL);
-  pthread_create (&this->render_thread, NULL, (thread_run_t) render_run, this);
+  if (pthread_create (&this->render_thread, NULL, (thread_run_t) render_run, this)) {
+    xprintf (this->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": pthread_create() failed\n");
+    opengl_dispose_internal (this, 0);
+    return NULL;
+  }
 
   /* Check for OpenGL capable visual */
   pthread_mutex_lock   (&this->render_action_mutex);
@@ -2043,7 +2100,7 @@ static vo_driver_t *opengl_open_plugin (video_driver_class_t *class_gen, const v
 
   if (! this->vinfo) {
     /* no OpenGL capable visual available */
-    opengl_dispose (&this->vo_driver);
+    opengl_dispose_internal (this, 1);
     return NULL;
   }
   if (! this->context)
@@ -2079,8 +2136,10 @@ static int opengl_verify_direct (x11_visual_t *vis) {
   }
   if (! (visinfo = glXChooseVisual (vis->display, vis->screen, attribs)))
       return 0;
-  if (! (ctx = glXCreateContext (vis->display, visinfo, NULL, 1)))
+  if (! (ctx = glXCreateContext (vis->display, visinfo, NULL, 1))) {
+      XFree(visinfo);
       return 0;
+  }
   memset (&xattr, 0, sizeof (xattr));
   xattr.colormap = XCreateColormap(vis->display, root, visinfo->visual, AllocNone);
   xattr.event_mask = StructureNotifyMask | ExposureMask;
@@ -2100,6 +2159,7 @@ static int opengl_verify_direct (x11_visual_t *vis) {
   }
   glXDestroyContext (vis->display, ctx);
   XFreeColormap     (vis->display, xattr.colormap);
+  XFree             (visinfo);
 
   return ret;
 }
