@@ -156,6 +156,7 @@ typedef struct {
   int		            zoom_y;
 
   int                cm_state;
+  uint8_t            cm_lut[32];
 
   int                exit_indx;
   int                exiting;
@@ -224,6 +225,7 @@ static void opengl2_exit_register (opengl2_driver_t *this) {
 /* !exit_stuff */
 
 /* import common color matrix stuff */
+#define CM_LUT
 #define CM_HAVE_YCGCO_SUPPORT 1
 #define CM_DRIVER_T opengl2_driver_t
 #include "color_matrix.c"
@@ -731,6 +733,7 @@ static void opengl2_frame_dispose( vo_frame_t *vo_img )
   opengl2_frame_t  *frame = (opengl2_frame_t *) vo_img ;
 
   xine_free_aligned (frame->vo_frame.base[0]);
+  pthread_mutex_destroy (&frame->vo_frame.mutex);
   free (frame);
 }
 
@@ -1719,7 +1722,7 @@ static void opengl2_dispose( vo_driver_t *this_gen )
 static vo_driver_t *opengl2_open_plugin( video_driver_class_t *class_gen, const void *visual_gen )
 {
   opengl2_class_t     *class   = (opengl2_class_t *) class_gen;
-  x11_visual_t        *visual  = (x11_visual_t *) visual_gen;
+  const x11_visual_t  *visual  = (const x11_visual_t *) visual_gen;
   opengl2_driver_t    *this;
   config_values_t     *config  = class->xine->config;
 
@@ -1864,7 +1867,7 @@ static vo_driver_t *opengl2_open_plugin( video_driver_class_t *class_gen, const
 
 
 
-static int opengl2_check_platform( opengl2_class_t *this_gen, x11_visual_t *vis )
+static int opengl2_check_platform( opengl2_class_t *this_gen, const x11_visual_t *vis )
 {
   int attribs[] = { GLX_RGBA, GLX_DOUBLEBUFFER, GLX_RED_SIZE, 8,
     GLX_GREEN_SIZE, 8, GLX_BLUE_SIZE, 8, GLX_DEPTH_SIZE, 16, None };
@@ -1880,8 +1883,10 @@ static int opengl2_check_platform( opengl2_class_t *this_gen, x11_visual_t *vis
   if ( !( visinfo = glXChooseVisual( vis->display, vis->screen, attribs ) ) )
     return 0;
   
-  if ( !( ctx = glXCreateContext( vis->display, visinfo, NULL, GL_TRUE  ) ) )
+  if ( !( ctx = glXCreateContext( vis->display, visinfo, NULL, GL_TRUE  ) ) ) {
+    XFree( visinfo );
     return 0;
+  }
   
   if ( glXMakeCurrent( vis->display, root, ctx ) ) {
     if ( !glXIsDirect( vis->display, ctx ) )
@@ -1909,6 +1914,8 @@ static int opengl2_check_platform( opengl2_class_t *this_gen, x11_visual_t *vis
   else
     this_gen->ctx = ctx;    
 
+  XFree( visinfo );
+
   return ret;
 }
 
@@ -1920,7 +1927,7 @@ static void *opengl2_init_class( xine_t *xine, void *visual_gen )
 {
   opengl2_class_t *this = (opengl2_class_t *) calloc(1, sizeof(opengl2_class_t));
 
-  if ( !opengl2_check_platform( this, (x11_visual_t *)visual_gen ) ) {
+  if ( !opengl2_check_platform( this, (const x11_visual_t *)visual_gen ) ) {
     free(this);
     return NULL;
   }
@@ -1937,7 +1944,7 @@ static void *opengl2_init_class( xine_t *xine, void *visual_gen )
 
 
 static const vo_info_t vo_info_opengl2 = {
-  7,                    /* priority    */
+  8,                    /* priority    */
   XINE_VISUAL_TYPE_X11  /* visual type */
 };
 
