@@ -1,6 +1,6 @@
 
 /*
- * Copyright (C) 2007-2016 the xine project
+ * Copyright (C) 2007-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -235,6 +235,15 @@ static vo_frame_t *raw_alloc_frame (vo_driver_t *this_gen)
   if (!frame)
     return NULL;
 
+  /*
+   * colorspace converter for this frame
+   */
+  frame->yuv2rgb = this->yuv2rgb_factory->create_converter (this->yuv2rgb_factory);
+  if (!frame->yuv2rgb) {
+    free(frame);
+    return NULL;
+  }
+
   frame->vo_frame.base[0] = frame->vo_frame.base[1] = frame->vo_frame.base[2] = frame->rgb = NULL;
   frame->width = frame->height = frame->format = frame->flags = 0;
 
@@ -249,11 +258,6 @@ static vo_frame_t *raw_alloc_frame (vo_driver_t *this_gen)
   frame->vo_frame.dispose    = raw_frame_dispose;
   frame->vo_frame.driver     = this_gen;
 
-  /*
-   * colorspace converter for this frame
-   */
-  frame->yuv2rgb = this->yuv2rgb_factory->create_converter (this->yuv2rgb_factory);
-
   return (vo_frame_t *) frame;
 }
 
@@ -390,6 +394,22 @@ static int raw_get_property (vo_driver_t *this_gen, int property)
 
 static int raw_set_property (vo_driver_t *this_gen, int property, int value)
 {
+  raw_driver_t *this = (raw_driver_t *) this_gen;
+
+  if (property == VO_PROP_DISCARD_FRAMES) {
+    if (value == -1) {
+      int i, n = 0;
+      for (i = NUM_FRAMES_BACKLOG - 1; i >= 0; i--) {
+        if (this->frame[i]) {
+          this->frame[i]->vo_frame.free (&this->frame[i]->vo_frame);
+          this->frame[i] = NULL;
+          n++;
+        }
+      }
+      value = n;
+    }
+  }
+
   return value;
 }
 
@@ -438,9 +458,9 @@ static void raw_dispose (vo_driver_t *this_gen)
 
 
 static vo_driver_t *raw_open_plugin (video_driver_class_t *class_gen, const void *visual_gen)
-{
+{ 
+  const raw_visual_t *visual  = (const raw_visual_t *) visual_gen;
   raw_class_t       *class   = (raw_class_t *) class_gen;
-  raw_visual_t         *visual  = (raw_visual_t *) visual_gen;
   raw_driver_t      *this;
   int i;
 
@@ -471,6 +491,11 @@ static vo_driver_t *raw_open_plugin (video_driver_class_t *class_gen, const void
   this->vo_driver.redraw_needed        = raw_redraw_needed;
 
   this->yuv2rgb_factory = yuv2rgb_factory_init (MODE_24_BGR, 1, NULL); /* converts to rgb */
+  if (!this->yuv2rgb_factory) {
+    xprintf (this->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": yuv2rgb initialization failed\n");
+    free(this);
+    return NULL;
+  }
 
   for (i = 0; i < NUM_FRAMES_BACKLOG; i++)
     this->frame[i] = 0;
