@@ -117,6 +117,7 @@ typedef struct {
 
   /* color matrix switching */
   int                cm_active, cm_state;
+  uint8_t            cm_lut[32];
 
   vo_scale_t         sc;
 
@@ -141,6 +142,7 @@ typedef struct {
 
 
 /* import common color matrix stuff */
+#define CM_LUT
 #define CM_DRIVER_T xshm_driver_t
 #include "color_matrix.c"
 
@@ -488,6 +490,7 @@ static void xshm_frame_dispose (vo_frame_t *vo_img) {
   xine_free_aligned (frame->vo_frame.base[0]);
   xine_free_aligned (frame->vo_frame.base[1]);
   xine_free_aligned (frame->vo_frame.base[2]);
+  pthread_mutex_destroy (&frame->vo_frame.mutex);
   free (frame);
 }
 
@@ -500,6 +503,16 @@ static vo_frame_t *xshm_alloc_frame (vo_driver_t *this_gen) {
   if (!frame)
     return NULL;
 
+  /*
+   * colorspace converter for this frame
+   */
+
+  frame->yuv2rgb = this->yuv2rgb_factory->create_converter (this->yuv2rgb_factory);
+  if (!frame->yuv2rgb) {
+    free(frame);
+    return NULL;
+  }
+
   memcpy (&frame->sc, &this->sc, sizeof(vo_scale_t));
 
   pthread_mutex_init (&frame->vo_frame.mutex, NULL);
@@ -514,12 +527,6 @@ static vo_frame_t *xshm_alloc_frame (vo_driver_t *this_gen) {
   frame->vo_frame.dispose    = xshm_frame_dispose;
   frame->vo_frame.driver     = this_gen;
 
-  /*
-   * colorspace converter for this frame
-   */
-
-  frame->yuv2rgb = this->yuv2rgb_factory->create_converter (this->yuv2rgb_factory);
-
   return (vo_frame_t *) frame;
 }
 
@@ -880,6 +887,17 @@ static int xshm_set_property (vo_driver_t *this_gen,
   xshm_driver_t *this = (xshm_driver_t *) this_gen;
 
   switch (property) {
+  case VO_PROP_DISCARD_FRAMES:
+    if (value == -1) {
+      value = 0;
+      if (this->cur_frame) {
+        this->cur_frame->vo_frame.free (&this->cur_frame->vo_frame);
+        this->cur_frame = NULL;
+        value = 1;
+      }
+    }
+    break;
+
   case VO_PROP_ASPECT_RATIO:
     if (value>=XINE_VO_ASPECT_NUM_RATIOS)
       value = XINE_VO_ASPECT_AUTO;
@@ -1036,7 +1054,8 @@ static void xshm_dispose (vo_driver_t *this_gen) {
   if (this->cur_frame)
     this->cur_frame->vo_frame.dispose (&this->cur_frame->vo_frame);
 
-  this->yuv2rgb_factory->dispose (this->yuv2rgb_factory);
+  if (this->yuv2rgb_factory)
+    this->yuv2rgb_factory->dispose (this->yuv2rgb_factory);
 
   cm_close (this);
 
@@ -1123,7 +1142,7 @@ static const char *visual_class_name(xcb_visualtype_t *visual) {
 static vo_driver_t *xshm_open_plugin(video_driver_class_t *class_gen, const void *visual_gen) {
   xshm_class_t         *class   = (xshm_class_t *) class_gen;
   config_values_t      *config  = class->config;
-  xcb_visual_t         *visual  = (xcb_visual_t *) visual_gen;
+  const xcb_visual_t   *visual  = (const xcb_visual_t *) visual_gen;
   xshm_driver_t        *this;
   xcb_visualtype_t     *visualtype;
   int                   mode;
@@ -1214,6 +1233,12 @@ static vo_driver_t *xshm_open_plugin(video_driver_class_t *class_gen, const void
 
   free(window_attrs_reply);
 
+  if (!visualtype) {
+    xprintf (this->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": visual type was not recognized\n");
+    xshm_dispose(&this->vo_driver);
+    return NULL;
+  }
+
   geometry_reply = xcb_get_geometry_reply(this->connection, geometry_cookie, NULL);
 
   this->depth = geometry_reply->depth;
@@ -1335,6 +1360,7 @@ static vo_driver_t *xshm_open_plugin(video_driver_class_t *class_gen, const void
   if (!mode) {
     xprintf (this->xine, XINE_VERBOSITY_LOG,
 	     _("%s: your video mode was not recognized, sorry :-(\n"), LOG_MODULE);
+    xshm_dispose(&this->vo_driver);
     return NULL;
   }
 
@@ -1345,6 +1371,11 @@ static vo_driver_t *xshm_open_plugin(video_driver_class_t *class_gen, const void
   this->saturation = 128;
 
   this->yuv2rgb_factory = yuv2rgb_factory_init (mode, swapped, this->yuv2rgb_cmap);
+  if (!this->yuv2rgb_factory) {
+    xprintf (this->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": yuv2rgb initialization failed\n");
+    xshm_dispose(&this->vo_driver);
+    return NULL;
+  }
 
   this->xoverlay = xcbosd_create(this->xine, this->connection, this->screen,
                                  this->window, XCBOSD_SHAPED);
