@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2016 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -118,6 +118,7 @@ typedef struct {
 
   /* color matrix switching */
   int                cm_active, cm_state;
+  uint8_t            cm_lut[32];
 
   vo_scale_t         sc;
 
@@ -148,6 +149,7 @@ typedef struct {
 
 
 /* import common color matrix stuff */
+#define CM_LUT
 #define CM_DRIVER_T xshm_driver_t
 #include "color_matrix.c"
 
@@ -582,6 +584,7 @@ static void xshm_frame_dispose (vo_frame_t *vo_img) {
   xine_free_aligned (frame->vo_frame.base[0]);
   xine_free_aligned (frame->vo_frame.base[1]);
   xine_free_aligned (frame->vo_frame.base[2]);
+  pthread_mutex_destroy (&frame->vo_frame.mutex);
   free (frame);
 }
 
@@ -594,6 +597,16 @@ static vo_frame_t *xshm_alloc_frame (vo_driver_t *this_gen) {
   if (!frame)
     return NULL;
 
+  /*
+   * colorspace converter for this frame
+   */
+
+  frame->yuv2rgb = this->yuv2rgb_factory->create_converter (this->yuv2rgb_factory);
+  if (!frame->yuv2rgb) {
+    free(frame);
+    return NULL;
+  }
+
   memcpy (&frame->sc, &this->sc, sizeof(vo_scale_t));
 
   pthread_mutex_init (&frame->vo_frame.mutex, NULL);
@@ -608,12 +621,6 @@ static vo_frame_t *xshm_alloc_frame (vo_driver_t *this_gen) {
   frame->vo_frame.dispose    = xshm_frame_dispose;
   frame->vo_frame.driver     = this_gen;
 
-  /*
-   * colorspace converter for this frame
-   */
-
-  frame->yuv2rgb = this->yuv2rgb_factory->create_converter (this->yuv2rgb_factory);
-
   return (vo_frame_t *) frame;
 }
 
@@ -966,6 +973,17 @@ static int xshm_set_property (vo_driver_t *this_gen,
   xshm_driver_t *this = (xshm_driver_t *) this_gen;
 
   switch (property) {
+  case VO_PROP_DISCARD_FRAMES:
+    if (value == -1) {
+      value = 0;
+      if (this->cur_frame) {
+        this->cur_frame->vo_frame.free (&this->cur_frame->vo_frame);
+        this->cur_frame = NULL;
+        value = 1;
+      }
+    }
+    break;
+
   case VO_PROP_ASPECT_RATIO:
     if (value>=XINE_VO_ASPECT_NUM_RATIOS)
       value = XINE_VO_ASPECT_AUTO;
@@ -1116,7 +1134,8 @@ static void xshm_dispose (vo_driver_t *this_gen) {
   if (this->cur_frame)
     this->cur_frame->vo_frame.dispose (&this->cur_frame->vo_frame);
 
-  this->yuv2rgb_factory->dispose (this->yuv2rgb_factory);
+  if (this->yuv2rgb_factory)
+    this->yuv2rgb_factory->dispose (this->yuv2rgb_factory);
 
   cm_close (this);
 
@@ -1194,7 +1213,7 @@ static const char *visual_class_name(Visual *visual) {
 static vo_driver_t *xshm_open_plugin_2 (video_driver_class_t *class_gen, const void *visual_gen) {
   xshm_class_t         *class   = (xshm_class_t *) class_gen;
   config_values_t      *config  = class->config;
-  x11_visual_t         *visual  = (x11_visual_t *) visual_gen;
+  const x11_visual_t   *visual  = (const x11_visual_t *) visual_gen;
   xshm_driver_t        *this;
   XWindowAttributes     attribs;
   XImage               *myimage;
@@ -1383,6 +1402,11 @@ static vo_driver_t *xshm_open_plugin_2 (video_driver_class_t *class_gen, const v
   this->saturation = 128;
 
   this->yuv2rgb_factory = yuv2rgb_factory_init (mode, swapped, this->yuv2rgb_cmap);
+  if (!this->yuv2rgb_factory) {
+    xprintf (this->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": yuv2rgb initialization failed\n");
+    xshm_dispose(&this->vo_driver);
+    return NULL;
+  }
 
   LOCK_DISPLAY(this);
   this->xoverlay = x11osd_create (this->xine, this->display, this->screen,
@@ -1393,7 +1417,7 @@ static vo_driver_t *xshm_open_plugin_2 (video_driver_class_t *class_gen, const v
 }
 
 static vo_driver_t *xshm_open_plugin_old (video_driver_class_t *class_gen, const void *visual_gen) {
-  x11_visual_t         *old_visual  = (x11_visual_t *) visual_gen;
+  const x11_visual_t  *old_visual  = (const x11_visual_t *) visual_gen;
   x11_visual_t         visual;
 
   /* provides compatibility for XINE_VISUAL_TYPE_X11 */
