@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004, 2008 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -39,7 +39,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
-#include <assert.h>
+#include <unistd.h>
+#include <pthread.h>
 
 #if defined(__FreeBSD__)
 #include <machine/param.h>
@@ -486,10 +487,10 @@ static void xvmc_render_macro_blocks(vo_frame_t *current_image,
   int           flags;
 
   lprintf ("xvmc_render_macro_blocks\n");
-  lprintf ("slices %d 0x%08lx 0x%08lx 0x%08lx\n",
+  lprintf ("slices %d %p %p %p\n",
 	   macroblocks->slices,
-	   (long) current_frame, (long) backward_frame,
-	   (long) forward_frame);
+           (void *) current_frame, (void *) backward_frame,
+           (void *) forward_frame);
   /* lprintf ("slices %d 0x%08lx 0x%08lx 0x%08lx\n",macroblocks->slices,
 	   (long) current_frame->surface, (long) backward_frame->surface,
 	   (long) forward_frame->surface);
@@ -592,8 +593,8 @@ static cxid_t *xvmc_set_context (xvmc_driver_t *this,
     macroblocks->blocks       = calloc(1, sizeof(XvMCBlockArray));
     macroblocks->macro_blocks = calloc(1, sizeof(XvMCMacroBlockArray));
 
-    lprintf("macroblocks->blocks %lx ->macro_blocks %lx\n",
-	    macroblocks->blocks,macroblocks->macro_blocks);
+    lprintf("macroblocks->blocks %p ->macro_blocks %p\n",
+            (void *)macroblocks->blocks, (void *)macroblocks->macro_blocks);
   }
 
   if((this->context_id.xid != NULL)   &&
@@ -873,21 +874,18 @@ static void xvmc_add_recent_frame (xvmc_driver_t *this, xvmc_frame_t *frame) {
   this->recent_frames[0] = frame;
 }
 
-/* currently not used - we could have a method to call this from video loop */
-#if 0
-static void xvmc_flush_recent_frames (xvmc_driver_t *this) {
-  int i;
+static int xv_flush_recent_frames (xvmc_driver_t *this) {
+  int i, n = 0;
 
-  lprintf ("xvmc_flush_recent_frames\n");
-
-  for( i=0; i < VO_NUM_RECENT_FRAMES; i++ ) {
-    if( this->recent_frames[i] )
-      this->recent_frames[i]->vo_frame.free
-	(&this->recent_frames[i]->vo_frame);
-    this->recent_frames[i] = NULL;
+  for (i = 0; i < VO_NUM_RECENT_FRAMES; i++) {
+    if (this->recent_frames[i]) {
+      this->recent_frames[i]->vo_frame.free (&this->recent_frames[i]->vo_frame);
+      this->recent_frames[i] = NULL;
+      n++;
+    }
   }
+  return n;
 }
-#endif
 
 static int xvmc_redraw_needed (vo_driver_t *this_gen) {
   xvmc_driver_t  *this = (xvmc_driver_t *) this_gen;
@@ -922,7 +920,7 @@ static void xvmc_display_frame (vo_driver_t *this_gen, vo_frame_t *frame_gen) {
   xvmc_driver_t  *this  = (xvmc_driver_t *) this_gen;
   xvmc_frame_t   *frame = (xvmc_frame_t *) frame_gen;
 
-  lprintf ("xvmc_display_frame %d %x\n",frame_gen->id,frame_gen);
+  lprintf ("xvmc_display_frame %d %p\n", frame_gen->id, (void *)frame_gen);
 
   /*
    * queue frames (deinterlacing)
@@ -1057,6 +1055,11 @@ static int xvmc_set_property (vo_driver_t *this_gen,
   }
   else {
     switch (property) {
+    case VO_PROP_DISCARD_FRAMES:
+      if (value == -1)
+        value = xv_flush_recent_frames (this);
+      break;
+
     case VO_PROP_ASPECT_RATIO:
       if (value>=XINE_VO_ASPECT_NUM_RATIOS)
 	value = XINE_VO_ASPECT_AUTO;
@@ -1317,7 +1320,7 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *vi
   XvAttribute          *attr;
   XvImageFormatValues  *fo;
   int                   nattr;
-  x11_visual_t         *visual  = (x11_visual_t *) visual_gen;
+  const x11_visual_t   *visual  = (const x11_visual_t *) visual_gen;
   XColor                dummy;
   XvAdaptorInfo        *adaptor_info = class->adaptor_info;
   unsigned int          adaptor_num  = class->adaptor_num;
@@ -1539,7 +1542,7 @@ static void dispose_class (video_driver_class_t *this_gen) {
 }
 
 static void *init_class (xine_t *xine, void *visual_gen) {
-  x11_visual_t      *visual = (x11_visual_t *) visual_gen;
+  const x11_visual_t *visual = (const x11_visual_t *) visual_gen;
   xvmc_class_t      *this;
   Display           *display = NULL;
   unsigned int       adaptors, j = 0;
