@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -30,8 +30,8 @@
 #include <string.h>
 #include <sched.h>
 #include <unistd.h>
-
-#define XINE_ENGINE_INTERNAL
+#include <pthread.h>
+#include <sys/time.h>
 
 #define LOG_MODULE "audio_decoder"
 #define LOG_VERBOSE
