@@ -49,14 +49,24 @@
 #include <signal.h>
 #include <errno.h>
 #include <sys/time.h>
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
-
-#ifndef WIN32
-#include <sys/ioctl.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif
+#ifdef HAVE_SYS_IOCTL_H
+#include <sys/ioctl.h>
+#endif
+#ifdef WIN32
+#include <ws2tcpip.h>  // socklen_t
+#endif
 
 #include <dlfcn.h>
 #include <pthread.h>
@@ -328,11 +338,21 @@ broadcaster_t *_x_init_broadcaster(xine_stream_t *stream, int port)
     return NULL;
   }
 
-  listen(msock,QLEN);
+  if (listen(msock,QLEN) < 0) {
+    xprintf(stream->xine, XINE_VERBOSITY_DEBUG, "broadcaster: error listening port %d\n", port);
+    close(msock);
+    return NULL;
+  }
 
+#ifndef _WIN32
   signal( SIGPIPE, SIG_IGN );
-
+#endif
   this = calloc(1, sizeof(broadcaster_t));
+  if (!this) {
+    close(msock);
+    return NULL;
+  }
+
   this->port = port;
   this->stream = stream;
   this->msock = msock;
@@ -340,7 +360,8 @@ broadcaster_t *_x_init_broadcaster(xine_stream_t *stream, int port)
 
   pthread_mutex_init (&this->lock, NULL);
 
-  stream->video_fifo->register_put_cb(stream->video_fifo, video_put_cb, this);
+  if (stream->video_fifo)
+    stream->video_fifo->register_put_cb(stream->video_fifo, video_put_cb, this);
 
   if(stream->audio_fifo)
     stream->audio_fifo->register_put_cb(stream->audio_fifo, audio_put_cb, this);
@@ -350,7 +371,10 @@ broadcaster_t *_x_init_broadcaster(xine_stream_t *stream, int port)
                              NULL, manager_loop, (void *)this)) != 0) {
     xprintf (stream->xine, XINE_VERBOSITY_NONE,
 	     "broadcaster: can't create new thread (%s)\n", strerror(err));
-    _x_abort();
+
+    this->running = 0;
+    _x_close_broadcaster(this);
+    return NULL;
   }
 
   return this;
@@ -358,9 +382,11 @@ broadcaster_t *_x_init_broadcaster(xine_stream_t *stream, int port)
 
 void _x_close_broadcaster(broadcaster_t *this_gen)
 {
-  this_gen->running = 0;
-  pthread_cancel(this_gen->manager_thread);
-  pthread_join(this_gen->manager_thread,NULL);
+  if (this_gen->running) {
+    this_gen->running = 0;
+    pthread_cancel(this_gen->manager_thread);
+    pthread_join(this_gen->manager_thread,NULL);
+  }
   close(this_gen->msock);
 
   if (this_gen->stream->video_fifo)
