@@ -32,6 +32,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <pthread.h>
+
 #include <xine/configfile.h>
 #include "bswap.h"
 
@@ -267,8 +269,7 @@ static void config_insert(config_values_t *this, cfg_entry_t *new_entry) {
   cur_base = NULL;
   for (cur = this->first, prev = NULL; cur; prev = cur, cur = cur->next) {
     /* extract parts of the cur key */
-    if (cur_base)
-      free(cur_base);
+    _x_freep(&cur_base);
     config_key_split(cur->key, &cur_base, &cur_section, &cur_subsect, &cur_name);
 
     /* sort by section name */
@@ -305,10 +306,8 @@ static void config_insert(config_values_t *this, cfg_entry_t *new_entry) {
 
     break;
   }
-  if (new_base)
-    free(new_base);
-  if (cur_base)
-    free(cur_base);
+  free(new_base);
+  free(cur_base);
 
   new_entry->next = cur;
   if (!cur)
@@ -430,37 +429,25 @@ static cfg_entry_t *config_lookup_entry(config_values_t *this, const char *key)
 
 static void config_reset_value(cfg_entry_t *entry) {
 
-  if (entry->str_value) {
-    free (entry->str_value);
-    entry->str_value = NULL;
-  }
-  if (entry->str_default) {
-    free (entry->str_default);
-    entry->str_default = NULL;
-  }
-  if (entry->description) {
-    free (entry->description);
-    entry->description = NULL;
-  }
-  if (entry->help) {
-    free (entry->help);
-    entry->help = NULL;
-  }
+  _x_freep (&entry->str_value);
+  _x_freep (&entry->str_default);
+  _x_freep (&entry->description);
+  _x_freep (&entry->help);
+
   if (entry->enum_values) {
     char **value;
 
     value = entry->enum_values;
     while (*value) {
-      free (*value);
+      _x_freep (value);
       value++;
     }
-    free (entry->enum_values);
-    entry->enum_values = NULL;
+    _x_freep (&entry->enum_values);
   }
   entry->num_value = 0;
 }
 
-static void config_shallow_copy(xine_cfg_entry_t *dest, cfg_entry_t *src);
+static void config_shallow_copy(xine_cfg_entry_t *dest, const cfg_entry_t *src);
 
 static cfg_entry_t *config_register_key (config_values_t *this,
 					 const char *key,
@@ -691,6 +678,21 @@ static int config_register_range (config_values_t *this,
   else
     entry->num_value = def_value;
 
+  /* validate value (config files can be edited ...) */
+  /* Allow default to be out of range. xine-ui uses this for brightness etc. */
+  if (entry->num_value != def_value) {
+    if (entry->num_value > max) {
+      printf ("configfile: WARNING: value %d for %s is larger than max (%d)\n",
+        entry->num_value, key, max);
+      entry->num_value = max;
+    }
+    if (entry->num_value < min) {
+      printf ("configfile: WARNING: value %d for %s is smaller than min (%d)\n",
+        entry->num_value, key, min);
+      entry->num_value = min;
+    }
+  }
+
   /* fill out rest of struct */
 
   entry->num_default   = def_value;
@@ -790,12 +792,20 @@ static int config_register_enum (config_values_t *this,
   entry->description   = (description) ? strdup(description) : NULL;
   entry->help          = (help) ? strdup(help) : NULL;
 
+  entry->range_min = 0;
+  entry->range_max = value_count;
+
+  if (entry->num_value < 0)
+    entry->num_value = 0;
+  if (entry->num_value >= value_count)
+    entry->num_value = value_count;
+
   pthread_mutex_unlock(&this->config_lock);
 
   return entry->num_value;
 }
 
-static void config_shallow_copy(xine_cfg_entry_t *dest, cfg_entry_t *src)
+static void config_shallow_copy(xine_cfg_entry_t *dest, const cfg_entry_t *src)
 {
   dest->key           = src->key;
   dest->type          = src->type;
@@ -839,6 +849,18 @@ static void config_update_num (config_values_t *this,
   }
 
   pthread_mutex_lock(&this->config_lock);
+
+  if (entry->type == XINE_CONFIG_TYPE_ENUM) {
+    if (value >= entry->range_max) {
+      printf("configfile: error - tried to update enum value to %d (max %d)\n", value, entry->range_max);
+      value = entry->range_max - 1;
+    }
+    if (value < 0) {
+      printf("configfile: error - tried to update enum value to %d\n", value);
+      value = 0;
+    }
+  }
+
   entry->num_value = value;
 
   if (entry->callback) {
@@ -904,8 +926,7 @@ static void config_update_string (config_values_t *this,
     entry->callback (entry->callback_data, &cb_entry);
   }
 
-  if (str_free)
-    free(str_free);
+  free(str_free);
   pthread_mutex_unlock(&this->config_lock);
 }
 
@@ -1129,7 +1150,9 @@ void xine_config_save (xine_t *xine, const char *filename) {
 	fprintf (f_config, "}, default: %d\n",
 		 entry->num_default);
 
-	if (entry->enum_values[entry->num_value] != NULL) {
+        if (entry->num_value >= 0 &&
+            entry->num_value < entry->range_max &&
+            entry->enum_values[entry->num_value] != NULL) {
 	  if (entry->num_value == entry->num_default) fprintf (f_config, "#");
 	  fprintf (f_config, "%s:", entry->key);
 	  fprintf (f_config, "%s\n", entry->enum_values[entry->num_value]);
@@ -1187,10 +1210,8 @@ static void config_dispose (config_values_t *this) {
     last = entry;
     entry = entry->next;
 
-    if (last->key)
-      free (last->key);
-    if (last->unknown_value)
-      free (last->unknown_value);
+    _x_freep (&last->key);
+    _x_freep (&last->unknown_value);
 
     config_reset_value(last);
 
@@ -1219,6 +1240,27 @@ static void config_unregister_cb (config_values_t *this, const char *key) {
   }
 }
 
+void _x_config_unregister_cb_class_d (config_values_t *this, void *callback_data) {
+
+  cfg_entry_t *entry;
+
+  _x_assert(this);
+  _x_assert(callback_data);
+
+  pthread_mutex_lock(&this->config_lock);
+
+  entry = this->first;
+  while (entry) {
+    if (entry->callback && entry->callback_data == callback_data) {
+      entry->callback = NULL;
+      entry->callback_data = NULL;
+    }
+    entry = entry->next;
+  }
+
+  pthread_mutex_unlock(&this->config_lock);
+}
+
 static void config_set_new_entry_callback (config_values_t *this, xine_config_cb_t new_entry_cb, void* cbdata) {
   pthread_mutex_lock(&this->config_lock);
   this->new_entry_cb = new_entry_cb;
@@ -1555,8 +1597,8 @@ config_values_t *_x_config_init (void) {
 
   if (!(this = calloc(1, sizeof(config_values_t)))) {
 
-    printf ("configfile: could not allocate config object\n");
-    _x_abort();
+    fprintf (stderr, "configfile: could not allocate config object\n");
+    return NULL;
   }
 
   this->first = NULL;
