@@ -34,8 +34,6 @@
 #include <unistd.h>
 #include <errno.h>
 
-#define XINE_ENGINE_INTERNAL
-
 #define LOG_MODULE "demux"
 #define LOG_VERBOSE
 /*
@@ -45,6 +43,7 @@
 #include <xine/xine_internal.h>
 #include <xine/demux.h>
 #include <xine/buffer.h>
+#include "xine_private.h"
 
 #ifdef WIN32
 #include <winsock.h>
@@ -63,7 +62,7 @@ void _x_demux_flush_engine (xine_stream_t *stream) {
 
   buf_element_t *buf;
 
-  if( stream->gapless_switch )
+  if (stream->gapless_switch || stream->finished_naturally)
     return;
 
   stream->xine->port_ticket->acquire(stream->xine->port_ticket, 1);
@@ -263,7 +262,7 @@ void _x_demux_control_headers_done (xine_stream_t *stream) {
 void _x_demux_control_start( xine_stream_t *stream ) {
 
   buf_element_t *buf;
-  uint32_t flags = (stream->gapless_switch) ? BUF_FLAG_GAPLESS_SW : 0;
+  uint32_t flags = (stream->gapless_switch || stream->finished_naturally) ? BUF_FLAG_GAPLESS_SW : 0;
 
   pthread_mutex_lock(&stream->demux_mutex);
 
@@ -659,7 +658,7 @@ void _x_demux_send_data(fifo_buffer_t *fifo, uint8_t *data, int size,
   _x_assert(size > 0);
   while (fifo && size > 0) {
 
-    buf = fifo->buffer_pool_alloc (fifo);
+    buf = fifo->buffer_pool_size_alloc (fifo, size);
 
     if ( size > buf->max_size ) {
       buf->size          = buf->max_size;
@@ -705,7 +704,7 @@ int _x_demux_read_send_data(fifo_buffer_t *fifo, input_plugin_t *input,
   _x_assert(size > 0);
   while (fifo && size > 0) {
 
-    buf = fifo->buffer_pool_alloc (fifo);
+    buf = fifo->buffer_pool_size_alloc (fifo, size);
 
     if ( size > buf->max_size ) {
       buf->size          = buf->max_size;
@@ -748,7 +747,9 @@ void _x_demux_send_mrl_reference (xine_stream_t *stream, int alternative,
   xine_event_t event;
   union {
     xine_mrl_reference_data_ext_t *e;
+    XINE_DISABLE_DEPRECATION_WARNINGS
     xine_mrl_reference_data_t *b;
+    XINE_ENABLE_DEPRECATION_WARNINGS
   } data;
   const size_t mrl_len = strlen (mrl);
 
@@ -773,7 +774,9 @@ void _x_demux_send_mrl_reference (xine_stream_t *stream, int alternative,
 
   /* plain MRL reference event */
 
+  XINE_DISABLE_DEPRECATION_WARNINGS
   event.data_length = offsetof (xine_mrl_reference_data_t, mrl) + mrl_len + 1;
+  XINE_ENABLE_DEPRECATION_WARNINGS
 
   /*data.b->alternative = alternative;*/
   strcpy (data.b->mrl, mrl);
