@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2011 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -24,7 +24,8 @@
 #include "config.h"
 #endif
 
-#define XINE_ENGINE_INTERNAL
+#include <pthread.h>
+#include <sys/time.h>
 
 #include <xine/xine_internal.h>
 
@@ -66,7 +67,7 @@ xine_event_t *xine_event_wait (xine_event_queue_t *queue) {
 }
 
 void xine_event_free (xine_event_t *event) {
-  if (event->data && event->data_length > 0) free (event->data);
+  _x_freep (&event->data);
   free (event);
 }
 
@@ -185,7 +186,7 @@ void xine_event_dispose_queue (xine_event_queue_t *queue) {
   if (queue->listener_thread) {
     void *p;
     pthread_join (*queue->listener_thread, &p);
-    free (queue->listener_thread);
+    _x_freep (&queue->listener_thread);
   }
 
   _x_refcounter_dec(stream->refcounter);
