@@ -29,6 +29,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <pthread.h>
 #ifdef HAVE_DIRENT_H
 #include <dirent.h>
 #endif
@@ -51,7 +52,7 @@
 */
 
 #define XINE_ENABLE_EXPERIMENTAL_FEATURES 1
-#define XINE_ENGINE_INTERNAL
+
 #include <xine/xine_internal.h>
 #include <xine/xine_plugin.h>
 #include <xine/plugin_catalog.h>
@@ -141,6 +142,7 @@ static void inc_file_ref(plugin_file_t *file) {
 
 static void dec_file_ref(plugin_file_t *file) {
   _x_assert(file);
+  _x_assert(file->ref > 0);
 
   file->ref--;
   lprintf("file %s referenced %d time(s)\n", file->filename, file->ref);
@@ -153,11 +155,27 @@ static void inc_node_ref(plugin_node_t *node) {
 
 static void dec_node_ref(plugin_node_t *node) {
   _x_assert(node);
+  _x_assert(node->ref > 0);
 
   node->ref--;
   lprintf("node %s referenced %d time(s)\n", node->info->id, node->ref);
 }
 
+static void _free_string_list(xine_list_t **plist) {
+  xine_list_t *list = *plist;
+
+  if (list) {
+    xine_list_iterator_t ite = xine_list_front (list);
+    while (ite) {
+      char *key = xine_list_get_value (list, ite);
+      free (key);
+      ite = xine_list_next (list, ite);
+    }
+    xine_list_delete(list);
+    *plist = NULL;
+  }
+}
+
 /*
  * plugin list/catalog management functions
  */
@@ -174,7 +192,7 @@ static int get_decoder_priority(xine_t *this, plugin_node_t *node) {
     /* user given priorities should definitely override defaults, so multiply them */
     return entry->num_value * 100;
   else
-    return ((decoder_info_t *)node->info->special_info)->priority;
+    return ((const decoder_info_t *)node->info->special_info)->priority;
 }
 
 static void map_decoder_list (xine_t *this,
@@ -195,7 +213,7 @@ static void map_decoder_list (xine_t *this,
   for (list_id = 0; list_id < list_size; list_id++) {
 
     plugin_node_t *node = xine_sarray_get(decoder_list, list_id);
-    const uint32_t *type = ((decoder_info_t *)node->info->special_info)->supported_types;
+    const uint32_t *type = ((const decoder_info_t *)node->info->special_info)->supported_types;
     int priority = get_decoder_priority(this, node);
 
     lprintf ("mapping decoder %s\n", node->info->id);
@@ -325,15 +343,42 @@ static void _insert_node (xine_t *this,
   uint32_t             *types;
   char                  key[80];
   int                   i;
+  int                   plugin_type = info->type & PLUGIN_TYPE_MASK;
 
   _x_assert(list);
   _x_assert(info);
+
+  if (!info->id) {
+    xprintf (this, XINE_VERBOSITY_LOG,
+             "load_plugins: plugin from %s is broken: id = NULL\n",
+             file ? file->filename : "xine-lib");
+    return;
+  }
   if (info->API != api_version) {
     xprintf(this, XINE_VERBOSITY_LOG,
 	    _("load_plugins: ignoring plugin %s, wrong iface version %d (should be %d)\n"),
 	    info->id, info->API, api_version);
     return;
   }
+  if (!node_cache && !info->init) {
+    xprintf (this, XINE_VERBOSITY_LOG,
+             "load_plugins: plugin %s from %s is broken: init = NULL\n",
+             info->id, file ? file->filename : "xine-lib");
+    return;
+  }
+  if (!info->special_info) {
+    if (plugin_type != PLUGIN_INPUT && plugin_type != PLUGIN_DEMUX) {
+      if (file)
+        xprintf (this, XINE_VERBOSITY_LOG,
+                 "load_plugins: plugin %s from %s is broken: special_info = NULL\n",
+                 info->id, file->filename);
+      else
+        xprintf (this, XINE_VERBOSITY_LOG,
+                 "load_plugins: statically linked plugin %s is broken: special_info = NULL\n",
+                 info->id);
+      return;
+    }
+  }
 
   entry = calloc(1, sizeof(plugin_node_t));
   entry->info         = calloc(1, sizeof(plugin_info_t));
@@ -344,9 +389,12 @@ static void _insert_node (xine_t *this,
   entry->file         = file;
   entry->ref          = 0;
   entry->priority     = 0; /* default priority */
-  entry->config_entry_list = (node_cache) ? node_cache->config_entry_list : NULL;
+  if (node_cache) {
+    entry->config_entry_list = node_cache->config_entry_list;
+    node_cache->config_entry_list = NULL;
+  }
 
-  switch (info->type & PLUGIN_TYPE_MASK){
+  switch (plugin_type) {
 
   case PLUGIN_VIDEO_OUT:
     vo_old = info->special_info;
@@ -368,17 +416,6 @@ static void _insert_node (xine_t *this,
   case PLUGIN_SPU_DECODER:
     decoder_old = info->special_info;
     decoder_new = calloc(1, sizeof(decoder_info_t));
-    if (decoder_old == NULL) {
-      if (file)
-	xprintf (this, XINE_VERBOSITY_DEBUG,
-		 "load_plugins: plugin %s from %s is broken: special_info = NULL\n",
-		 info->id, entry->file->filename);
-      else
-	xprintf (this, XINE_VERBOSITY_DEBUG,
-		 "load_plugins: statically linked plugin %s is broken: special_info = NULL\n",
-		 info->id);
-      _x_abort();
-    }
     {
       size_t supported_types_size;
       for (supported_types_size=0; decoder_old->supported_types[supported_types_size] != 0; ++supported_types_size);
@@ -718,8 +755,8 @@ static inline int _plugin_info_equal(const plugin_info_t *a,
     case PLUGIN_VIDEO_OUT:
       /* FIXME: Could special_info be NULL? */
       if (a->special_info && b->special_info) {
-        return (((vo_info_t*)a->special_info)->visual_type ==
-                ((vo_info_t*)b->special_info)->visual_type);
+        return (((const vo_info_t*)a->special_info)->visual_type ==
+                ((const vo_info_t*)b->special_info)->visual_type);
       }
       break;
 
@@ -783,7 +820,8 @@ static int _load_plugin_class(xine_t *this,
 	  /* the callback is called for each entry registered by this plugin */
           lprintf("plugin init %s\n", node->info->id);
 	  config->set_new_entry_callback(config, _new_entry_cb, node);
-	  node->plugin_class = info->init(this, data);
+          if (info->init)
+            node->plugin_class = info->init(this, data);
 	  config->unset_new_entry_callback(config);
 
 	  if (node->plugin_class) {
@@ -985,7 +1023,7 @@ static void save_plugin_list(xine_t *this, FILE *fp, xine_sarray_t *list) {
         /* now serialize the config key */
         char *key_value = this->config->get_serialized_entry(this->config, key);
 
-        lprintf("  config key: %s, serialization: %d bytes\n", key, strlen(key_value));
+        lprintf("  config key: %s, serialization: %zu bytes\n", key, strlen(key_value));
         fprintf(fp, "config_key=%s\n", key_value);
 
         free (key_value);
@@ -1328,22 +1366,31 @@ static void push_if_dir (xine_list_t *plugindirs, void *path)
 /*
  *  initialize catalog, load all plugins into new catalog
  */
-void _x_scan_plugins (xine_t *this) {
+int _x_scan_plugins (xine_t *this) {
 
-  char *homedir, *pluginpath;
-  xine_list_t *plugindirs = xine_list_new ();
+  char *homedir = NULL, *pluginpath = NULL;
+  xine_list_t *plugindirs = NULL;
   xine_list_iterator_t iter;
+  int result = -1;
 
   lprintf("_x_scan_plugins()\n");
 
-  if (this == NULL || this->config == NULL) {
-    fprintf(stderr, "%s(%s@%d): parameter should be non null, exiting\n",
-	    __FILE__, __XINE_FUNCTION__, __LINE__);
-    _x_abort();
-  }
+  _x_assert(this);
+  _x_assert(this->config);
+  _x_assert(!this->plugin_catalog);
 
   homedir = strdup(xine_get_homedir());
+  plugindirs = xine_list_new ();
+
+  if (!homedir || !plugindirs) {
+    goto fail;
+  }
+
   this->plugin_catalog = _new_catalog();
+  if (!this->plugin_catalog) {
+    goto fail;
+  }
+
   XINE_PROFILE(load_cached_catalog (this));
 
 #ifdef XINE_MAKE_BUILTINS
@@ -1389,8 +1436,6 @@ void _x_scan_plugins (xine_t *this) {
     collect_plugins(this, dir);
     free (dir);
   }
-  xine_list_delete (plugindirs);
-  free(homedir);
 
   load_required_plugins (this);
 
@@ -1398,6 +1443,15 @@ void _x_scan_plugins (xine_t *this) {
     XINE_PROFILE(save_catalog (this));
 
   map_decoders (this);
+
+  result = 0;
+
+ fail:
+  if (plugindirs)
+    xine_list_delete (plugindirs);
+  free(homedir);
+
+  return result;
 }
 
 /*
@@ -1808,12 +1862,12 @@ static vo_driver_t *_load_video_driver (xine_t *this, plugin_node_t *node,
 }
 
 vo_driver_t *_x_load_video_output_plugin(xine_t *this,
-					   char *id,
+                                         const char *id,
 					   int visual_type, void *visual) {
 
   plugin_node_t      *node;
   vo_driver_t        *driver;
-  vo_info_t          *vo_info;
+  const vo_info_t    *vo_info;
   plugin_catalog_t   *catalog = this->plugin_catalog;
   int                 list_id, list_size;
 
@@ -1829,7 +1883,7 @@ vo_driver_t *_x_load_video_output_plugin(xine_t *this,
 
     node = xine_sarray_get (catalog->plugin_lists[PLUGIN_VIDEO_OUT - 1], list_id);
 
-    vo_info = (vo_info_t *)node->info->special_info;
+    vo_info = (const vo_info_t *)node->info->special_info;
     if (vo_info->visual_type == visual_type) {
       if (id) {
 	if (!strcasecmp (node->info->id, id)) {
@@ -1861,7 +1915,7 @@ xine_video_port_t *xine_open_video_driver (xine_t *this,
   vo_driver_t        *driver;
   xine_video_port_t  *port;
 
-  driver = _x_load_video_output_plugin(this, (char *)id, visual_type, visual);
+  driver = _x_load_video_output_plugin(this, id, visual_type, visual);
 
   if (!driver) {
     lprintf ("failed to load video output plugin <%s>\n", id);
@@ -1947,7 +2001,7 @@ const char *const *xine_list_video_output_plugins_typed(xine_t *xine, uint64_t t
   for (list_id = i = 0; list_id < list_size; list_id++)
   {
     node = xine_sarray_get (catalog->plugin_lists[PLUGIN_VIDEO_OUT - 1], list_id);
-    if (typemask & (1ULL << ((vo_info_t *)node->info->special_info)->visual_type))
+    if (typemask & (1ULL << ((const vo_info_t *)node->info->special_info)->visual_type))
     {
       const char *id = node->info->id;
       int j = i;
@@ -2017,7 +2071,7 @@ xine_audio_port_t *xine_open_audio_driver (xine_t *this, const char *id,
   plugin_node_t      *node;
   ao_driver_t        *driver;
   xine_audio_port_t  *port;
-  ao_info_t          *ao_info;
+  const ao_info_t    *ao_info;
   plugin_catalog_t   *catalog = this->plugin_catalog;
   int                 list_id, list_size;
 
@@ -2033,7 +2087,7 @@ xine_audio_port_t *xine_open_audio_driver (xine_t *this, const char *id,
 
     node = xine_sarray_get (this->plugin_catalog->plugin_lists[PLUGIN_AUDIO_OUT - 1], list_id);
 
-    ao_info = (ao_info_t *)node->info->special_info;
+    ao_info = (const ao_info_t *)node->info->special_info;
 
     if (id) {
       if (!strcasecmp(node->info->id, id)) {
@@ -2074,6 +2128,40 @@ xine_audio_port_t *xine_new_framegrab_audio_port (xine_t *this) {
   return port;
 }
 
+void _x_free_audio_driver (xine_t *xine, ao_driver_t **pdriver) {
+
+  ao_driver_t      *driver = *pdriver;
+  plugin_catalog_t *catalog = xine->plugin_catalog;
+  plugin_node_t    *node = driver->node;
+
+  *pdriver = NULL;
+
+  driver->exit(driver);
+
+  if (node) {
+    pthread_mutex_lock(&catalog->lock);
+    dec_node_ref(node);
+    pthread_mutex_unlock(&catalog->lock);
+  }
+}
+
+void _x_free_video_driver (xine_t *xine, vo_driver_t **pdriver) {
+
+  vo_driver_t      *driver = *pdriver;
+  plugin_catalog_t *catalog = xine->plugin_catalog;
+  plugin_node_t    *node = driver->node;
+
+  *pdriver = NULL;
+
+  driver->dispose (driver);
+
+  if (node) {
+    pthread_mutex_lock(&catalog->lock);
+    dec_node_ref(node);
+    pthread_mutex_unlock(&catalog->lock);
+  }
+}
+
 void xine_close_audio_driver (xine_t *this, xine_audio_port_t  *ao_port) {
 
   if( ao_port )
@@ -2166,6 +2254,7 @@ video_decoder_t *_x_get_video_decoder (xine_stream_t *stream, uint8_t stream_typ
   video_decoder_t  *vd = NULL;
 
   lprintf ("looking for video decoder for streamtype %02x\n", stream_type);
+  _x_assert(stream_type < DECODER_MAX);
 
   pthread_mutex_lock (&catalog->lock);
 
@@ -2243,6 +2332,7 @@ audio_decoder_t *_x_get_audio_decoder (xine_stream_t *stream, uint8_t stream_typ
   audio_decoder_t  *ad = NULL;
 
   lprintf ("looking for audio decoder for streamtype %02x\n", stream_type);
+  _x_assert(stream_type < DECODER_MAX);
 
   pthread_mutex_lock (&catalog->lock);
 
@@ -2315,6 +2405,8 @@ int _x_decoder_available (xine_t *xine, uint32_t buftype)
   plugin_catalog_t *catalog = xine->plugin_catalog;
   int stream_type = (buftype>>16) & 0xFF;
 
+  _x_assert(stream_type < DECODER_MAX);
+
   if ( (buftype & 0xFF000000) == BUF_VIDEO_BASE ) {
     if( catalog->video_decoder_map[stream_type][0] )
       return 1;
@@ -2420,6 +2512,7 @@ spu_decoder_t *_x_get_spu_decoder (xine_stream_t *stream, uint8_t stream_type) {
   spu_decoder_t    *sd = NULL;
 
   lprintf ("looking for spu decoder for streamtype %02x\n", stream_type);
+  _x_assert(stream_type < DECODER_MAX);
 
   pthread_mutex_lock (&catalog->lock);
 
@@ -2554,7 +2647,7 @@ const char *const *xine_list_post_plugins_typed(xine_t *xine, uint32_t type) {
 
   for (list_id = 0; list_id < list_size; list_id++) {
     node = xine_sarray_get (catalog->plugin_lists[PLUGIN_POST - 1], list_id);
-    if (((post_info_t *)node->info->special_info)->type == type)
+    if (((const post_info_t *)node->info->special_info)->type == type)
       catalog->ids[i++] = node->info->id;
   }
   catalog->ids[i] = NULL;
@@ -2668,7 +2761,7 @@ xine_post_t *xine_post_init(xine_t *xine, const char *name, int inputs,
 	post->output_ids[i] = NULL;
 
 	/* copy the post plugin type to the public part */
-	post->xine_post.type = ((post_info_t *)node->info->special_info)->type;
+	post->xine_post.type = ((const post_info_t *)node->info->special_info)->type;
 
 	break;
       } else {
@@ -2826,6 +2919,18 @@ char *xine_get_demux_for_mime_type (xine_t *self, const char *mime_type) {
   return id;
 }
 
+static void _dispose_file_entry(plugin_file_t **pfile)
+{
+  if (*pfile) {
+    plugin_file_t *file = *pfile;
+
+    _x_assert(file->lib_handle == NULL);
+    _x_assert(file->ref == 0);
+
+    _x_freep (&file->filename);
+    _x_freep(pfile);
+  }
+}
 
 static void dispose_plugin_list (xine_sarray_t *list, int is_cache) {
 
@@ -2854,29 +2959,26 @@ static void dispose_plugin_list (xine_sarray_t *list, int is_cache) {
       case PLUGIN_VIDEO_DECODER:
 	decoder_info = (decoder_info_t *)node->info->special_info;
 
-	free ((void *)decoder_info->supported_types);
+        _x_freep (&decoder_info->supported_types);
 
+        /* fall thru */
       default:
-	free ((void *)node->info->special_info);
+        _x_freep (&node->info->special_info);
 	break;
       }
 
       /* free info structure and string copies */
-      free ((void *)node->info->id);
-      free (node->info);
+      _x_freep (&node->info->id);
+      _x_freep (&node->info);
 
-      /* don't free the entry list if the node is cache */
-      if (!is_cache) {
-        if (node->config_entry_list) {
-          xine_list_iterator_t ite = xine_list_front (node->config_entry_list);
-          while (ite) {
-            char *key = xine_list_get_value (node->config_entry_list, ite);
-            free (key);
-            ite = xine_list_next (node->config_entry_list, ite);
-          }
-          xine_list_delete(node->config_entry_list);
-        }
+      _free_string_list(&node->config_entry_list);
+
+      /* file entries in cache list are "dummies" (do not refer to opened files) */
+      /* those are not in file list, so free here */
+      if (is_cache) {
+        _dispose_file_entry(&node->file);
       }
+
       free (node);
     }
     xine_sarray_delete(list);
@@ -2891,7 +2993,7 @@ static void dispose_plugin_file_list (xine_list_t *list) {
   ite = xine_list_front (list);
   while (ite) {
     file = xine_list_get_value (list, ite);
-    free (file->filename);
+    _x_freep (&file->filename);
     free (file);
     ite = xine_list_next (list, ite);
   }
@@ -2908,6 +3010,10 @@ void _x_dispose_plugins (xine_t *this) {
   if(this->plugin_catalog) {
     int i;
 
+    if (this->config) {
+      _x_config_unregister_cb_class_d (this->config, this);
+    }
+
     for (i = 0; i < PLUGIN_TYPE_MAX; i++) {
       dispose_plugin_list (this->plugin_catalog->plugin_lists[i], 0);
     }
@@ -2916,10 +3022,10 @@ void _x_dispose_plugins (xine_t *this) {
     dispose_plugin_file_list (this->plugin_catalog->file_list);
 
     for (i = 0; this->plugin_catalog->prio_desc[i]; i++)
-      free(this->plugin_catalog->prio_desc[i]);
+      _x_freep(&this->plugin_catalog->prio_desc[i]);
 
     pthread_mutex_destroy(&this->plugin_catalog->lock);
 
-    free (this->plugin_catalog);
+    _x_freep (&this->plugin_catalog);
   }
 }
