@@ -27,6 +27,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include <pthread.h>
 #include <zlib.h>
 #include <sys/types.h>
 #ifdef HAVE_DIRENT_H
@@ -46,8 +47,6 @@
 #define LOG
 */
 
-#define XINE_ENGINE_INTERNAL
-
 #include <xine/xine_internal.h>
 #include "xine-engine/bswap.h"
 #include <xine/xineutils.h>
@@ -223,8 +222,7 @@ static void osd_free_ft2 (osd_object_t *osd)
       FT_Done_Face (osd->ft2->face);
     if ( osd->ft2->library )
       FT_Done_FreeType(osd->ft2->library);
-    free( osd->ft2 );
-    osd->ft2 = NULL;
+    _x_freep( &osd->ft2 );
   }
 }
 #else
@@ -889,7 +887,7 @@ static int osd_renderer_load_font(osd_renderer_t *this, char *filename) {
         font->fontchar[i].code = gzread_i16(fp);
         font->fontchar[i].width = gzread_i16(fp);
         font->fontchar[i].height = gzread_i16(fp);
-        font->fontchar[i].bmp = malloc(font->fontchar[i].width*font->fontchar[i].height);
+        font->fontchar[i].bmp = malloc((size_t)font->fontchar[i].width * (size_t)font->fontchar[i].height);
         if( gzread(fp, font->fontchar[i].bmp,
               font->fontchar[i].width*font->fontchar[i].height) <= 0 )
           break;
@@ -987,13 +985,13 @@ static int osd_renderer_unload_font(osd_renderer_t *this, char *fontname ) {
   while( font ) {
     if ( !strcasecmp(font->name,fontname) ) {
 
-      free( font->filename );
+      _x_freep( &font->filename );
 
       if( font->loaded ) {
         for( i = 0; i < font->num_fontchars; i++ ) {
-          free( font->fontchar[i].bmp );
+          _x_freep( &font->fontchar[i].bmp );
         }
-        free( font->fontchar );
+        _x_freep( &font->fontchar );
       }
 
       if( last )
@@ -1108,8 +1106,7 @@ static int osd_set_font_freetype2( osd_object_t *osd, const char *fontname, int
     if(FT_Init_FreeType( &osd->ft2->library )) {
       xprintf(osd->renderer->stream->xine, XINE_VERBOSITY_LOG,
 	      _("osd: cannot initialize ft2 library\n"));
-      free(osd->ft2);
-      osd->ft2 = NULL;
+      _x_freep(&osd->ft2);
       return 0;
     }
   }
@@ -1277,10 +1274,7 @@ static void osd_free_encoding(osd_object_t *osd) {
     iconv_close(osd->cd);
     osd->cd = (iconv_t)-1;
   }
-  if (osd->encoding) {
-    free(osd->encoding);
-    osd->encoding = NULL;
-  }
+  _x_freep(&osd->encoding);
 #endif
 }
 
@@ -1715,7 +1709,7 @@ static void osd_free_object (osd_object_t *osd_to_close) {
   osd = this->osds;
   while( osd ) {
     if ( osd == osd_to_close ) {
-      free( osd->area );
+      _x_freep( &osd->area );
 
       osd_free_ft2 (osd);
       osd_free_encoding(osd);
@@ -1744,7 +1738,7 @@ static void osd_renderer_close (osd_renderer_t *this) {
 
   pthread_mutex_destroy (&this->osd_mutex);
 
-  free(this->event.object.overlay);
+  _x_freep(&this->event.object.overlay);
   free(this);
 }
 
@@ -1873,14 +1867,15 @@ osd_renderer_t *_x_osd_renderer_init( xine_stream_t *stream ) {
     if ( data_dirs )
       while( (*data_dirs) && *(*data_dirs) ) {
 	/* sizeof("") takes care of the final NUL byte */
-	char *fontpath = xine_xmalloc( strlen(*data_dirs) + sizeof("/"PACKAGE"/fonts/") );
-	strcpy(fontpath, *data_dirs);
-	strcat(fontpath, "/"PACKAGE"/fonts/");
+        char *fontpath = malloc( strlen(*data_dirs) + sizeof("/"PACKAGE"/fonts/") );
+        if (fontpath) {
+          strcpy(fontpath, *data_dirs);
+          strcat(fontpath, "/"PACKAGE"/fonts/");
 
-	osd_preload_fonts(this, fontpath);
-
-	free(fontpath);
+          osd_preload_fonts(this, fontpath);
 
+          free(fontpath);
+        }
 	data_dirs++;
       }
   }
