@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2009 the xine project
+ * Copyright (C) 2007-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -19,8 +19,10 @@
  *
  */
 
+#include <xine/configfile.h>
 #include <xine/xine_internal.h>
 #include <xine/spu.h>
+#include <xine.h>
 
 #define BLACK_OPACITY   67
 #define COLOUR_OPACITY 100
