@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2005 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -26,9 +26,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <pthread.h>
 #include <errno.h>
-
-#define XINE_ENGINE_INTERNAL
+#include <sys/time.h>
 
 #define LOG_MODULE "video_decoder"
 #define LOG_VERBOSE
