@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -23,17 +23,16 @@
 #endif
 
 #include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
+#include <unistd.h>
+#include <pthread.h>
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
 
 #include <xine/buffer.h>
 #include <xine/xine_internal.h>
-#include "bswap.h"
 #include <xine/xineutils.h>
 #include <xine/video_overlay.h>
+#include "bswap.h"
 
 /*
 #define LOG_DEBUG
@@ -214,6 +213,7 @@ static void video_overlay_reset (video_overlay_t *this) {
   }
   pthread_mutex_unlock (&this->events_mutex);
 
+  pthread_mutex_lock( &this->showing_mutex );
   for( i = 0; i < MAX_SHOWING; i++ )
     this->showing[i].handle = -1;
 
@@ -222,6 +222,7 @@ static void video_overlay_reset (video_overlay_t *this) {
   }
 
   this->showing_changed = 0;
+  pthread_mutex_unlock( &this->showing_mutex );
 }
 
 
