@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2010 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -26,8 +26,6 @@
 #endif
 
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
 #include <string.h>
 #include <stdlib.h>
 #include <pthread.h>
@@ -38,8 +36,6 @@
 #include <machine/endian.h>
 #endif
 
-#define XINE_ENGINE_INTERNAL
-
 #include <xine/xine_internal.h>
 #include <xine/audio_out.h>
 #include <xine/video_out.h>
@@ -264,8 +260,10 @@ int xine_config_lookup_entry (xine_t *this, const char *key,
   int result;
   config_values_t *config = this->config;
 
-  config->cur = config->lookup_entry (config, key);
   pthread_mutex_lock(&config->config_lock);
+  /* safe because of the mutex is recursive */
+  config->cur = config->lookup_entry (config, key);
+
   /* do not hand out unclaimed entries */
   if (config->cur && config->cur->type == XINE_CONFIG_TYPE_UNKNOWN)
     config->cur = NULL;
@@ -486,6 +484,24 @@ void xine_set_param (xine_stream_t *stream, int param, int value) {
     stream->xine->port_ticket->release(stream->xine->port_ticket, 0);
     break;
 
+  case XINE_PARAM_VO_SINGLE_STEP:
+    pthread_mutex_lock (&stream->frontend_lock);
+    if (_x_get_fine_speed (stream) != XINE_SPEED_PAUSE) {
+      _x_set_fine_speed (stream, XINE_SPEED_PAUSE);
+    } else {
+      /* HACK: temporarily resume decoders. */
+      _x_set_fine_speed (stream, XINE_LIVE_PAUSE_ON);
+      /* rather miss 1 strobe than wait or freeze. */
+      if (stream->xine->port_ticket->ticket_revoked == 0) {
+        stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
+        stream->video_out->set_property (stream->video_out, param, value);
+        stream->audio_out->set_property (stream->audio_out, param, value);
+        stream->xine->port_ticket->release(stream->xine->port_ticket, 0);
+      }
+    }
+    pthread_mutex_unlock (&stream->frontend_lock);
+    break;
+
   case XINE_PARAM_IGNORE_VIDEO:
     _x_stream_info_set(stream, XINE_STREAM_INFO_IGNORE_VIDEO, value);
     break;
