@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2011 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -44,6 +44,14 @@
 # define INTERNAL
 #endif
 
+#if defined (__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6 ))
+#    define XINE_DISABLE_DEPRECATION_WARNINGS _Pragma("GCC diagnostic ignored \"-Wdeprecated-declarations\"")
+#    define XINE_ENABLE_DEPRECATION_WARNINGS  _Pragma("GCC diagnostic warning \"-Wdeprecated-declarations\"")
+#else
+#    define XINE_DISABLE_DEPRECATION_WARNINGS
+#    define XINE_ENABLE_DEPRECATION_WARNINGS
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -62,7 +70,7 @@ extern "C" {
  * Decoder plugins are loaded on demand. Video/audio output plugins
  * have special load/probe functions
  */
-void _x_scan_plugins (xine_t *this) INTERNAL;
+int _x_scan_plugins (xine_t *this) INTERNAL;
 
 /**
  * @ingroup load_plugins
@@ -71,6 +79,9 @@ void _x_scan_plugins (xine_t *this) INTERNAL;
  */
 void _x_dispose_plugins (xine_t *this) INTERNAL;
 
+void _x_free_video_driver (xine_t *xine, vo_driver_t **driver) INTERNAL;
+void _x_free_audio_driver (xine_t *xine, ao_driver_t **driver) INTERNAL;
+
 ///@{
 /**
  * @defgroup
@@ -111,8 +122,36 @@ int _x_set_file_close_on_exec(int fd) INTERNAL;
 
 int _x_set_socket_close_on_exec(int s) INTERNAL;
 
+
+#if defined(HAVE_PTHREAD_RWLOCK)
+#  define xine_rwlock_t                pthread_rwlock_t
+#  define xine_rwlock_init_default(l)  pthread_rwlock_init (l, NULL)
+#  define xine_rwlock_rdlock(l)        pthread_rwlock_rdlock (l)
+#  define xine_rwlock_tryrdlock(l)     pthread_rwlock_tryrdlock (l)
+#  define xine_rwlock_timedrdlock(l,t) pthread_rwlock_timedrdlock (l, t)
+#  define xine_rwlock_wrlock(l)        pthread_rwlock_wrlock (l)
+#  define xine_rwlock_trywrlock(l)     pthread_rwlock_trywrlock (l)
+#  define xine_rwlock_timedwrlock(l,t) pthread_rwlock_timedwrlock (l, t)
+#  define xine_rwlock_unlock(l)        pthread_rwlock_unlock (l)
+#  define xine_rwlock_destroy(l)       pthread_rwlock_destroy (l)
+#else
+#  define xine_rwlock_t                pthread_mutex_t
+#  define xine_rwlock_init_default(l)  pthread_mutex_init (l, NULL)
+#  define xine_rwlock_rdlock(l)        pthread_mutex_lock (l)
+#  define xine_rwlock_tryrdlock(l)     pthread_mutex_trylock (l)
+#  define xine_rwlock_timedrdlock(l,t) pthread_mutex_timedlock (l, t)
+#  define xine_rwlock_wrlock(l)        pthread_mutex_lock (l)
+#  define xine_rwlock_trywrlock(l)     pthread_mutex_trylock (l)
+#  define xine_rwlock_timedwrlock(l,t) pthread_mutex_timedlock (l, t)
+#  define xine_rwlock_unlock(l)        pthread_mutex_unlock (l)
+#  define xine_rwlock_destroy(l)       pthread_mutex_destroy (l)
+#endif
+
+
 #ifdef __cplusplus
 }
 #endif
 
 #endif
+
+
