@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2013 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -70,63 +70,22 @@
 #include "xine_mmx.h"
 
 /*
- * In search of the perfect colorspace conversion formulae...
- * These are the conversion equations that xine currently uses
- * (before normalisation):
- *
- *      Y  =  0.29900 * R + 0.58700 * G + 0.11400 * B
- *      U  = -0.16874 * R - 0.33126 * G + 0.50000 * B + 128
- *      V  =  0.50000 * R - 0.41869 * G - 0.08131 * B + 128
- */
-
-/*
-#define Y_R (SCALEFACTOR *  0.29900 * 219.0 / 255.0)
-#define Y_G (SCALEFACTOR *  0.58700 * 219.0 / 255.0)
-#define Y_B (SCALEFACTOR *  0.11400 * 219.0 / 255.0)
-
-#define U_R (SCALEFACTOR * -0.16874 * 224.0 / 255.0)
-#define U_G (SCALEFACTOR * -0.33126 * 224.0 / 255.0)
-#define U_B (SCALEFACTOR *  0.50000 * 224.0 / 255.0)
-
-#define V_R (SCALEFACTOR *  0.50000 * 224.0 / 255.0)
-#define V_G (SCALEFACTOR * -0.41869 * 224.0 / 255.0)
-#define V_B (SCALEFACTOR * -0.08131 * 224.0 / 255.0)
-*/
-
-#define Y_R (SCALEFACTOR *  0.299         * 219.0 / 255.0)
-#define Y_G (SCALEFACTOR *  0.587         * 219.0 / 255.0)
-#define Y_B (SCALEFACTOR *  0.114         * 219.0 / 255.0)
-
-#define U_R (SCALEFACTOR * -0.299 / 1.772 * 224.0 / 255.0)
-#define U_G (SCALEFACTOR * -0.587 / 1.772 * 224.0 / 255.0)
-#define U_B (SCALEFACTOR *  0.886 / 1.772 * 224.0 / 255.0)
-
-#define V_R (SCALEFACTOR *  0.701 / 1.402 * 224.0 / 255.0)
-#define V_G (SCALEFACTOR * -0.587 / 1.402 * 224.0 / 255.0)
-#define V_B (SCALEFACTOR * -0.114 / 1.402 * 224.0 / 255.0)
-
-/*
- * With the normalisation factors above, Y needs 16 added.
- * This is done during setup, not in the macros in xineutils.h, because
- * doing it there would be an API change.
- */
-#define Y_MOD (16 * SCALEFACTOR)
-
-/*
- * Precalculate all of the YUV tables since it requires fewer than
- * 10 kilobytes to store them.
+ * Precalculate all of the YUV tables since it requires only 8 kilobytes to store them
+ * (plus 2 for backwards compatibility).
  */
 int y_r_table[256];
 int y_g_table[256];
 int y_b_table[256];
 
+int uv_br_table[256];
+
 int u_r_table[256];
 int u_g_table[256];
-int u_b_table[256];
-
-int v_r_table[256];
-int v_g_table[256];
 int v_b_table[256];
+int v_g_table[256];
+
+int u_b_table[256];
+int v_r_table[256];
 
 void (*yuv444_to_yuy2) (const yuv_planes_t *yuv_planes, unsigned char *yuy2_map, int pitch);
 void (*yuv9_to_yv12)
@@ -159,14 +118,16 @@ void (*yuy2_to_yv12)
  * and height passed to it. The width must be divisible by 2.
  */
 void init_yuv_planes(yuv_planes_t *yuv_planes, int width, int height) {
-  memset (yuv_planes, 0, sizeof (yuv_planes));
+  size_t plane_size = width * height;
+
+  memset (yuv_planes, 0, sizeof (*yuv_planes));
 
   yuv_planes->row_width = width;
   yuv_planes->row_count = height;
 
-  yuv_planes->y = calloc(width, height);
-  yuv_planes->u = calloc(width, height);
-  yuv_planes->v = calloc(width, height);
+  yuv_planes->y = xine_mallocz_aligned(plane_size);
+  yuv_planes->u = xine_mallocz_aligned(plane_size);
+  yuv_planes->v = xine_mallocz_aligned(plane_size);
 }
 
 /*
@@ -175,12 +136,9 @@ void init_yuv_planes(yuv_planes_t *yuv_planes, int width, int height) {
  * This frees the memory used by the YUV planes.
  */
 void free_yuv_planes(yuv_planes_t *yuv_planes) {
-  if (yuv_planes->y)
-    free(yuv_planes->y);
-  if (yuv_planes->u)
-    free(yuv_planes->u);
-  if (yuv_planes->v)
-    free(yuv_planes->v);
+  xine_freep_aligned(&yuv_planes->y);
+  xine_freep_aligned(&yuv_planes->u);
+  xine_freep_aligned(&yuv_planes->v);
 }
 
 /*
@@ -314,9 +272,9 @@ static void yuv444_to_yuy2_c(const yuv_planes_t *yuv_planes, unsigned char *yuy2
  * enough.
  *
  */
+#if defined(ARCH_X86)
 static void yuv444_to_yuy2_mmx(const yuv_planes_t *yuv_planes, unsigned char *yuy2_map,
   int pitch) {
-#if defined(ARCH_X86) || defined(ARCH_X86_64)
   unsigned int i;
   int h, j, k;
   int width_div_8 = yuv_planes->row_width / 8;
@@ -461,8 +419,8 @@ static void yuv444_to_yuy2_mmx(const yuv_planes_t *yuv_planes, unsigned char *yu
 
   /* be a good MMX citizen and empty MMX state */
   emms();
-#endif
 }
+#endif
 
 static void hscale_chroma_line (unsigned char *dst, const unsigned char *src,
   int width) {
@@ -496,8 +454,8 @@ static void vscale_chroma_line (unsigned char *dst, int pitch,
 
   /* process blocks of 4 pixels */
   for (x=0; x < (width / 4); x++) {
-    n1  = *((unsigned int *) src1); src1 = (const unsigned char *)(((const unsigned int *) src1) + 1);
-    n2  = *((unsigned int *) src2); src2 = (const unsigned char *)(((const unsigned int *) src2) + 1);
+    n1  = *((const unsigned int *) src1); src1 = (const unsigned char *)(((const unsigned int *) src1) + 1);
+    n2  = *((const unsigned int *) src2); src2 = (const unsigned char *)(((const unsigned int *) src2) + 1);
     n3  = (n1 & 0xFF00FF00) >> 8;
     n4  = (n2 & 0xFF00FF00) >> 8;
     n1 &= 0x00FF00FF;
@@ -806,7 +764,7 @@ static void yv12_to_yuy2_c
 }
 
 
-#if defined(ARCH_X86) || defined(ARCH_X86_64)
+#if defined(ARCH_X86)
 static const int64_t __attribute__((__used__)) byte_one = 0x0101010101010101ll;
 
 #define MMX_YUV420_YUYV_PROGRESSIVE(p_y1,p_y2,p_u,p_ut,p_ub,p_v,p_vt,p_vb,p_line1,p_line2)  \
@@ -1127,13 +1085,14 @@ do {
 
 #endif
 
+#if defined(ARCH_X86)
 static void yv12_to_yuy2_mmxext
   (const unsigned char *y_src, int y_src_pitch,
    const unsigned char *u_src, int u_src_pitch,
    const unsigned char *v_src, int v_src_pitch,
    unsigned char *yuy2_map, int yuy2_pitch,
    int width, int height, int progressive ) {
-#if defined(ARCH_X86) || defined(ARCH_X86_64)
+
     uint8_t *p_line1, *p_line2 = yuy2_map;
     const uint8_t *p_y1, *p_y2 = y_src;
     const uint8_t *p_u = u_src;
@@ -1250,17 +1209,16 @@ static void yv12_to_yuy2_mmxext
 
     sfence();
     emms();
-
-#endif
 }
+#endif
 
+#if defined(ARCH_X86)
 static void yv12_to_yuy2_sse2
   (const unsigned char *y_src, int y_src_pitch,
    const unsigned char *u_src, int u_src_pitch,
    const unsigned char *v_src, int v_src_pitch,
    unsigned char *yuy2_map, int yuy2_pitch,
    int width, int height, int progressive ) {
-#if defined(ARCH_X86) || defined(ARCH_X86_64)
 
     /* check alignment */
     if (((uintptr_t)y_src | (uintptr_t)yuy2_map | yuy2_pitch | y_src_pitch) & 15) {
@@ -1385,17 +1343,18 @@ static void yv12_to_yuy2_sse2
     }
 
     sfence();
-#endif
 }
+#endif
 
 /* identical to yv12_to_yuy2_c with the obvious exception... */
+#if defined(ARCH_X86)
 static void yv12_to_yuy2_mmx
   (const unsigned char *y_src, int y_src_pitch,
    const unsigned char *u_src, int u_src_pitch,
    const unsigned char *v_src, int v_src_pitch,
    unsigned char *yuy2_map, int yuy2_pitch,
    int width, int height, int progressive ) {
-#if defined(ARCH_X86) || defined(ARCH_X86_64)
+
     uint8_t *p_line1, *p_line2 = yuy2_map;
     const uint8_t *p_y1, *p_y2 = y_src;
     const uint8_t *p_u = u_src;
@@ -1527,9 +1486,8 @@ static void yv12_to_yuy2_mmx
 
     sfence();
     emms();
-
-#endif
 }
+#endif
 
 #define C_YUYV_YUV420( )                                          \
     *p_y1++ = *p_line1++; *p_y2++ = *p_line2++;                   \
@@ -1581,7 +1539,7 @@ static void yuy2_to_yv12_c
 }
 
 
-#if defined(ARCH_X86) || defined(ARCH_X86_64)
+#if defined(ARCH_X86)
 
 /* yuy2->yv12 with subsampling (some ideas from mplayer's yuy2toyv12) */
 #define MMXEXT_YUYV_YUV420( )                                                      \
@@ -1630,13 +1588,14 @@ do {
 
 #endif
 
+#if defined(ARCH_X86)
 static void yuy2_to_yv12_mmxext
   (const unsigned char *yuy2_map, int yuy2_pitch,
    unsigned char *y_dst, int y_dst_pitch,
    unsigned char *u_dst, int u_dst_pitch,
    unsigned char *v_dst, int v_dst_pitch,
    int width, int height) {
-#if defined(ARCH_X86) || defined(ARCH_X86_64)
+
     const uint8_t *p_line1, *p_line2 = yuy2_map;
     uint8_t *p_y1, *p_y2 = y_dst;
     uint8_t *p_u = u_dst;
@@ -1675,8 +1634,8 @@ static void yuy2_to_yv12_mmxext
 
     sfence();
     emms();
-#endif
 }
+#endif
 
 
 /*
@@ -1689,69 +1648,83 @@ static void yuy2_to_yv12_mmxext
 void init_yuv_conversion(void) {
 
   int i;
+#if defined(ARCH_X86)
+  uint32_t accel;
+#endif
+
+  /* ITU-R 601 */
+  #define KB 0.114
+  #define KR 0.299
+
+  #define YR (219.0/255.0)*KR*SCALEFACTOR
+  #define YG (219.0/255.0)*(1.0-KB-KR)*SCALEFACTOR
+  #define YB (219.0/255.0)*KB*SCALEFACTOR
+
+  #define UR (112.0/255.0)*(KR/(KB-1.0))*SCALEFACTOR
+  #define UG (112.0/255.0)*((1.0-KB-KR)/(KB-1.0))*SCALEFACTOR
+  #define UB (112.0/255.0)*SCALEFACTOR
+
+  #define VR (112.0/255.0)*SCALEFACTOR
+  #define VG (112.0/255.0)*((1.0-KB-KR)/(KR-1.0))*SCALEFACTOR
+  #define VB (112.0/255.0)*(KB/(KR-1.0))*SCALEFACTOR
 
   /* initialize the RGB -> YUV tables */
   for (i = 0; i < 256; i++) {
 
-    y_r_table[i] = Y_R * i + Y_MOD;
-    y_g_table[i] = Y_G * i;
-    y_b_table[i] = Y_B * i;
+    /* fast and correctly rounded */
+    y_r_table[i] = YR * i;
+    y_g_table[i] = YG * i + 16 * SCALEFACTOR + SCALEFACTOR / 2;
+    y_b_table[i] = YB * i;
 
-    u_r_table[i] = U_R * i;
-    u_g_table[i] = U_G * i;
-    u_b_table[i] = U_B * i;
+    uv_br_table[i] = UB * i + CENTERSAMPLE * SCALEFACTOR + SCALEFACTOR / 2;
+    u_r_table[i] = UR * i;
+    u_g_table[i] = UG * i;
+    v_g_table[i] = VG * i;
+    v_b_table[i] = VB * i;
 
-    v_r_table[i] = V_R * i;
-    v_g_table[i] = V_G * i;
-    v_b_table[i] = V_B * i;
+    /* keep supporting old binaries */
+    u_b_table[i] = UB * i;
+    v_r_table[i] = VR * i;
   }
 
-  /* determine best YUV444 -> YUY2 converter to use */
-  if (xine_mm_accel() & MM_ACCEL_X86_MMX)
-    yuv444_to_yuy2 = yuv444_to_yuy2_mmx;
-  else
-    yuv444_to_yuy2 = yuv444_to_yuy2_c;
-
-  /* determine best YV12 -> YUY2 converter to use */
-  if (xine_mm_accel() & MM_ACCEL_X86_SSE2)
-    yv12_to_yuy2 = yv12_to_yuy2_sse2;
-  else if (xine_mm_accel() & MM_ACCEL_X86_MMXEXT)
-    yv12_to_yuy2 = yv12_to_yuy2_mmxext;
-  else if (xine_mm_accel() & MM_ACCEL_X86_MMX)
-    yv12_to_yuy2 = yv12_to_yuy2_mmx;
-  else
-    yv12_to_yuy2 = yv12_to_yuy2_c;
-
-  /* determine best YUY2 -> YV12 converter to use */
-  if (xine_mm_accel() & MM_ACCEL_X86_MMXEXT)
-    yuy2_to_yv12 = yuy2_to_yv12_mmxext;
-  else
-    yuy2_to_yv12 = yuy2_to_yv12_c;
-
-
-  /* determine best YUV9 -> YV12 converter to use (only the portable C
-   * version is available so far) */
+  /* determine best converters to use */
+  yuv444_to_yuy2 = yuv444_to_yuy2_c;
+  yv12_to_yuy2 = yv12_to_yuy2_c;
+  yuy2_to_yv12 = yuy2_to_yv12_c;
   yuv9_to_yv12 = yuv9_to_yv12_c;
-
-  /* determine best YUV411 -> YV12 converter to use (only the portable C
-   * version is available so far) */
   yuv411_to_yv12 = yuv411_to_yv12_c;
 
+#if defined(ARCH_X86)
+  accel = xine_mm_accel();
+  if (accel & MM_ACCEL_X86_MMX) {
+    yuv444_to_yuy2 = yuv444_to_yuy2_mmx;
+    yv12_to_yuy2   = yv12_to_yuy2_mmx;
+  }
+  if (accel & MM_ACCEL_X86_MMXEXT) {
+    yv12_to_yuy2   = yv12_to_yuy2_mmxext;
+    yuy2_to_yv12   = yuy2_to_yv12_mmxext;
+  }
+  if (accel & MM_ACCEL_X86_SSE2) {
+    yv12_to_yuy2   = yv12_to_yuy2_sse2;
+  }
+#endif
 }
 
 /* TJ. direct sliced rgb -> yuy2 conversion */
 typedef struct {
   /* unused:1 u:21 v:21 y:21 */
   uint64_t t0[256], t1[256], t2[256];
-  /* u:12 v:12 y:8 */
-  uint32_t p[256];
+  /* u:12 v:12 y:8 (rgb_pal8) */
+  /* native endian yuyv (other rgb_pal*) */
+  uint32_t p[272];
   int cm, fmt, pfmt;
 } rgb2yuy2_t;
 
 typedef enum {
   rgb_bgr = 0, rgb_rgb, rgb_bgra, rgb_argb, rgb_rgba,
   rgb_rgb555le, rgb_rgb555be, rgb_rgb565le, rgb_rgb565be,
-  rgb_pal8
+  rgb_bgra_clut, rgb_rgba_clut,
+  rgb_pal8, rgb_pal4, rgb_pal2, rgb_pal1
 } rgb_fmt_t;
 
 void *rgb2yuy2_alloc (int color_matrix, const char *format) {
@@ -1760,9 +1733,13 @@ void *rgb2yuy2_alloc (int color_matrix, const char *format) {
   float _ry, _gy, _by, _yoffs;
   float _bv, _bvoffset, _ru, _ruoffset, _gu, _guoffset, _gv, _gvoffset, _burv;
   int i = -1;
-  const char *fmts[] = {"bgr", "rgb", "bgra", "argb", "rgba", "rgb555le", "rgb555be", "rgb565le", "rgb565be"};
+  const char *fmts[] = {
+    "bgr", "rgb", "bgra", "argb", "rgba",
+    "rgb555le", "rgb555be", "rgb565le", "rgb565be",
+    "bgra_clut", "rgba_clut"
+  };
 
-  if (format) for (i = 8; i >= 0; i--) if (!strcmp (format, fmts[i])) break;
+  if (format) for (i = 10; i >= 0; i--) if (!strcmp (format, fmts[i])) break;
   if (i < 0) return NULL;
 
   b = malloc (sizeof (*b));
@@ -1802,9 +1779,11 @@ void *rgb2yuy2_alloc (int color_matrix, const char *format) {
     case rgb_rgb:
     case rgb_bgra:
     case rgb_argb:
-    case rgb_rgba: { /* 24/32 bit */
+    case rgb_rgba:
+    case rgb_bgra_clut:
+    case rgb_rgba_clut: { /* 24/32 bit */
       uint64_t *rr, *gg, *bb;
-      if ((b->fmt == rgb_bgr) || (b->fmt == rgb_bgra)) {
+      if ((b->fmt == rgb_bgr) || (b->fmt == rgb_bgra) || (b->fmt == rgb_bgra_clut)) {
         bb = b->t0;
         gg = b->t1;
         rr = b->t2;
@@ -1930,20 +1909,29 @@ void rgb2yuy2_free (void *rgb2yuy2) {
 void rgb2yuy2_palette (void *rgb2yuy2, const uint8_t *pal, int num_colors, int bits_per_pixel) {
   rgb2yuy2_t *b = rgb2yuy2;
   uint64_t v;
-  uint32_t w;
-  int mode, i = 0;
+  uint32_t w, *p;
+  int mode, i = 0, max_colors;
 
-  if (!b || (num_colors < 2) || (num_colors > 256)) return;
+  if (!b || (num_colors < 2)) return;
   switch (bits_per_pixel) {
-    case 8: mode = rgb_pal8; break;
-/*  case 4: mode = 10; break;  not implemented yet
-    case 2: mode = 11; break;
-    case 1: mode = 12; break; */
+    case 8: mode = rgb_pal8; max_colors = 256; p = b->p;       break;
+    case 4: mode = rgb_pal4; max_colors = 16;  p = b->p + 256; break;
+    case 2: mode = rgb_pal2; max_colors = 4;   p = b->p + 256; break;
+    case 1: mode = rgb_pal1; max_colors = 2;   p = b->p + 256; break;
     default: return;
   }
+  if (num_colors > max_colors) num_colors = max_colors;
   /* fmt now refers to the format of the _palette_ */
   if (b->pfmt == -1) b->pfmt = b->fmt;
   b->fmt = mode;
+  /* default grays */
+  if (!pal) {
+    for (i = 0; i < num_colors; i++) {
+      w = (i * 255 + (num_colors - 1) / 2) / (num_colors - 1);
+      v = b->t0[w] + b->t1[w] + b->t2[w];
+      p[i] = ((v >> 31) & 0xfff00000) | ((v >> 22) & 0xfff00) | ((v >> 13) & 0xff);
+    }
+  } else
   /* convert palette */
   switch (b->pfmt) {
     case rgb_bgr:
@@ -1952,7 +1940,7 @@ void rgb2yuy2_palette (void *rgb2yuy2, const uint8_t *pal, int num_colors, int b
         v  = b->t0[*pal++];
         v += b->t1[*pal++];
         v += b->t2[*pal++];
-        b->p[i] = ((v >> 31) & 0xfff00000) | ((v >> 22) & 0xfff00) | ((v >> 13) & 0xff);
+        p[i] = ((v >> 31) & 0xfff00000) | ((v >> 22) & 0xfff00) | ((v >> 13) & 0xff);
       }
     break;
     case rgb_argb:
@@ -1964,7 +1952,7 @@ void rgb2yuy2_palette (void *rgb2yuy2, const uint8_t *pal, int num_colors, int b
         v += b->t1[*pal++];
         v += b->t2[*pal];
         pal += 2;
-        b->p[i] = ((v >> 31) & 0xfff00000) | ((v >> 22) & 0xfff00) | ((v >> 13) & 0xff);
+        p[i] = ((v >> 31) & 0xfff00000) | ((v >> 22) & 0xfff00) | ((v >> 13) & 0xff);
       }
     break;
     default: ;
@@ -1972,7 +1960,50 @@ void rgb2yuy2_palette (void *rgb2yuy2, const uint8_t *pal, int num_colors, int b
   /* black pad rest of palette */
   v = b->t0[0] + b->t1[0] + b->t2[0];
   w = ((v >> 31) & 0xfff00000) | ((v >> 22) & 0xfff00) | ((v >> 13) & 0xff);
-  for (; i < 256; i++) b->p[i] = w;
+  for (; i < max_colors; i++) p[i] = w;
+  /* more optimization ahead? */
+  if (mode != rgb_pal8) {
+    union {uint8_t bytes[4]; uint32_t word;} tmp;
+    switch (mode) {
+      case rgb_pal4:
+        for (i = 0; i < 256; i++) {
+          w = p[i >> 4];
+          tmp.bytes[0] = w;       /* y1 */
+          w &= ~255;
+          w += p[i & 15];
+          tmp.bytes[1] = w >> 24; /* u */
+          tmp.bytes[2] = w;       /* y2 */
+          tmp.bytes[3] = w >> 12; /* v */
+          b->p[i] = tmp.word;
+        }
+      break;
+      case rgb_pal2:
+        for (i = 0; i < 16; i++) {
+          w = p[i >> 2];
+          tmp.bytes[0] = w;       /* y1 */
+          w &= ~255;
+          w += p[i & 3];
+          tmp.bytes[1] = w >> 24; /* u */
+          tmp.bytes[2] = w;       /* y2 */
+          tmp.bytes[3] = w >> 12; /* v */
+          b->p[i] = tmp.word;
+        }
+      break;
+      case rgb_pal1:
+        for (i = 0; i < 4; i++) {
+          w = p[i >> 1];
+          tmp.bytes[0] = w;       /* y1 */
+          w &= ~255;
+          w += p[i & 1];
+          tmp.bytes[1] = w >> 24; /* u */
+          tmp.bytes[2] = w;       /* y2 */
+          tmp.bytes[3] = w >> 12; /* v */
+          b->p[i] = tmp.word;
+        }
+      break;
+      default: ;
+    }
+  }
 }
 
 void rgb2yuy2_slice (void *rgb2yuy2, const uint8_t *in, int ipitch, uint8_t *out, int opitch,
@@ -2072,5 +2103,70 @@ void rgb2yuy2_slice (void *rgb2yuy2, const uint8_t *in, int ipitch, uint8_t *out
         out += opad;
       }
     break;
+    case rgb_pal4:
+      ipad = ipitch - (width / 2);
+      for (y = height; y; y--) {
+        uint32_t *o2 = (uint32_t *)out;
+        for (x = width / 2; x; x--)
+          *o2++ = b->p[*in++];
+        in  += ipad;
+        out += opitch;
+      }
+    break;
+    case rgb_pal2:
+      ipad = ipitch - (width / 4);
+      for (y = height; y; y--) {
+        uint32_t *o2 = (uint32_t *)out;
+        for (x = width / 4; x; x--) {
+          *o2++ = b->p[(*in)   >> 4];
+          *o2++ = b->p[(*in++) & 15];
+        }
+        if (width & 3)
+          *o2++ = b->p[(*in) >> 4];
+        in  += ipad;
+        out += opitch;
+      }
+    break;
+    case rgb_pal1:
+      ipad = ipitch - (width / 8);
+      for (y = height; y; y--) {
+        uint32_t *o2 = (uint32_t *)out;
+        for (x = width / 8; x; x--) {
+          *o2++ = b->p[(*in) >> 6];
+          *o2++ = b->p[((*in) >> 4) & 3];
+          *o2++ = b->p[((*in) >> 2) & 3];
+          *o2++ = b->p[(*in++) & 3];
+        }
+        x = width & 7;
+        if (x) {
+          *o2++ = b->p[(*in) >> 6];
+          if (x > 2) {
+            *o2++ = b->p[((*in) >> 4) & 3];
+            if (x > 4)
+             *o2++ = b->p[((*in) >> 2) & 3];
+          }
+        }
+        in  += ipad;
+        out += opitch;
+      }
+    break;
+    case rgb_rgba_clut:
+    case rgb_bgra_clut:
+      ipad = ipitch - 4 * width;
+      opad = opitch - 4 * width;
+      for (y = height; y; y--) {
+        for (x = width; x; x--) {
+          v  = b->t0[*in++];
+          v += b->t1[*in++];
+          v += b->t2[*in++];
+          *out++  = v >> 54; /* u */
+          *out++  = v >> 33; /* v */
+          *out++  = v >> 13; /* y */
+          *out++  = *in++;   /* a */
+        }
+        in  += ipad;
+        out += opad;
+      }
+    break;
   }
 }
