@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -40,21 +40,31 @@ typedef struct {
 } xine_profiler_t;
 
 static xine_profiler_t profiler[MAX_ID];
+static pthread_mutex_t profiler_lock = PTHREAD_MUTEX_INITIALIZER;
 
 void xine_profiler_init () {
+  pthread_mutex_lock(&profiler_lock);
   memset(profiler, 0, sizeof(profiler));
+  pthread_mutex_unlock(&profiler_lock);
 }
 
 int xine_profiler_allocate_slot (const char *label) {
   int id;
 
+  pthread_mutex_lock(&profiler_lock);
+
   for (id = 0; id < MAX_ID && profiler[id].p_label != NULL; id++)
     ;
 
-  if (id >= MAX_ID)
+  if (id >= MAX_ID) {
+    pthread_mutex_unlock(&profiler_lock);
     return -1;
+  }
 
   profiler[id].p_label = label;
+
+  pthread_mutex_unlock(&profiler_lock);
+
   return id;
 }
 
@@ -66,7 +76,7 @@ static __inline__ uint64_t rdtsc(void)
   __asm__ volatile ("rdtsc\n\t" : "=A" (x));
   return x;
 }
-#elif defined(ARCH_X86_64)
+#elif defined(ARCH_X86_X32) || defined(ARCH_X86_64)
 static __inline__ uint64_t rdtsc(void)
 {
   unsigned long long int a, d;
@@ -78,7 +88,7 @@ static __inline__ uint64_t rdtsc(void)
 void xine_profiler_start_count (int id) {
   if ( id >= MAX_ID || id < 0 ) return;
 
-#if defined(ARCH_X86) || defined(ARCH_X86_64)
+#if defined(ARCH_X86)
   profiler[id].p_start = rdtsc();
 #endif
 }
@@ -86,7 +96,7 @@ void xine_profiler_start_count (int id) {
 void xine_profiler_stop_count (int id) {
   if ( id >= MAX_ID || id < 0 ) return;
 
-#if defined(ARCH_X86) || defined(ARCH_X86_64)
+#if defined(ARCH_X86)
   profiler[id].p_times += rdtsc() - profiler[id].p_start;
 #endif
   profiler[id].p_calls++;
@@ -95,7 +105,7 @@ void xine_profiler_stop_count (int id) {
 void xine_profiler_print_results (void) {
   int i;
 
-#if defined(ARCH_X86) || defined(ARCH_X86_64)
+#if defined(ARCH_X86)
   static uint64_t cpu_speed;	/* cpu cyles/usec */
 
   if (!cpu_speed) {
@@ -122,12 +132,12 @@ void xine_profiler_print_results (void) {
 	  "ID", "name", "cpu cycles", "calls", "cycles/call", "usec/call");
   for (i=0; i<MAX_ID; i++) {
     if (profiler[i].p_label) {
-      printf ("%2d: %-24.24s %12lld %9ld",
+      printf ("%2d: %-24.24s %12" PRIu64 " %9ld",
 	      i, profiler[i].p_label, profiler[i].p_times, profiler[i].p_calls);
       if (profiler[i].p_calls) {
-	  printf(" %12lld", profiler[i].p_times / profiler[i].p_calls);
-#if defined(ARCH_X86) || defined(ARCH_X86_64)
-	  printf(" %9lld", profiler[i].p_times / (cpu_speed * profiler[i].p_calls));
+          printf(" %12" PRIu64, profiler[i].p_times / (uint64_t)profiler[i].p_calls);
+#if defined(ARCH_X86)
+	  printf(" %9" PRIu64, profiler[i].p_times / ((uint64_t)cpu_speed * profiler[i].p_calls));
 #endif
       }
       printf ("\n");
