@@ -36,14 +36,18 @@
 #include "../xine-engine/bswap.h"
 
 #include <errno.h>
+#ifdef HAVE_PWD_H
 #include <pwd.h>
+#endif
 #include <sys/types.h>
 #include <sys/time.h>
 #include <time.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <ctype.h>
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
 
 #if HAVE_EXECINFO_H
 #include <execinfo.h>
@@ -59,6 +63,7 @@
 
 #if defined(WIN32)
 #include <windows.h>
+#include <shlobj.h>
 #endif
 
 #ifndef O_CLOEXEC
@@ -303,17 +308,23 @@ void *xine_xcalloc(size_t nmemb, size_t size) {
 void *xine_memdup (const void *src, size_t length)
 {
   void *dst = malloc (length);
+  if (!dst) {
+    return NULL;
+  }
   return xine_fast_memcpy (dst, src, length);
 }
 
 void *xine_memdup0 (const void *src, size_t length)
 {
-  char *dst = xine_xmalloc (length + 1);
+  char *dst = malloc (length + 1);
+  if (!dst) {
+    return NULL;
+  }
   dst[length] = 0;
   return xine_fast_memcpy (dst, src, length);
 }
 
-#ifdef WIN32
+#ifdef __CYGWIN__
 /*
  * Parse command line with Windows XP syntax and copy the command (argv[0]).
  */
@@ -367,12 +378,12 @@ static size_t xine_strcpy_command(const char *cmdline, char *cmd, size_t maxlen)
 #endif
 
 const char *xine_get_homedir(void) {
-#ifdef WIN32
+#if defined(__CYGWIN__)
+
   static char homedir[1024] = {0, };
   char *s;
-  int len;
 
-  len = xine_strcpy_command(GetCommandLine(), homedir, sizeof(homedir));
+  xine_strcpy_command(GetCommandLine(), homedir, sizeof(homedir));
   s = strdup(homedir);
   GetFullPathName(s, sizeof(homedir), homedir, NULL);
   free(s);
@@ -380,7 +391,25 @@ const char *xine_get_homedir(void) {
     *s = '\0';
 
   return homedir;
-#else
+
+#elif defined (WIN32)
+
+  static char homedir[MAX_PATH] = {0, };
+  wchar_t wdir[MAX_PATH];
+
+  /* Get the "Application Data" folder for the user */
+  if (!homedir[0]) {
+    if (S_OK == SHGetFolderPathW(NULL, CSIDL_APPDATA | CSIDL_FLAG_CREATE,
+                                 NULL, SHGFP_TYPE_CURRENT, wdir)) {
+      WideCharToMultiByte (CP_UTF8, 0, wdir, -1, homedir, MAX_PATH, NULL, NULL);
+    } else {
+      fprintf(stderr, "Can't find user configuration directory !\n");
+    }
+  }
+  return homedir;
+
+#else /* __CYGWIN__ , WIN32 */
+
   struct passwd pwd, *pw = NULL;
   static char homedir[BUFSIZ] = {0,};
 
@@ -408,16 +437,44 @@ const char *xine_get_homedir(void) {
   }
 
   return homedir;
-#endif /* WIN32 */
+#endif /* __CYGWIN__ , WIN32 */
 }
 
 #if defined(WIN32) || defined(__CYGWIN__)
 static void xine_get_rootdir(char *rootdir, size_t maxlen) {
+# if defined(__CYGWIN__)
   char *s;
 
   strncpy(rootdir, xine_get_homedir(), maxlen - 1);
   rootdir[maxlen - 1] = '\0';
   if ((s = strrchr(rootdir, XINE_DIRECTORY_SEPARATOR_CHAR))) *s = '\0';
+# else /* WIN32 */
+  /* use location of libxine.dll */
+  static char marker;
+  HMODULE hModule;
+  wchar_t wpath[MAX_PATH];
+
+  rootdir[0] = 0;
+
+  if (GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS |
+                        GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT,
+                        (LPCTSTR)&marker, &hModule)) {
+
+    DWORD dw = GetModuleFileNameW(hModule, wpath, MAX_PATH);
+    if (dw > 0 && dw < MAX_PATH) {
+      WideCharToMultiByte(CP_UTF8, 0, wpath, -1, rootdir, maxlen, NULL, NULL);
+      /* cut library name from path */
+      char *p = strrchr(rootdir, '\\');
+      if (p) {
+        *p = 0;
+      }
+      lprintf("xine library dir is %s\n", rootdir);
+      return;
+    }
+  }
+
+  fprintf(stderr, "Can't determine libxine.dll install path\n");
+# endif
 }
 
 const char *xine_get_pluginroot(void) {
@@ -463,7 +520,7 @@ const char *xine_get_localedir(void) {
 
   return localedir;
 }
-#endif
+#endif /* _WIN32 || __CYGWIN__ */
 
 char *xine_chomp(char *str) {
   char *pbuf;
@@ -750,7 +807,7 @@ int _x_set_socket_close_on_exec(int s)
 #ifndef WIN32
   return fcntl(s, F_SETFD, FD_CLOEXEC);
 #else
-  return SetHandleInformation((HANDLE)s, HANDLE_FLAG_INHERIT, 0);
+  return SetHandleInformation((HANDLE)(intptr_t)s, HANDLE_FLAG_INHERIT, 0);
 #endif
 }
 
@@ -794,7 +851,7 @@ int xine_socket_cloexec(int domain, int type, int protocol)
 #  define XINE_MEM_ALIGN 32
 #endif
 #define XINE_MEM_ADD (sizeof (size_t) + XINE_MEM_ALIGN)
-#define XINE_MEM_MASK ((unsigned long int)XINE_MEM_ALIGN - 1)
+#define XINE_MEM_MASK ((uintptr_t)XINE_MEM_ALIGN - 1)
 
 void *xine_mallocz_aligned (size_t size) {
   uint8_t *new;
@@ -804,7 +861,7 @@ void *xine_mallocz_aligned (size_t size) {
     return NULL;
   sp = (size_t *)new;
   *sp = size;
-  new = (uint8_t *)(((unsigned long int)new + XINE_MEM_ADD) & ~XINE_MEM_MASK);
+  new = (uint8_t *)(((uintptr_t)new + XINE_MEM_ADD) & ~XINE_MEM_MASK);
   new[-1] = new - (uint8_t *)sp;
   return new;
 }
@@ -817,7 +874,7 @@ void *xine_malloc_aligned (size_t size) {
     return NULL;
   sp = (size_t *)new;
   *sp = size;
-  new = (uint8_t *)(((unsigned long int)new + XINE_MEM_ADD) & ~XINE_MEM_MASK);
+  new = (uint8_t *)(((uintptr_t)new + XINE_MEM_ADD) & ~XINE_MEM_MASK);
   new[-1] = new - (uint8_t *)sp;
   return new;
 }
@@ -843,7 +900,7 @@ void *xine_realloc_aligned (void *ptr, size_t size) {
     return NULL;
   sp = (size_t *)new;
   *sp = size;
-  new = (uint8_t *)(((unsigned long int)new + XINE_MEM_ADD) & ~XINE_MEM_MASK);
+  new = (uint8_t *)(((uintptr_t)new + XINE_MEM_ADD) & ~XINE_MEM_MASK);
   new[-1] = new - (uint8_t *)sp;
   /* realloc () may break the alignment, requiring a slow memmove () afterwards */
   if (old) {
@@ -865,6 +922,7 @@ size_t xine_base64_encode (uint8_t *from, char *to, size_t size) {
   uint8_t *q = (uint8_t *)to;
   int l = size;
   from[size] = 0;
+  from[size+1] = 0;
   while (l > 0) {
     uint32_t v = _X_BE_24 (p);
     p += 3;
@@ -965,9 +1023,9 @@ uint32_t xine_crc32_ieee (uint32_t crc, const uint8_t *data, size_t len) {
     }
   }
   {
-    unsigned long int u;
+    size_t u;
     const uint32_t *d32;
-    u = (~3ul - (unsigned long int)data) & 3;
+    u = (~(uintptr_t)3 - (uintptr_t)data) & 3;
     if (u > len)
       u = len;
     len -= u;
@@ -1032,10 +1090,10 @@ uint32_t xine_crc16_ansi (uint32_t crc, const uint8_t *data, size_t len) {
     }
   }
   {
-    unsigned long int u;
+    size_t u;
     const uint32_t *d32;
     crc &= 0xffff;
-    u = (~3ul - (unsigned long int)data) & 3;
+    u = (~(uintptr_t)3 - (uintptr_t)data) & 3;
     if (u > len)
       u = len;
     len -= u;
