@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2012 the xine project
+ *  Copyright (C) 2002-2017 the xine project
  *
  *  This file is part of xine, a free video player.
  *
@@ -45,6 +45,8 @@
 
 #include "bswap.h"
 
+#include "xine_private.h"
+
 /* private constants*/
 
 /* private global variables */
@@ -108,6 +110,10 @@ struct lexer *lexer_init_r(const char * buf, int size) {
 		    bom_utf8[] = { 0xEF, 0xBB, 0xBF };
   struct lexer * lexer = calloc (1, sizeof (*lexer));
 
+  if (!lexer) {
+    return NULL;
+  }
+
   lexer->lexbuf      = buf;
   lexer->lexbuf_size = size;
 
@@ -135,7 +141,7 @@ struct lexer *lexer_init_r(const char * buf, int size) {
 
 void lexer_finalize_r(struct lexer * lexer)
 {
-  free(lexer->lex_malloc);
+  _x_freep(&lexer->lex_malloc);
   free(lexer);
 }
 
@@ -593,7 +599,7 @@ int lexer_get_token_d_r(struct lexer * lexer, char ** _tok, int * _tok_size, int
 /* for ABI compatibility */
 int lexer_get_token (char *tok, int tok_size)
 {
-  return lexer_get_token_d (&tok, &tok_size, 1);
+  return lexer_get_token_d_r (static_lexer, &tok, &tok_size, 1);
 }
 
 static struct {
