@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2014 the xine project
+ *  Copyright (C) 2002-2017 the xine project
  *
  *  This file is part of xine, a free video player.
  *
@@ -74,15 +74,11 @@ static char * strtoupper(char * str) {
   return str;
 }
 
-static xml_node_t * new_xml_node(void) {
+static xml_node_t *XINE_MALLOC new_xml_node(void) {
   xml_node_t * new_node;
 
-  new_node = (xml_node_t*) malloc(sizeof(xml_node_t));
-  new_node->name  = NULL;
-  new_node->data  = NULL;
-  new_node->props = NULL;
-  new_node->child = NULL;
-  new_node->next  = NULL;
+  new_node = (xml_node_t*) calloc(1, sizeof(xml_node_t));
+
   return new_node;
 }
 
@@ -90,24 +86,22 @@ static const char cdata[] = CDATA_MARKER;
 
 static void free_xml_node(xml_node_t * node) {
   if (node->name != cdata)
-    free (node->name);
-  free (node->data);
+    _x_freep (&node->name);
+  _x_freep (&node->data);
   free(node);
 }
 
 static xml_property_t *XINE_MALLOC new_xml_property(void) {
   xml_property_t * new_property;
 
-  new_property = (xml_property_t*) malloc(sizeof(xml_property_t));
-  new_property->name  = NULL;
-  new_property->value = NULL;
-  new_property->next  = NULL;
+  new_property = (xml_property_t*) calloc(1, sizeof(xml_property_t));
+
   return new_property;
 }
 
 static void free_xml_property(xml_property_t * property) {
-  free (property->name);
-  free (property->value);
+  _x_freep (&property->name);
+  _x_freep (&property->value);
   free(property);
 }
 
@@ -121,8 +115,19 @@ void xml_parser_init(const char * buf, int size, int mode) {
 
 xml_parser_t *xml_parser_init_r(const char * buf, int size, int mode) {
   xml_parser_t *xml_parser = malloc(sizeof(*xml_parser));
+
+  if (!xml_parser) {
+    return NULL;
+  }
+
   xml_parser->lexer = lexer_init_r(buf, size);
+  if (!xml_parser->lexer) {
+    free(xml_parser);
+    return NULL;
+  }
+
   xml_parser->mode = mode;
+
   return xml_parser;
 }
 
@@ -132,13 +137,10 @@ void xml_parser_finalize_r(xml_parser_t *xml_parser) {
 }
 
 static void xml_parser_free_props(xml_property_t *current_property) {
-  if (current_property) {
-    if (!current_property->next) {
-      free_xml_property(current_property);
-    } else {
-      xml_parser_free_props(current_property->next);
-      free_xml_property(current_property);
-    }
+  while (current_property) {
+    xml_property_t *next = current_property->next;
+    free_xml_property(current_property);
+    current_property = next;
   }
 }
 
@@ -677,7 +679,9 @@ static int xml_parser_get_node (xml_parser_t *xml_parser, xml_node_t *current_no
   char *pname_buffer = calloc(1, pname_buffer_size);
   char *nname_buffer = calloc(1, nname_buffer_size);
   char *root_names[MAX_RECURSION + 1];
-  root_names[0] = (char *)"";
+  char  tmp_name[] = "";
+
+  root_names[0] = tmp_name;
 
   res = xml_parser_get_node_internal (xml_parser,
 			     &token_buffer, &token_buffer_size,
@@ -884,13 +888,9 @@ void xml_parser_dump_tree (const xml_node_t *node) {
 #include <sys/stat.h>
 #include <fcntl.h>
 
-void *xine_xmalloc (size_t size)
-{
-  return malloc (size);
-}
-
 int main (int argc, char **argv)
 {
+  xml_parser_t *parser;
   int i, ret = 0;
   for (i = 1; argv[i]; ++i)
   {
@@ -928,8 +928,8 @@ int main (int argc, char **argv)
       continue;
     }
 
-    xml_parser_init (buf, st.st_size, 0);
-    if (!xml_parser_build_tree (&tree))
+    parser = xml_parser_init_r (buf, st.st_size, 0);
+    if (parser && !xml_parser_build_tree_r (parser, &tree))
     {
       puts (argv[i]);
       xml_parser_dump_tree (tree);
@@ -938,6 +938,9 @@ int main (int argc, char **argv)
     else
       printf ("%s: parser failure\n", argv[i]);
 
+    if (parser)
+      xml_parser_finalize_r(parser);
+
     if (close (fd))
     {
       perror (argv[i]);
